/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib;

import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.basislib.menu.RwToolBarManager;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;

/**
 * Schnittstelle zum Zugriff auf die Funktionen des Rahmenwerks (wird als
 * Service zur Verfgung gestellt).
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: Rahmenwerk.java 45400 2013-08-08 12:09:33Z peuker $
 */
public interface Rahmenwerk {

	/**
	 * Definition der Job-Familiy, auf die bei eine Trennung der
	 * Datenverteilerverbindung ein Join ausgefhrt wird.
	 */
	String JOB_FAMILY = "RahmenwerkJobs";

	/**
	 * liefert die Startparameter der Rahmenwerksapplikation.
	 * 
	 * @return die Parameter
	 */
	ArgumentList getStartParameter();

	/**
	 * liefert den Name des angemeldeten Benutzers.
	 * 
	 * @return der Name
	 */
	String getBenutzerName();

	/**
	 * liefert das {@link SystemObject}, das den angemeldeten Nutzer
	 * reprsentiert oder <code>null</code> im Offline-Betrieb.
	 * 
	 * @return das Objekt oder <code>null</code>
	 */
	SystemObject getBenutzer();

	/**
	 * ermittelt, ob die Rahmenwerkapplikation eine Datenverteilerverbindung
	 * etabliert hat.
	 * 
	 * @return den Zustand
	 */
	boolean isOnline();

	/**
	 * liefert die potentiell etablierte Datenverteilerverbindung oder
	 * <code>null</code>.
	 * 
	 * @return due Verbindung oder <code>null</code>
	 */
	ClientDavInterface getDavVerbindung();

	/**
	 * fgt einen Listener hinzu der ber Zustandsnderungen der
	 * Datenverteilerverbindung informiert wird.
	 * 
	 * @param listener
	 *            der Listener
	 */
	void addDavVerbindungsListener(DavVerbindungsListener listener);

	/**
	 * entfernt einen Listener der ber Zustandsnderungen der
	 * Datenverteilerverbindung informiert wird.
	 * 
	 * @param listener
	 *            der Listener
	 */
	void removeDavVerbindungsListener(DavVerbindungsListener listener);

	/**
	 * liefert das bei der Anmeldung eingegebene Passwort.
	 * 
	 * @return das Passwort
	 */
	String getPasswort();

	/**
	 * ermittelt, ob die Berechtigungsklassen (true) oder die
	 * BerechtigungsklassenAlt verwendet werden sollen.
	 * 
	 * @return den Zustand
	 */
	boolean usesBerechtigungenNeu();

	/**
	 * liefert das definierte Oberflchenobjekt (Standard ist die AOE) oder
	 * <code>null</code> im Offline-Modus.
	 * 
	 * @return das Objekt oder <code>null</code>
	 */
	SystemObject getOberflaechenObject();

	/**
	 * liefert den ToolBar-Manager des Rahmenwerks.
	 * 
	 * @return dem Manager
	 */
	RwToolBarManager getRwToolBarManager();
}
