/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.basislib.berechtigung;

import org.eclipse.core.expressions.PropertyTester;

import de.bsvrz.buv.rw.basislib.internal.RahmenwerkService;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Property-Tester, der in Plugins zum berprfen von Rechten eingesetzt werden
 * kann.
 * 
 * Die ID des Testers ist: de.bsvrz.buv.rw.basislib.benutzerrecht.hatRecht
 * 
 * Als Parameter wird die ID der zu prfenden Funktion erwartet.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungTester.java 49030 2014-02-28 15:19:54Z peuker $
 */
public class BerechtigungTester extends PropertyTester {

	private static final Debug LOGGER = Debug.getLogger();

	@Override
	public boolean test(final Object receiver, final String property,
			final Object[] args, final Object expectedValue) {

		boolean result = false;

		if ("hatRecht".equals(property)) {
			try {
				final Berechtigungen berechtigungen = RahmenwerkService
						.getService().getBerechtigungen();

				final FunktionMitBerechtigung funktion = berechtigungen
						.getFunktion(args[0].toString());
				if (funktion != null) {
					result = berechtigungen.hasBerechtigung(funktion);
				}
			} catch (final Exception ex) {
				BerechtigungTester.LOGGER.warning(ex.getLocalizedMessage());
			}
		}

		return result;
	}
}
