/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse fr das Abfragen der Oberflchenberechtigung durch die Plugins.<br>
 * Hinweis: Diese Klasse implementiert das Singleton-Pattern und somit muss die
 * Instanz mittels <code>getInstanz</code> geholt werden.
 * 
 * Realisierung der technischen Anforderungen TBuV 103 bis 105.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/08 08:50:11 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * @deprecated es sollte auf die {@link Berechtigungen} des zurckgegriffen
 *             werden (Service)
 */
@Deprecated
public final class OberflaechenBerechtigung {
	/**
	 * Instanzvariable dieses Singletons.
	 */
	private static OberflaechenBerechtigung instanz = null;

	/**
	 * Liste aller registrierten Funktionen.
	 */
	private final Map<String, OberflaechenFunktion> funktionen = new LinkedHashMap<String, OberflaechenFunktion>();

	/**
	 * Liste der verwendeten SystemObjekte.
	 */
	private final List<SystemObject> systemObjekte = new LinkedList<SystemObject>();

	private static DaVKommunikation dav;

	/**
	 * Liefert die einzigste Instanz dieses Singletons.
	 * 
	 * @return Einzigste Instanz dieses Singletons.
	 */
	public static OberflaechenBerechtigung getInstanz() {
		if (OberflaechenBerechtigung.instanz == null) {
			OberflaechenBerechtigung.instanz = new OberflaechenBerechtigung();
		}
		return OberflaechenBerechtigung.instanz;
	}

	/**
	 * Privater Konstruktor.
	 * 
	 */
	private OberflaechenBerechtigung() {
		OberflaechenBerechtigung.dav = DaVKommunikation.getInstanz();
	}

	/**
	 * Fgt eine {@link OberflaechenFunktion} zur Liste der
	 * OberflaechenFunktionen hinzu.
	 * 
	 * @param funktion
	 *            {@link OberflaechenFunktion}, die hinzugefgt werden soll.
	 */
	public void hinzufuegenOberflaechenFunktion(
			final OberflaechenFunktion funktion) {
		funktionen.put(funktion.getFunktionsName(), funktion);

		final FunktionMitBerechtigung funktionMitBerechtigung = new FunktionMitBerechtigung(
				null, null, funktion.getFunktionsName(),
				funktion.getFunktionsAlias(), null);
		OberflaechenBerechtigung.getBerechtigungen().addOberflaechenFunktion(
				funktionMitBerechtigung);
	}

	/**
	 * Entfernt eine {@link OberflaechenFunktion} aus der Liste der
	 * OberflaechenFunktionen.
	 * 
	 * @param funktion
	 *            {@link OberflaechenFunktion}, die entfernt werden soll.
	 */
	public void entferneOberflaechenFunktion(final OberflaechenFunktion funktion) {
		funktionen.remove(funktion.getFunktionsName());
		OberflaechenBerechtigung.getBerechtigungen()
				.removeOberflaechenFunktion(funktion.getFunktionsName());
	}

	/**
	 * Prft fr alle Elemente, die sich unter ID einer Gruppe, die z.B. einen
	 * Dialog bezeichnet, registriert haben, ob die registrierten Berechtigungen
	 * fr die bergebenen Systemobjekte bestehen.
	 * 
	 * @param objekte
	 *            Systemobjekte, fr die die Berechtigung geprft werden soll.
	 * @param dialogId
	 *            Id des Dialogs, desse Elementberechtigung fr die bergebenen
	 *            Systemobjekte geprft werden soll.
	 */
	public void pruefeBerechtigungElementGruppe(
			final List<SystemObject> objekte, final String dialogId) {
		updateListeners();
	}

	/**
	 * Prft fr ein registriertes Element, ob fr die bergebenen Systemobjekte
	 * die Berechtigung besteht.
	 * 
	 * @param objekte
	 *            Systemobjekte, fr die die Berechtigung geprft werden soll.
	 * @param elementId
	 *            Id des Elements, dessen Berechtigung fr die bergebenen
	 *            Systemobjekte geprft werden soll.
	 */
	public void pruefeBerechtigungElement(final List<SystemObject> objekte,
			final String elementId) {
		updateListeners();
	}

	/**
	 * Dieser Event wird dann abgefeuert, wenn sich etwas am
	 * Berechtigungskonzept gendert hat.
	 * 
	 */
	private void feureEreignis(final OberflaechenBerechtigungEreignis ereignis) {
		OberflaechenBerechtigung.dav
				.benachrichtigeOberflaechenBenutzerZuhoerer(ereignis);
	}

	public List<OberflaechenFunktion> getFunktionen() {
		return new ArrayList<OberflaechenFunktion>(funktionen.values());
	}

	public List<SystemObject> getSystemObjekte() {
		return Collections.emptyList();
	}

	public void setSystemObjekte(final List<SystemObject> systemObjekte) {
		this.systemObjekte.addAll(systemObjekte);
	}

	public static DaVKommunikation getDaV() {
		return OberflaechenBerechtigung.dav;
	}

	public static Berechtigungen getBerechtigungen() {
		return RahmenwerkService.getService().getBerechtigungen();
	}

	public static void setDaV(final DaVKommunikation dav) {
		OberflaechenBerechtigung.dav = dav;
	}

	public OberflaechenFunktion getFunktion(final String id) {
		return funktionen.get(id);
	}

	private void updateListeners() {

		synchronized (funktionen) {
			for (final SystemObject klasse : OberflaechenBerechtigung
					.getBerechtigungen().getBerechtigungsKlassen(
							RahmenwerkService.getService().getRahmenWerk()
									.getBenutzer())) {

				final List<OberflaechenFunktion> freigegebeneFkt = new ArrayList<OberflaechenFunktion>();
				final List<OberflaechenFunktion> gesperrteFkt = new ArrayList<OberflaechenFunktion>();

				for (final OberflaechenFunktion fkt : funktionen.values()) {
					if (OberflaechenBerechtigung.getDaV().hatBerechtigung(null,
							klasse.getPid(), fkt)) {
						freigegebeneFkt.add(fkt);
					} else {
						gesperrteFkt.add(fkt);
					}
				}

				if (!freigegebeneFkt.isEmpty()) {
					final OberflaechenBerechtigungEreignis ereignis = new OberflaechenBerechtigungEreignis(
							this, true, new ArrayList<SystemObject>(),
							new ArrayList<SystemObject>(), freigegebeneFkt,
							new ArrayList<OberflaechenFunktion>());
					feureEreignis(ereignis);
				}

				if (!gesperrteFkt.isEmpty()) {
					final OberflaechenBerechtigungEreignis ereignis = new OberflaechenBerechtigungEreignis(
							this, false, new ArrayList<SystemObject>(),
							new ArrayList<SystemObject>(),
							new ArrayList<OberflaechenFunktion>(), gesperrteFkt);
					feureEreignis(ereignis);
				}
			}
		}
	}
}
