/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.einstellungen;

import java.io.Serializable;

/**
 * Adresse zur Lokalisierung einer Einstellung.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: EinstellungsAdresse.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class EinstellungsAdresse implements Serializable {

	/** ID fr Serialisierung. */
	private static final long serialVersionUID = 1L;
	private final String typ;
	private final String id;
	private final String pid;
	private final EinstellungOwnerType ownerType;
	private final EinstellungLocation location;

	/**
	 * Konstruktor.
	 * 
	 * @param typ
	 *            der Typ des Einstellungsobjekts oder <code>null</code> fr den
	 *            Standardtyp {@link String}
	 * @param id
	 *            die ID der Einstellung
	 * @param key
	 *            der Key fr die Definition des korrekten Einstellungsspeichers
	 */
	public EinstellungsAdresse(final String typ, final String id,
			final SpeicherKey key) {
		this(typ, id, key.getOwnerType(), key.getPid(), key.getLocation());
	}

	/**
	 * Konstruktor.
	 * 
	 * @param typ
	 *            der Typ des Einstellungsobjekts oder <code>null</code> fr den
	 *            Standardtyp {@link String}
	 * @param id
	 *            die ID der Einstellung
	 * @param ownerType
	 *            der Besitzertyp
	 * @param pid
	 *            die PID oder <code>null</code>
	 * @param location
	 *            der logische Speicherort
	 */
	public EinstellungsAdresse(final String typ, final String id,
			final EinstellungOwnerType ownerType, final String pid,
			final EinstellungLocation location) {
		super();
		if (typ == null) {
			this.typ = String.class.getName();
		} else {
			this.typ = typ;
		}
		this.id = id;
		this.ownerType = ownerType;
		this.pid = pid;
		this.location = location;
	}

	/**
	 * liefert den Name des Typs des Einstellungsobjekts.
	 * 
	 * @return den Name
	 */
	public String getTyp() {
		return typ;
	}

	/**
	 * liefert die ID der Einstellung.
	 * 
	 * @return die ID
	 */
	public String getId() {
		return id;
	}

	/**
	 * liefert den Besitzertyp.
	 * 
	 * @return den Typ
	 */
	public EinstellungOwnerType getOwnerType() {
		return ownerType;
	}

	/**
	 * liefert den logischen Speicherort.
	 * 
	 * @return den Ort
	 */
	public EinstellungLocation getLocation() {
		return location;
	}

	/**
	 * liefert die PID des Besitzers oder <code>null</code> fr allgemeine
	 * Einstellungen.
	 * 
	 * @return die PID oder <code>null</code>
	 */
	public String getPid() {
		return pid;
	}
}
