/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

/**
 * Schnittstelle zur Beschreibung eines Zeitbereiches.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: IEintragBereich.java 45393 2013-08-07 21:32:57Z peuker $
 * 
 */
public interface IEintragBereich {

	/**
	 * Liefert den Von-Zeitpunkt dieses Bereiches.
	 * 
	 * @return Von-Zeitpunkt
	 */
	Long getVon();

	/**
	 * Liefert den Bis-Zeitpunkt dieses Bereiches.
	 * 
	 * @return Bis-Zeitpunkt
	 */
	Long getBis();

	/**
	 * liefert den Anfang des lesbaren Zeitbereiches als Zeichenkette.
	 * 
	 * @return den Anfangszeitpunkt
	 */
	String getVonLesbar();

	/**
	 * liefert das Ende des lesbaren Zeitbereiches als Zeichenkette.
	 * 
	 * @return den Endzeitpunkt
	 */
	String getBisLesbar();

	/**
	 * Prft ob dieser Bereich atomar ist, d.h. Von- und Bis-Zeitpunkt sind
	 * gleich.
	 * 
	 * @return Liefert <code>true</code>, wenn dieser Bereich atomar ist, sonst
	 *         <code>false</code>.
	 */
	boolean istAtomar();

	@Override
	String toString();
}
