/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Eingabedialog fr Zeitfilter. <br>
 * 
 * In der Klasse KalenderZeitFilter werden Eingabefelder fr Zeitfilter zur
 * Verfgung gestellt.<br>
 * Eingabefelder: Zeitfilter Monatstag (1-31)<br>
 * Zeitfilter Monat (Januar,...)<br>
 * Zeitfilter Wochentag (Montag,...)<br>
 * Zeitfilter Zeitspanne(Zeitgruppen (Anfangs- und Endzeitpunkt)<br>
 * Bei den Zeitgruppen gibt es neben der Tastatureingabe noch die Mglichkeit
 * der Eingabe ber eine Auswahlbox und die Mglichkeit weitere Zeitgruppen
 * hinzuzufgen. <br>
 * Die Klasse erfllt folgende Technische Anforderungen des Segments Bedienung
 * und Visualisierung (BUV)<br>
 * - TBuV-43 und 46
 * 
 * 
 * @author Dambach-Werke GmbH
 * @author HofmannJ
 * @version 1.0
 */
public class KalenderZeitFilterDialog extends Dialog implements
		SelectionListener {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Shell, in der dieser Dialog dargestellt wird.
	 */
	private Shell shell;

	private Group gruppeMonat;

	private Group gruppeTag;

	private Group gruppeWochentag;

	private Group gruppeZeitgruppen;

	private ScrolledComposite verschiebbaresCompositeZeitgruppen;

	private Composite compositeZeitGruppenInhalt;

	private Composite compositeUnten;

	private Button[] monatCheckboxButtons;

	private Button[] tagCheckboxButtons;

	private Button[] wochentagCheckboxButtons;

	/**
	 * Datenstruktur, die in diesem Dialog angezeigt/manipuliert werden kann.
	 */
	private KalenderZeitFilterDatenStruktur datenStruktur;

	/**
	 * Liste der Zeitgruppenzeilen, wird fr die Verwaltung der Darstellung der
	 * Zeitgruppenzeilen bentigt.
	 */
	private List<ZeitGruppenZeile> listeZeitGruppenZeilen;

	/**
	 * Flag, signalisiert ob eine Initialisierung der Datenstrukturen
	 * durchgefhrt wurde oder nicht. Ist keine Initialisierung durchgefhrt
	 * worden, darf sich der Dialog nicht ffnen.
	 */
	private boolean initialisierungDurchgefuehrt;

	/**
	 * Konstruktor fr Dialoga. Shell & Style mssen angegben werden.
	 * 
	 * @param parent
	 *            Parent-Shell
	 * @param style
	 *            Style
	 */
	public KalenderZeitFilterDialog(final Shell parent, final int style) {
		super(parent, style);
	}

	/**
	 * ffnet den Dialog und gibt die erstellte/manipulierte
	 * KalenderZeitFilterDatenStruktur zurck.
	 * 
	 * @return Erstellte/Manipulierte KalenderZeitFilterDatenStruktur
	 */
	public KalenderZeitFilterDatenStruktur oeffne(
			final KalenderZeitFilterDatenStruktur daten) {
		if (daten != null) {
			init(daten);
		}

		/*
		 * Initialisieren mit leerer Datenstruktur.
		 */
		if (initialisierungDurchgefuehrt) {
			init();
		}

		final Shell parent = getParent();
		// _shell = new Shell(parent, SWT.TITLE | SWT.BORDER
		// | SWT.APPLICATION_MODAL);
		shell = new Shell(parent, SWT.TITLE | SWT.BORDER | SWT.RESIZE);
		shell.setText("Zeitfilter");

		erzeugeGui();
		leseDatenstruktur();

		shell.pack();
		shell.open();
		shell.setSize(720, 500);

		final Display display = parent.getDisplay();

		// Dialog zentrieren
		final Monitor primary = display.getPrimaryMonitor();
		final Rectangle bounds = primary.getBounds();
		final Rectangle rect = shell.getBounds();
		final int x = bounds.x + ((bounds.width - rect.width) / 2);
		final int y = bounds.y + ((bounds.height - rect.height) / 2);
		shell.setLocation(x, y);

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		if (datenStruktur != null) {
			System.out.println(datenStruktur.erzeugeFilterString());
		} else {
			System.out.println(">>null<<");
		}

		return datenStruktur;
	}

	/**
	 * Fhrt eine Initialisierung ohne KalenderZeitFilterDatenStruktur durch.
	 * Diese Datenstruktur wird frisch erstellt.
	 * 
	 */
	private void init() {
		datenStruktur = new KalenderZeitFilterDatenStruktur();
		listeZeitGruppenZeilen = new LinkedList<ZeitGruppenZeile>();
		initialisierungDurchgefuehrt = true;
	}

	/**
	 * Fhrt eine Initialisierung mit einer vorgegebenen
	 * KalenderZeitFilterDatenStruktur durch.
	 * 
	 * @param daten
	 *            Vorgegebene KalenderZeitFilterDatenStruktur
	 */
	private void init(final KalenderZeitFilterDatenStruktur daten) {
		datenStruktur = daten;
		listeZeitGruppenZeilen = new LinkedList<ZeitGruppenZeile>();
		initialisierungDurchgefuehrt = true;
	}

	/**
	 * Erzeugt die Benutzerschnittstelle.
	 * 
	 */
	private void erzeugeGui() {

		// Gruppe fr die Monate
		gruppeMonat = new Group(shell, SWT.NONE);
		gruppeMonat.setText("Monat");

		// Gruppe fr die Tage
		gruppeTag = new Group(shell, SWT.NONE);
		gruppeTag.setText("Tag");

		// Gruppe fr die Wochentage
		gruppeWochentag = new Group(shell, SWT.NONE);
		gruppeWochentag.setText("Wochentag");

		// Gruppe fr die Zeitgruppen
		gruppeZeitgruppen = new Group(shell, SWT.NONE);
		gruppeZeitgruppen.setText("Zeitgruppen");

		// ScrolledComposite fr die Zeitgruppen
		verschiebbaresCompositeZeitgruppen = new ScrolledComposite(
				gruppeZeitgruppen, SWT.H_SCROLL | SWT.V_SCROLL);

		verschiebbaresCompositeZeitgruppen.getVerticalBar().setIncrement(20);

		compositeZeitGruppenInhalt = new Composite(
				verschiebbaresCompositeZeitgruppen, SWT.NONE);
		compositeZeitGruppenInhalt.setLayout(new FillLayout());

		compositeUnten = new Composite(shell, SWT.NONE);

		fuelleGruppeMonatTagWochentTag();
		fuelleGruppeTag();
		fuelleGruppeWochenTag();
		fuelleCompositeUnten();

		// Layout groupMonat
		final GridLayout glGroupMonat = new GridLayout();
		glGroupMonat.numColumns = 12;
		glGroupMonat.makeColumnsEqualWidth = true;
		gruppeMonat.setLayout(glGroupMonat);

		// Layout groupTag
		final GridLayout glGroupTag = new GridLayout();
		glGroupTag.numColumns = 16;
		glGroupTag.makeColumnsEqualWidth = true;
		gruppeTag.setLayout(glGroupTag);

		// Layout groupWochenTag
		final GridLayout glGroupWochentag = new GridLayout();
		glGroupWochentag.numColumns = 7;
		glGroupWochentag.makeColumnsEqualWidth = true;
		gruppeWochentag.setLayout(glGroupWochentag);

		// Layout groupZeitgruppen

		gruppeZeitgruppen.setLayout(new FillLayout());

		// Layout compositeZeitgruppen
		final GridLayout glGroupZeitgruppen = new GridLayout();
		glGroupZeitgruppen.numColumns = 1;
		glGroupZeitgruppen.makeColumnsEqualWidth = false;

		verschiebbaresCompositeZeitgruppen
				.setContent(compositeZeitGruppenInhalt);
		verschiebbaresCompositeZeitgruppen.setMinSize(600, 4000);
		verschiebbaresCompositeZeitgruppen.setExpandHorizontal(true);
		verschiebbaresCompositeZeitgruppen.setExpandVertical(true);

		compositeZeitGruppenInhalt.setLayout(glGroupZeitgruppen);

		// GridDate CompositeUnten
		final GridData gridDataCompositeUnten = new GridData();
		gridDataCompositeUnten.horizontalAlignment = GridData.END;
		gridDataCompositeUnten.grabExcessHorizontalSpace = true;
		gridDataCompositeUnten.verticalAlignment = GridData.CENTER;

		// Layout compositeUnten
		final GridLayout glCompositeUnten = new GridLayout();
		glCompositeUnten.numColumns = 3;
		glCompositeUnten.makeColumnsEqualWidth = true;

		compositeUnten.setLayout(glCompositeUnten);
		compositeUnten.setLayoutData(gridDataCompositeUnten);

		final GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = false;
		gridData.verticalAlignment = GridData.BEGINNING;
		gridData.horizontalAlignment = GridData.FILL;

		final GridData gridDataMehrPlatz = new GridData();
		gridDataMehrPlatz.grabExcessHorizontalSpace = true;
		gridDataMehrPlatz.grabExcessVerticalSpace = true;
		gridDataMehrPlatz.verticalAlignment = GridData.FILL;
		gridDataMehrPlatz.horizontalAlignment = GridData.FILL;

		gruppeMonat.setLayoutData(gridData);
		gruppeTag.setLayoutData(gridDataMehrPlatz);
		gruppeWochentag.setLayoutData(gridData);
		gruppeZeitgruppen.setLayoutData(gridDataMehrPlatz);

		// Zentrierte Elemente
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.CENTER;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;

		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		shell.setLayout(gridLayout);

	}

	/**
	 * Fllt die Gruppe Monate.
	 * 
	 */
	private void fuelleGruppeMonatTagWochentTag() {
		final Label label1 = new Label(gruppeMonat, SWT.NONE);
		label1.setText("Januar");
		final Label label2 = new Label(gruppeMonat, SWT.NONE);
		label2.setText("Februar");
		final Label label3 = new Label(gruppeMonat, SWT.NONE);
		label3.setText("Mrz");
		final Label label4 = new Label(gruppeMonat, SWT.NONE);
		label4.setText("April");
		final Label label5 = new Label(gruppeMonat, SWT.NONE);
		label5.setText("Mai");
		final Label label6 = new Label(gruppeMonat, SWT.NONE);
		label6.setText("Juni");
		final Label label7 = new Label(gruppeMonat, SWT.NONE);
		label7.setText("Juli");
		final Label label8 = new Label(gruppeMonat, SWT.NONE);
		label8.setText("August");
		final Label label9 = new Label(gruppeMonat, SWT.NONE);
		label9.setText("September");
		final Label label10 = new Label(gruppeMonat, SWT.NONE);
		label10.setText("Oktober");
		final Label label11 = new Label(gruppeMonat, SWT.NONE);
		label11.setText("November");
		final Label label12 = new Label(gruppeMonat, SWT.NONE);
		label12.setText("Dezember");

		monatCheckboxButtons = new Button[12];

		for (int i = 0; i < monatCheckboxButtons.length; i++) {
			final Button button = new Button(gruppeMonat, SWT.CHECK);
			monatCheckboxButtons[i] = button;
			button.addSelectionListener(this);
		}

	}

	/**
	 * Fllt die Gruppe Tag.
	 * 
	 */
	private void fuelleGruppeTag() {
		tagCheckboxButtons = new Button[31];
		for (int i = 0; i < tagCheckboxButtons.length; i++) {
			final Label tagLabel = new Label(gruppeTag, SWT.NONE);
			tagLabel.setText((i + 1) + ": ");
			final Button button = new Button(gruppeTag, SWT.CHECK);
			tagCheckboxButtons[i] = button;
			button.addSelectionListener(this);
		}
	}

	/**
	 * Fllt die Gruppe Wochentag.
	 * 
	 */
	private void fuelleGruppeWochenTag() {
		wochentagCheckboxButtons = new Button[7];
		final String[] wochentagsNamen = { "Monatag", "Dienstag", "Mittwoch",
				"Donnerstag", "Freitag", "Samstag", "Sonntag" };
		for (final String tagName : wochentagsNamen) {
			final Label tagNameLabel = new Label(gruppeWochentag, SWT.NONE);
			tagNameLabel.setText(tagName);
		}
		for (int i = 0; i < wochentagCheckboxButtons.length; i++) {
			final Button button = new Button(gruppeWochentag, SWT.CHECK);
			wochentagCheckboxButtons[i] = button;
			button.addSelectionListener(this);
		}
	}

	/**
	 * Fllt das CompositeUnten.
	 * 
	 */
	private void fuelleCompositeUnten() {
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.CENTER;

		final Button buttonOk = new Button(compositeUnten, SWT.PUSH);
		buttonOk.setText("OK");
		buttonOk.setLayoutData(gridData);
		buttonOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				KalenderZeitFilterDialog.LOGGER
						.fine("Button OK wurde gedrckt.");

				shell.dispose();
			}
		});

		final Button buttonAbbrechen = new Button(compositeUnten, SWT.PUSH);
		buttonAbbrechen.setText("Abbrechen");
		buttonAbbrechen.setLayoutData(gridData);
		buttonAbbrechen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				KalenderZeitFilterDialog.LOGGER
						.fine("Button Abbrechen wurde gedrckt!");

				shell.dispose();
			}
		});

		final Button buttonRuecksetzen = new Button(compositeUnten, SWT.PUSH);
		buttonRuecksetzen.setText("Rcksetzen");
		buttonRuecksetzen.setLayoutData(gridData);
		buttonRuecksetzen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				KalenderZeitFilterDialog.LOGGER
						.fine("Button Rcksetzen wurde gedrckt!");

				final Label testLabel = new Label(compositeZeitGruppenInhalt,
						SWT.BORDER);
				testLabel.setText("Nachweis");
			}
		});

	}

	/**
	 * Findet die Position eines Controls im <code>Monate</code> Array.
	 * 
	 * @param monKontrolle
	 *            Das Widget, dessen zugehriges Monat gefunden werden soll
	 * @return Index des gefundenen Elements
	 */
	private int findeMonatCheckbox(final Button bu) {
		for (int i = 0; i < monatCheckboxButtons.length; i++) {
			if (monatCheckboxButtons[i] == bu) {
				return i;
			}
		}
		return -1;
	}

	private int findeTagCheckbox(final Button bu) {
		for (int i = 0; i < tagCheckboxButtons.length; i++) {
			if (tagCheckboxButtons[i] == bu) {
				return i;
			}
		}
		return -1;
	}

	private int findeWochentagCheckbox(final Button bu) {
		for (int i = 0; i < wochentagCheckboxButtons.length; i++) {
			if (wochentagCheckboxButtons[i] == bu) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Liest die Datenstruktur ein.
	 * 
	 */
	private void leseDatenstruktur() {
		// Monate setzen
		final int[] selektierteMonate = datenStruktur.getSelektierteMonate();
		for (final int i : selektierteMonate) {
			monatCheckboxButtons[i].setSelection(true);
		}

		// Tage setzen
		final int[] selektierteTage = datenStruktur.getSelektierteTage();
		for (final int i : selektierteTage) {
			tagCheckboxButtons[i].setSelection(true);
		}

		// Wochentage setzen
		final int[] selektierteWochenTage = datenStruktur
				.getSelektierteWochentage();
		for (final int i : selektierteWochenTage) {
			wochentagCheckboxButtons[i].setSelection(true);
		}

		// Zeitgruppen setzen
		final List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitGruppen = datenStruktur
				.getListeZeitGruppen();
		// Wenn die ZeitGruppenListe nicht leer ist, dann auslesen und
		// darstellen
		if (zeitGruppen.size() > 0) {
			final Iterator<KalenderZeitFilterDatenStrukturZeitGruppe> it = zeitGruppen
					.iterator();
			while (it.hasNext()) {
				final KalenderZeitFilterDatenStrukturZeitGruppe zg = it.next();
				generiereZeitgruppeMitVorgabe(zg);
			}
		} else {
			// Sonst nur eine Leerzeile darstellen.
			generiereZeitgruppeLeer();
		}

	}

	/**
	 * Generiert eine leere Zeitgruppe.
	 * 
	 */
	private void generiereZeitgruppeLeer() {
		final GregorianCalendar anfang = new GregorianCalendar(1970, 1, 1, 0,
				0, 0);
		final long nu = anfang.getTimeInMillis();

		final KalenderZeitFilterDatenStrukturZeitGruppe kz = new KalenderZeitFilterDatenStrukturZeitGruppe(
				nu, nu);
		datenStruktur.fuegeZeitGruppeHinzu(kz);
		final ZeitGruppenZeile zgz = new ZeitGruppenZeile(
				compositeZeitGruppenInhalt, kz);
		zgz.generiereComposite();

		listeZeitGruppenZeilen.add(zgz);
		compositeZeitGruppenInhalt.layout();
	}

	/**
	 * Generiert eine Zeitgruppe mit Zeitvorgabe in Form einer Instanz der
	 * Klasse KalenderZeitFilterDatensStrukturZeitGruppe.
	 * 
	 * @param zg
	 *            Vorgabe von Start- und Endzeitpunkt
	 */
	private void generiereZeitgruppeMitVorgabe(
			final KalenderZeitFilterDatenStrukturZeitGruppe zg) {
		datenStruktur.fuegeZeitGruppeHinzu(zg);
		final ZeitGruppenZeile zgz = new ZeitGruppenZeile(
				compositeZeitGruppenInhalt, zg);
		zgz.generiereComposite();
		listeZeitGruppenZeilen.add(zgz);
		compositeZeitGruppenInhalt.layout();
	}

	private void loescheZeitGruppenZeile(final ZeitGruppenZeile zgz) {
		if (listeZeitGruppenZeilen.contains(zgz)) {
			listeZeitGruppenZeilen.remove(zgz);
			zgz.dispose();
			compositeZeitGruppenInhalt.layout();
		}
	}

	private void einfuegenZeitGruppenZeileNachPosition(final ZeitGruppenZeile zg) {
		generiereZeitgruppeLeer();

		final LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe> dsAlt = new LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe>();

		final int idx = listeZeitGruppenZeilen.indexOf(zg) + 1;

		if (idx < listeZeitGruppenZeilen.size()) {
			if (idx > -1) {
				for (int i = idx; i < (listeZeitGruppenZeilen.size() - 1); i++) {
					dsAlt.add(listeZeitGruppenZeilen.get(i)
							.getDatenStrukturZeitGruppe());
				}
			}

			int i = idx + 1;
			final Iterator<KalenderZeitFilterDatenStrukturZeitGruppe> it = dsAlt
					.iterator();
			while (it.hasNext()) {
				final KalenderZeitFilterDatenStrukturZeitGruppe datenStrukturZeitGruppe = it
						.next();
				listeZeitGruppenZeilen.get(i).setDatenStrukturZeitGruppe(
						datenStrukturZeitGruppe);
				i++;
			}

			/*
			 * Neue Zeile mit Nullstellung erzeugen
			 */
			final GregorianCalendar gcNullStellung = new GregorianCalendar();
			gcNullStellung.set(Calendar.HOUR_OF_DAY, 0);
			gcNullStellung.set(Calendar.MINUTE, 0);
			gcNullStellung.set(Calendar.SECOND, 0);
			gcNullStellung.set(Calendar.MILLISECOND, 0);

			listeZeitGruppenZeilen.get(idx).setDatenStrukturZeitGruppe(
					new KalenderZeitFilterDatenStrukturZeitGruppe(
							gcNullStellung.getTimeInMillis(), gcNullStellung
									.getTimeInMillis()));
		}

	}

	/**
	 * Bildet die Struktur einer ZeitgruppenZeile.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ZeitGruppenZeile {
		private final Composite parent;

		private KalenderZeitFilterDatenStrukturZeitGruppe daten;

		private Label labelVon;

		private Label labelBis;

		private Label labelInfo;

		private Button schaltflaecheHinzufuegen;

		private Button schaltflaecheLoeschen;

		/**
		 * Diese Composite enthlt die Darstellung eines Zeitgruppen-Eintrags.
		 */
		private Composite returnComposite;

		private DatumZeit dzVon;

		private DatumZeit dzBis;

		/**
		 * Konstruktor.
		 * 
		 * @param parent
		 *            Eltern-Composite, in dass die Zeitgruppenzeile
		 *            eingeblendet werden soll.
		 * @param Daten
		 *            Von-Bis.
		 */
		public ZeitGruppenZeile(final Composite parent,
				final KalenderZeitFilterDatenStrukturZeitGruppe neueDaten) {
			this.parent = parent;
			this.daten = neueDaten;
		}

		/**
		 * Liefert die {@link KalenderZeitFilterDatenStrukturZeitGruppe} dieser
		 * {@link ZeitGruppenZeile}.
		 * 
		 * @return {@link KalenderZeitFilterDatenStrukturZeitGruppe}
		 */
		public KalenderZeitFilterDatenStrukturZeitGruppe getDatenStrukturZeitGruppe() {
			return this.daten;
		}

		/**
		 * Setzt die {@link KalenderZeitFilterDatenStrukturZeitGruppe} dieser
		 * {@link ZeitGruppenZeile}.
		 * 
		 * @param datenStrukturZeitGruppe
		 *            {@link KalenderZeitFilterDatenStrukturZeitGruppe}
		 */
		public void setDatenStrukturZeitGruppe(
				final KalenderZeitFilterDatenStrukturZeitGruppe datenStrukturZeitGruppe) {
			this.daten = datenStrukturZeitGruppe;
			dzVon.setDatum(new Date(daten.getStartzeitpunkt()));
			dzBis.setDatum(new Date(daten.getEndzeitpunkt()));
		}

		/**
		 * Liefert true zurck, wenn der Startzeitpunkt VOR dem Endzeitpunkt
		 * liegt. Andernfalls wird false geliefert.
		 * 
		 * @return true, wenn der Startzeitpunkt VOR dem Endzeitpunkt liegt.
		 */
		private boolean pruefePlausibilitaet() {
			boolean returnWert = true;
			if (daten.getStartzeitpunkt() > daten.getEndzeitpunkt()) {
				returnWert = false;
			}

			return returnWert;
		}

		/**
		 * Dispose-Methode fr das {@link Composite} dieser
		 * {@link ZeitGruppenZeile}.
		 */
		public void dispose() {
			returnComposite.dispose();
		}

		/**
		 * Lscht diese {@link ZeitGruppenZeile}.
		 */
		private void aktionZeitgruppenzeileLoeschen() {
			loescheZeitGruppenZeile(this);
		}

		/**
		 * Hngt eine Zeitgruppenzeile an.
		 */
		private void aktionZeitgruppenzeileAnhaengen() {
			// generiereZeitgruppeLeer();
			einfuegenZeitGruppenZeileNachPosition(this);
		}

		/**
		 * Generiert das Composite.
		 * 
		 * @return Ein Composite
		 */
		public Composite generiereComposite() {
			returnComposite = new Composite(parent, SWT.BORDER);

			final RowLayout rl = new RowLayout(SWT.HORIZONTAL);
			rl.pack = false;
			returnComposite.setLayout(rl);

			labelVon = new Label(returnComposite, SWT.NONE);
			labelVon.setText("von: ");

			// _kzVon = new KalenderZeit(_returnComposite, SWT.NONE, true, true,
			// true);
			// _kzVon.setZeit(_daten.get_startzeitpunkt());

			dzVon = new DatumZeit(returnComposite, SWT.NONE,
					DatumZeit.Eingabetyp.uhrMS, false, false);
			// _dzVon = new DatumZeit(_returnComposite, SWT.NONE,
			// DatumZeit.Eingabetyp.uhrEinfachMS, false, false);
			dzVon.setDatum(new Date(daten.getStartzeitpunkt()));
			dzVon.setLayout(new GridLayout());

			// CDateTime _cdt = new CDateTime(_returnComposite, CDT.DROP_DOWN |
			// CDT.DATE_MEDIUM | CDT.TIME_MEDIUM |
			// CDT.CLOCK_24_HOUR);

			labelBis = new Label(returnComposite, SWT.NONE);
			labelBis.setText("bis: ");

			// _kzBis = new KalenderZeit(_returnComposite, SWT.NONE, true, true,
			// true);
			// _kzBis.setZeit(_daten.get_endzeitpunkt());

			dzBis = new DatumZeit(returnComposite, SWT.NONE,
					DatumZeit.Eingabetyp.uhrMS, false, false);
			// _dzBis = new DatumZeit(_returnComposite, SWT.NONE,
			// DatumZeit.Eingabetyp.uhrEinfachMS, false, false);

			dzBis.setDatum(new Date(daten.getStartzeitpunkt()));
			dzBis.setLayout(new GridLayout());

			labelInfo = new Label(returnComposite, SWT.NONE);

			dzVon.hinzufuegenSelektionsZuhoerer(new SelectionListener() {

				private long startzeitpunkt = -1;

				@Override
				public void widgetDefaultSelected(final SelectionEvent e) {
					gemeinsam(e);
				}

				@Override
				public void widgetSelected(final SelectionEvent e) {
					gemeinsam(e);
				}

				private void gemeinsam(final SelectionEvent e) {
					if (e.data instanceof Date) {
						final Date datum = (Date) e.data;
						startzeitpunkt = datum.getTime();
						daten.setStartzeitpunkt(startzeitpunkt);

						if (!pruefePlausibilitaet()) {
							final Color color = new Color(Display.getDefault(),
									255, 0, 0);
							labelInfo.setForeground(color);
							labelInfo
									.setText("Startzeitpunkt liegt vor dem Endzeitpunkt!");
							compositeZeitGruppenInhalt.layout();
						} else {
							labelInfo.setText("");
							compositeZeitGruppenInhalt.layout();
						}

						// fireEvent();
					}
				}

			});

			dzBis.hinzufuegenSelektionsZuhoerer(new SelectionListener() {

				private long endzeitpunkt = -1;

				@Override
				public void widgetDefaultSelected(final SelectionEvent e) {
					gemeinsam(e);
				}

				@Override
				public void widgetSelected(final SelectionEvent e) {
					gemeinsam(e);
				}

				private void gemeinsam(final SelectionEvent e) {
					if (e.data instanceof Date) {
						final Date datum = (Date) e.data;
						endzeitpunkt = datum.getTime();
						daten.setEndzeitpunkt(endzeitpunkt);

						if (!pruefePlausibilitaet()) {
							final Color color = new Color(Display.getDefault(),
									255, 0, 0);
							labelInfo.setForeground(color);
							labelInfo
									.setText("Startzeitpunkt liegt vor dem Endzeitpunkt!");
							compositeZeitGruppenInhalt.layout();
						} else {
							labelInfo.setText("");
							compositeZeitGruppenInhalt.layout();
						}

						// fireEvent();
					}
				}

			});

			schaltflaecheHinzufuegen = new Button(returnComposite, SWT.NONE);
			schaltflaecheHinzufuegen.setText("+");
			schaltflaecheLoeschen = new Button(returnComposite, SWT.NONE);
			schaltflaecheLoeschen.setText("-");

			schaltflaecheHinzufuegen
					.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(final SelectionEvent e) {
							aktionZeitgruppenzeileAnhaengen();
						}
					});

			schaltflaecheLoeschen.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(final SelectionEvent e) {
					aktionZeitgruppenzeileLoeschen();
				}
			});

			final GridData gridData = new GridData();
			gridData.horizontalAlignment = GridData.FILL;
			gridData.grabExcessHorizontalSpace = false;
			gridData.grabExcessVerticalSpace = false;
			gridData.verticalAlignment = GridData.CENTER;
			returnComposite.setLayoutData(gridData);

			final GridLayout gridLayout = new GridLayout();
			gridLayout.numColumns = 7;
			returnComposite.setLayout(gridLayout);

			return returnComposite;
		}

	}

	@Override
	public void widgetDefaultSelected(final SelectionEvent e) {
		widgetSelected(e);
	}

	@Override
	public void widgetSelected(final SelectionEvent e) {
		if (e.getSource() instanceof Button) {
			System.out.println("HERE");
			// Monat finden
			final int monatIndex = findeMonatCheckbox((Button) e.getSource());
			if (monatIndex > -1) {
				System.out.println("Monat: " + monatIndex);
				if (((Button) e.getSource()).getSelection()) {
					datenStruktur.selektiereMonat(monatIndex);
				} else {
					datenStruktur.deSelektiereMonat(monatIndex);
				}

				// fireEvent();
			}

			// Tag finden
			final int tagIndex = findeTagCheckbox((Button) e.getSource());
			if (tagIndex > -1) {
				KalenderZeitFilterDialog.LOGGER.fine("Tag: " + tagIndex);
				if (((Button) e.getSource()).getSelection()) {
					datenStruktur.selektiereTag(tagIndex);
				} else {
					datenStruktur.deSelektiereTag(tagIndex);
				}
			}

			// Wochentag finden
			final int wochentagIndex = findeWochentagCheckbox((Button) e
					.getSource());
			if (wochentagIndex > -1) {
				KalenderZeitFilterDialog.LOGGER.fine("Wochentag: "
						+ wochentagIndex);
				if (((Button) e.getSource()).getSelection()) {
					datenStruktur.selektiereWochentag(wochentagIndex);
				} else {
					datenStruktur.deSelektiereWochentag(wochentagIndex);
				}
			}

		} else {
			System.out.println("null");
		}
	}

}
