/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.internal.RwBasisLibActivator;
import de.bsvrz.buv.rw.basislib.kalender.IEintragBereich;
import de.bsvrz.buv.rw.basislib.kalender.KalenderBereichDialog;

/**
 * Farbik-Klasse, produziert Buttons zum Aufruf vom Dialog zum Auswhlen von
 * Ereignis- und Systemkalendereintrgen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: SchaltflaechenFabrikEreignisSystemKalenderDialog.java 47112 2013-11-12 15:25:14Z peuker $
 */
public final class SchaltflaechenFabrikEreignisSystemKalenderDialog {

	private SchaltflaechenFabrikEreignisSystemKalenderDialog() {

	}

	/**
	 * Diese Methode erzeugt einen {@link Button}, mit dessen Hilfe sich der
	 * Dialog zur Auswahl von Ereignis- und Systemkalendereintrgen aufrufen
	 * lsst und sich Ereignisse bernehmen lassen.
	 * 
	 * @param elternComposite
	 *            Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *            erfolgen soll.
	 * @param felderFuellen
	 *            Instanz des Interface {@link IFelderFuellen}, bestimmt welche
	 *            Felder mit dem Ergebnissen der Auswahl aus dem Dialog gefllt
	 *            werden.
	 * @param text
	 *            Text des Buttons. Wenn dieser gesetzt ist, wird der Button mit
	 *            Text dargestellt. Wenn <code>null</code>, dann wird auf dem
	 *            Button ein Bild dargestellt.
	 * @param bild
	 *            Bild welches auf dem Button dargestellt werden soll, wenn der
	 *            Text auf <code>null</code> gesetzt wurde. Wenn hier
	 *            <code>null</code> angegeben wird, wird ein Default-Bild
	 *            dargestellt.
	 * @return den Button
	 */
	public static Button erzeugeButton(final Composite elternComposite,
			final IFelderFuellen felderFuellen, final String text,
			final Image bild) {
		Button re = null;

		if (felderFuellen != null) {
			re = new Button(elternComposite, SWT.PUSH);
			if (text != null) {
				re.setText(text);
			} else if (bild != null) {
				re.setImage(bild);
			} else {
				re.setImage(RwBasisLibActivator.getDefault().getImage(
						"icons/etool16/zeit_datum.png"));
			}

			re.setToolTipText("ffnet den Dialog zur bernahme von Ereignis- und Systemkalendereintrgen");

			re.addSelectionListener(new SchaltflaechenFabrikEreignisSystemKalenderDialog.MausSelectionAdapter(
					felderFuellen));
		}

		return re;
	}

	/**
	 * Hilfsklasse, implementiert einen Listener fr den Button, welcher den
	 * Dialog zur bernahme von System- und Ereigniskalendereintrgen ffnet.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class MausSelectionAdapter extends SelectionAdapter {
		/**
		 * Interface, enthlt alle notwendigen Informationen zu den Feldern, die
		 * gefllt werden sollen.
		 */
		private final IFelderFuellen felderFuellen;

		public MausSelectionAdapter(final IFelderFuellen felderFuellen) {
			this.felderFuellen = felderFuellen;
		}

		/**
		 * ffnet den Dialog zur bernahme von Ereignis- und
		 * Systemkalendereintrgen. Wurde das Interface {@link IFelderFuellen}
		 * gesetzt, wird dessen Methode {@link IFelderFuellen#fuellen(List)} mit
		 * den von dem Dialog zurckgelieferten Daten als Parameter aufgerufen.
		 */
		private void aktionDialogOeffnen() {
			final KalenderBereichDialog dialog = new KalenderBereichDialog(
					PlatformUI.getWorkbench().getDisplay().getActiveShell());

			dialog.open();
			final List<IEintragBereich> bereich = dialog
					.getSelektierteBereiche();

			if ((felderFuellen != null) && (bereich != null)
					&& (!bereich.isEmpty())) {
				felderFuellen.fuellen(bereich);
			}
		}

		@Override
		public void widgetSelected(final SelectionEvent e) {
			aktionDialogOeffnen();
		}

	}

}
