/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.kalender.internal;

/**
 * Definition der Zeitklassen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: ZeitKlasse.java 45393 2013-08-07 21:32:57Z peuker $
 */
public enum ZeitKlasse {

	/** Definition von Jahren. */
	JAHR("Jahr", "Jahre", "J"),

	/** Definition von Monaten. */
	MONAT("Monat", "Monate", "M"),

	/** Definition von Tagen. */
	TAG("Tag", "Tage", "T"),

	/** Definition von Stunden. */
	STUNDEN("Stunde", "Stunden", "h"),

	/** Definition von Minuten. */
	MINUTE("Minute", "Minuten", "m"),

	/** Definition von Sekunden. */
	SEKUNDE("Sekunde", "Sekunden", "s"),

	/** Definition von Millisekunden. */
	MILLSEKUNDE("Tausendstel", "Tausendstel", "t");

	private final String bezeichnung;
	private final String mehrzahl;
	private final String kurzform;

	// XXX Beschrnkung wird nie gesetzt? */
	/** Beschrnkung im Wertebereich nach unten. */
	private final int beschraenkungMin = -1;

	/** Beschrnkung im Wertebereich nach oben. */
	private final int beschraenkungMax = -1;

	/**
	 * Konstruktor. bernimmt Bezeichnung und Kurzform dieser {@link ZeitKlasse}
	 * 
	 * @param bezeichnung
	 *            Bezeichnung
	 * @param mehrzahl
	 *            Mehrzahl der Bezeichnung
	 * @param kurzform
	 *            Kurzform der Bezeichnung
	 */
	private ZeitKlasse(final String bezeichnung, final String mehrzahl,
			final String kurzform) {
		this.bezeichnung = bezeichnung;
		this.mehrzahl = mehrzahl;
		this.kurzform = kurzform;
	}

	/**
	 * Liefert die Bezeichnung dieser {@link ZeitKlasse}.
	 * 
	 * @return Bezeichnung
	 */
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * Mehrzahl der Bezeichnung dieser {@link ZeitKlasse}.
	 * 
	 * @return Mehrzahl der Bezeichnung
	 */
	public String getMehrzahl() {
		return mehrzahl;
	}

	/**
	 * Liefert die Kurzform dieser {@link ZeitKlasse}.
	 * 
	 * @return Kurzform
	 */
	public String getKurzform() {
		return kurzform;
	}

	/**
	 * ermittelt, ob die Zeitklasse beschrnkt ist.
	 * 
	 * @return der Status
	 */
	public boolean istBeschraenkt() {
		return false;
	}

	/**
	 * Liefert die untere Grenze fr den Wert dieser Zeitklasse.
	 * 
	 * @return Untere Grenze. <code>-1</code> bedeutet, keine untere Grenze.
	 */
	public int getBeschraenkungMin() {
		return beschraenkungMin;
	}

	/**
	 * Liefert die obere Grenze fr den Wert dieser Zeitklasse.
	 * 
	 * @return Obere Grenze. <code>-1</code> bedeutet, keine obere Grenze.
	 */
	public int getBeschraenkungMax() {
		return beschraenkungMax;
	}

	@Override
	public String toString() {
		final String re = "ZeitKlasse - Bezeichnung: " + bezeichnung
				+ " Kurzform: " + kurzform;
		return re;
	}

	/**
	 * Prft an Hand eines Strings, ob es sich um die Bezeichnung dieser
	 * {@link ZeitKlasse} handelt.
	 * 
	 * @param pruefString
	 *            Zu prfender String
	 * @return Liefert <code>true</code>, wenn es sich um die Bezeichnung dieser
	 *         {@link ZeitKlasse} handelt, sonst <code>false</code>.
	 */
	public boolean istStringBezeichnung(final String pruefString) {
		boolean re = true;

		if (!bezeichnung.equalsIgnoreCase(pruefString)) {
			re = false;
		}

		return re;
	}

	/**
	 * Prft an Hand eines Strings, ob es sich um die Mehrzahl dieser
	 * {@link ZeitKlasse} handelt.
	 * 
	 * @param pruefString
	 *            Zu prfender String
	 * @return Liefert <code>true</code>, wenn es sich um die Mehrzahl dieser
	 *         {@link ZeitKlasse} handelt, sonst <code>false</code>.
	 */
	public boolean istStringMehrzahl(final String pruefString) {
		boolean re = true;

		if (!mehrzahl.equalsIgnoreCase(pruefString)) {
			re = false;
		}

		return re;
	}

	/**
	 * Prft an Hand eines Strings, ob es sich um die Kurzform dieser
	 * {@link ZeitKlasse} handelt.
	 * 
	 * @param pruefString
	 *            Zu prfender String
	 * @return Liefert <code>true</code>, wenn es sich um die Kurzform dieser
	 *         {@link ZeitKlasse} handelt, sonst <code>false</code>.
	 */
	public boolean istStringKurzform(final String pruefString) {
		boolean re = true;

		if (!kurzform.equals(pruefString)) {
			re = false;
		}

		return re;
	}

	/**
	 * liefert eine fromatierte Darstellung fr den angegebenen Wert.
	 * 
	 * @param wert
	 *            der Wert
	 * @param kurzDarstellung
	 *            Kurzdarstellung verwenden?
	 * @return der Text
	 */
	public String format(final int wert, final boolean kurzDarstellung) {

		final StringBuilder result = new StringBuilder();

		result.append(wert);
		result.append(' ');

		if (kurzDarstellung) {
			result.append(getKurzform());
		} else {
			if (wert > 1) {
				result.append(getMehrzahl());
			} else {
				result.append(getBezeichnung());
			}
		}

		return result.toString();
	}
}
