/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen.FaltStatusSpalten;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.kupzog.ktable.KTable;
import de.kupzog.ktable.KTableCellResizeListener;
import de.kupzog.ktable.KTableCellSelectionListener;

/**
 * Dialog fr die Listendarstellung.
 * 
 * @author HofmannJ
 * 
 */
public class ListenDarstellungDialog {

	private Shell shell;

	private Group groupOben;

	private Composite compositeUnten;

	private Composite compositeUntenRechts;

	private Button buttonOk;

	private Button buttonAbbrechen;

	private DavEmpfangenSendenAenderungsDarstellung davEmpfangAenderung;

	/**
	 * Verbindung zum DaV.
	 */
	private final ClientDavConnection davVerbindung;

	/**
	 * ObjektTyp.
	 */
	private String typ = "typ.autarkeOrganisationsEinheit";

	/**
	 * Pid des SystemObjektes, auf welches die Anmeldung erfolgt.
	 */
	private String pidSystemObject = "kv.testKonfiguration";

	/**
	 * Attributgruppe, unter welcher die Anmeldung erfolgt.
	 */
	private String atg = "atg.oberflchenBerechtigung";

	/**
	 * Aspekt, unter dem die Anmeldung erfolgt.
	 */
	private String asp = "asp.parameterIst";

	/**
	 * Tabelle, auf deren Basis die Darstellung erfolgt.
	 */
	private KTable kTable;

	/**
	 * Das Tabellenmodell, mit dem die Tabelle versorgt wird.
	 */
	private TabellenModell tabellenModell;

	/**
	 * Art, wie die Darstellung erfolgen soll.
	 */
	private EnumDarstellungsArt darstellungsArt;

	/**
	 * Gibt die Breite fr die Faltung an.
	 */
	private static final int BREITE_FALTUNG = 10;

	/**
	 * Konstruktor.
	 * 
	 * @param davVerbindung
	 *            Verbindung zum DaV.
	 */
	public ListenDarstellungDialog(final ClientDavConnection davVerbindung) {
		this.davVerbindung = davVerbindung;
	}

	/**
	 * This method initializes groupOben.
	 * 
	 */
	private void createGroupOben() {
		final GridData gridDataObenInhalt = new GridData();
		gridDataObenInhalt.horizontalAlignment = GridData.FILL;
		gridDataObenInhalt.grabExcessHorizontalSpace = true;
		gridDataObenInhalt.grabExcessVerticalSpace = true;
		gridDataObenInhalt.verticalAlignment = GridData.FILL;
		final GridData gridDataOben = new GridData();
		gridDataOben.grabExcessHorizontalSpace = true;
		gridDataOben.verticalAlignment = GridData.FILL;
		gridDataOben.grabExcessVerticalSpace = true;
		gridDataOben.horizontalAlignment = GridData.FILL;
		groupOben = new Group(shell, SWT.NONE);
		groupOben.setLayout(new GridLayout());
		groupOben.setLayoutData(gridDataOben);
		groupOben.setText("Tabelle:");

		kTable = new KTable(groupOben, SWT.V_SCROLL | SWT.H_SCROLL
				| SWT.HIDE_SELECTION);
		kTable.setLayoutData(gridDataObenInhalt);

		final LinkedList<DataTableObjectErweitert> dtos = new LinkedList<DataTableObjectErweitert>();

		tabellenModell = new TabellenModell(davVerbindung, typ, atg, asp, dtos);

		// _tabellenModell
		// .setDarstellungsArt(EnumDarstellungsArt.aenderungsDarstellung);

		tabellenModell.setDarstellungsArt(darstellungsArt);

		kTable.setModel(tabellenModell);

		/*
		 * Spalten der Tabelle auf die optimale Breite bringen
		 */
		final int anzSpalten = kTable.getModel().getColumnCount();
		for (int i = 0; i < anzSpalten; i++) {
			kTable.resizeColumnOptimal(i);
		}

		kTable.addCellSelectionListener(new KTableCellSelectionListener() {
			@Override
			public void cellSelected(final int col, final int row,
					final int statemask) {
				System.out.println("Cell [" + col + ";" + row + "] selected.");
			}

			@Override
			public void fixedCellSelected(final int col, final int row,
					final int statemask) {
				System.out
						.println("Header [" + col + ";" + row + "] selected.");

				falten(col, row);

			}

		});

		kTable.addCellResizeListener(new KTableCellResizeListener() {
			@Override
			public void columnResized(final int col, final int newWidth) {
				System.out
						.println("Column: " + col + " resized to " + newWidth);
				// _kTable.getModel().getFaltStatusSpalten().setZelleGefaltet(col,
				// 1,
				// newWidth, false);

				if (kTable.getModel() instanceof TabellenModell) {
					final TabellenModell modell = (TabellenModell) kTable
							.getModel();
					modell.getFaltStatusSpalten().setZelleGefaltet(col, 1,
							newWidth, false);
				}

				kTable.redraw();
			}

			@Override
			public void rowResized(final int row, final int newHeight) {
				System.out.println("Row " + row + " resized to " + newHeight);
			}
		});

	}

	/**
	 * Fhrt die Faltung durch.
	 * 
	 * @param x
	 *            Spalte
	 * @param y
	 *            Zeile
	 */
	protected void falten(final int x, final int y) {
		if (kTable.getModel() instanceof TabellenModell) {
			final TabellenModell modell = (TabellenModell) kTable.getModel();

			final int xNeu = x - modell.getFixedColumnCount();
			System.out.println("XNEU: " + xNeu);

			final Knoten wurzelKnoten = modell.getWurzelKopfzeile();
			final LinkedList<Knoten> knotenAufEbene = wurzelKnoten
					.bestimmeKnotenAufEbene(y);
			final Iterator<Knoten> it = knotenAufEbene.iterator();
			Knoten knotenImBereich = null;

			while (it.hasNext()) {
				final Knoten knoten = it.next();
				if (knoten.istInBereich(y, xNeu)) {
					knotenImBereich = knoten;
					System.out.println("=> " + knoten.getWert());
					break;
				}
			}

			if (knotenImBereich != null) {
				final LinkedList<Knoten> faltListe = new LinkedList<Knoten>();
				faltListe.add(knotenImBereich);
				if (knotenImBereich.hatNachfolger()) {
					faltListe.addAll(knotenImBereich
							.getListeAllerKnotenDfsRekursiv(knotenImBereich));
				}

				if (faltListe.size() > 0) {

					final FaltStatusSpalten faltStatusSpalten = modell
							.getFaltStatusSpalten();
					final Knoten ersterKnoten = faltListe.get(0);
					int vonX = ersterKnoten.getVonX()
							+ modell.getFixedColumnCount();
					int vonY = ersterKnoten.getVonY();
					final boolean falten = !faltStatusSpalten.isZelleGefaltet(
							vonX, vonY);

					final Iterator<Knoten> itFalten = faltListe.iterator();
					while (itFalten.hasNext()) {
						final Knoten faltKnoten = itFalten.next();
						vonX = faltKnoten.getVonX()
								+ modell.getFixedColumnCount();
						vonY = faltKnoten.getVonY();
						System.out.println(faltKnoten.getSpaltenBezeichnung()
								+ " => VONX: " + vonX + " VONY: " + vonY
								+ " Breite: " + modell.getColumnWidth(vonX));

						faltStatusSpalten.setZelleGefaltet(vonX, vonY,
								modell.getColumnWidth(vonX), falten);

						if (falten) {
							System.out.println("X WIDTH: " + vonX);

							modell.setColumnWidth(vonX,
									ListenDarstellungDialog.BREITE_FALTUNG);
						} else {
							modell.setColumnWidth(vonX, faltStatusSpalten
									.getAusgangsBreite(vonX, vonY));
							System.out.println("X WIDTH: " + vonX);
						}
					}

					kTable.redraw();
				}

			}

		}
	}

	/**
	 * This method initializes compositeUnten.
	 * 
	 */
	private void createCompositeUnten() {
		final GridData gridDataUnten = new GridData();
		gridDataUnten.grabExcessHorizontalSpace = true;
		gridDataUnten.verticalAlignment = GridData.CENTER;
		gridDataUnten.horizontalAlignment = GridData.FILL;
		compositeUnten = new Composite(shell, SWT.NONE);
		compositeUnten.setLayout(new GridLayout());
		createCompositeUntenRechts();
		compositeUnten.setLayoutData(gridDataUnten);
	}

	/**
	 * Schliet die Shell.
	 */
	protected void aktionSchliessen() {
		if (davEmpfangAenderung != null) {
			davEmpfangAenderung.abmeldenEmpfang(pidSystemObject, atg, asp);
		}

		shell.dispose();
	}

	private void aktionAktualisieren() {
		kTable.redraw();
	}

	/**
	 * This method initializes compositeUntenRechts.
	 * 
	 */
	private void createCompositeUntenRechts() {
		final GridData gridDataUntenRechts = new GridData();
		gridDataUntenRechts.horizontalAlignment = GridData.END;
		gridDataUntenRechts.grabExcessHorizontalSpace = true;
		gridDataUntenRechts.verticalAlignment = GridData.CENTER;
		final RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		compositeUntenRechts = new Composite(compositeUnten, SWT.NONE);
		compositeUntenRechts.setLayout(rowLayout);
		compositeUntenRechts.setLayoutData(gridDataUntenRechts);
		buttonOk = new Button(compositeUntenRechts, SWT.NONE);
		buttonOk.setText("OK");
		buttonOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionSchliessen();
			}
		});

		buttonAbbrechen = new Button(compositeUntenRechts, SWT.NONE);
		buttonAbbrechen.setText("Abbrechen");
		buttonAbbrechen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				// aktionSchliessen();
				aktionAktualisieren();
			}
		});
	}

	protected void anmeldenTabelleZumEmpfang() {
		if (davEmpfangAenderung == null) {

			davEmpfangAenderung = new DavEmpfangenSendenAenderungsDarstellung(
					davVerbindung);

			davEmpfangAenderung.setDisplay(Display.getDefault());

		}

		davEmpfangAenderung.setKTable(kTable);
		davEmpfangAenderung.setTabellenModell(tabellenModell);

		davEmpfangAenderung.anmeldenEmpfang(pidSystemObject, atg, asp);
	}

	/**
	 * ffnet den Dialog zur ListenDarstellung mit notwendigen Parametern. Der
	 * Entwickler hat dafr sorge zu tragen, dass die genannten Parameter
	 * korrekt sind.
	 * 
	 * @param newObjekt
	 *            SystemObject, auf welches die Anmeldung erfolgen soll.
	 * @param newTyp
	 *            Objekttyp
	 * @param newAtg
	 *            AttributGruppe der Anmeldung
	 * @param newAsp
	 *            Aspekt, unter dem die Anmeldung erfolgen soll.
	 * @param newDarstellungsArt
	 *            Art, wie die Darstellung erfolgen soll.
	 * @return
	 */
	public boolean open(final String newObjekt, final String newTyp,
			final String newAtg, final String newAsp,
			final EnumDarstellungsArt newDarstellungsArt) {
		this.pidSystemObject = newObjekt;
		this.typ = newTyp;
		this.atg = newAtg;
		this.asp = newAsp;
		this.darstellungsArt = newDarstellungsArt;

		boolean re = false;

		final Display display = Display.getDefault();

		erzeugeShell();
		shell.open();

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		// display.dispose();

		re = true;
		return re;

	}

	/**
	 * This method initializes sShell.
	 */
	private void erzeugeShell() {
		shell = new Shell();
		shell.setText("Listendarstellung");
		createGroupOben();
		createCompositeUnten();
		shell.setLayout(new GridLayout());
		shell.setSize(800, 600);

		/*
		 * Anmeldung zum Empfang
		 */
		anmeldenTabelleZumEmpfang();
	}

	public TabellenModell getTabellenModell() {
		return tabellenModell;
	}

}
