/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.login;

import java.util.Collection;

import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.buv.rw.basislib.urlasser.DefaultDatenSender;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserDatenDialog;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserInfo;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * UrlasserDialog.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: UrlasserDialog.java 47179 2013-11-18 08:03:03Z peuker $
 * 
 * @deprecated sollte durch die Basisklasse {@link UrlasserDatenDialog} ersetzt
 *             werden.
 */
@Deprecated
public class UrlasserDialog extends UrlasserDatenDialog {

	/**
	 * Diese Klasse wird von der anzeigen-Methode der Klasse
	 * {@link UrlasserDialog} zurckgeliefert und enthlt Informationen zu den
	 * Eingaben, die der Anwender im {@link UrlasserDialog} gettigt hat. Dazu
	 * zhlen:
	 * <ul>
	 * <li>Wurde OK oder Abbrechen angeklickt?</li>
	 * <li>Der eingegebene Benutzername.</li>
	 * <li>Das eingegebene Passwort.</li>
	 * <li>Die eingegebene Ursache.</li>
	 * <li>Der eingegebene Veranlasser.</li>
	 * </ul>
	 * 
	 * @author HofmannJ
	 */
	public static class UrlasserDialogInfo {
		/** Flag fr ok oder abbrechen. */
		private final boolean eingabeOk;

		/** Angegebener Benutzername. */
		private final String benutzer;

		/** Eingegbenes Passwort. */
		private final String passwort;

		/** Angegebene Ursache. */
		private final String ursache;

		/** Angegebener Veranlasser. */
		private final String veranlasser;

		/**
		 * Speichert Informationen, warum eine Datensatz nicht versendet werden
		 * konnt.
		 */
		private final String fehlerInfo;

		/**
		 * Konstruktor. bernimmt die nachfolgenden Parameter:
		 * 
		 * @param eingabeOk
		 *            Wurde OK oder Abbrechen geklickt?
		 * @param benutzer
		 *            Benutzername
		 * @param passwort
		 *            Passwort des Benutzers
		 * @param ursache
		 *            Ursache
		 * @param veranlasser
		 *            Veranlasser
		 * @param fehlerInfo
		 *            Fehlerinformationen.
		 */
		public UrlasserDialogInfo(final boolean eingabeOk,
				final String benutzer, final String passwort,
				final String ursache, final String veranlasser,
				final String fehlerInfo) {
			this.eingabeOk = eingabeOk;
			this.benutzer = benutzer;
			this.passwort = passwort;
			this.ursache = ursache;
			this.veranlasser = veranlasser;
			this.fehlerInfo = fehlerInfo;
		}

		/**
		 * Liefert den Benutzernamen, der im Dialog eingegeben wurde.
		 * 
		 * @return Benutzername
		 */
		public String getBenutzer() {
			return benutzer;
		}

		/**
		 * Prft, ob der Dialog mit OK oder Abbrechen beendet wurde.
		 * 
		 * @return Liefert <code>true</code>, wenn der Dialog mit OK beendet
		 *         wurde und <code>false</code>, wenn der Dialog mit Abbrechen
		 *         beendet wurde.
		 */
		public boolean getEingabeOk() {
			return eingabeOk;
		}

		/**
		 * Liefert Informationen, warum ein Datensatz nicht versendet werden
		 * konnte.
		 * 
		 * @return Fehlerinformationen
		 */
		public String getFehlerInfo() {
			return fehlerInfo;
		}

		/**
		 * Liefert das Passwort, welches im Dialog eingegeben wurde.
		 * 
		 * @return Passwort
		 */
		public String getPasswort() {
			return passwort;
		}

		/**
		 * Liefert die Ursache, die im Dialog eingegben wurde.
		 * 
		 * @return Ursache
		 */
		public String getUrsache() {
			return ursache;
		}

		/**
		 * Liefert den Veranlasser, der im Dialog eingegeben wurde.
		 * 
		 * @return Veranlasser
		 */
		public String getVeranlasser() {
			return veranlasser;
		}
	}

	public UrlasserDialog(final Shell parent) {
		super(parent, new DefaultDatenSender(null));
	}

	/**
	 * Diese Methode initialisiert die Shell dieses Dialogs und bernimmt die zu
	 * versendenden Daten.
	 * 
	 * @param resultDatas
	 *            Zu versendende {@link ResultData}s. Knnen z.B. in einer Liste
	 *            geliefert werden. Im {@link ResultData} muss der korrekte
	 *            Aspekt fr das Versenden in der DataDescription eingetragen
	 *            sein.
	 * @return Liefert in einer {@link UrlasserDialogInfo}-Instanz Informationen
	 *         dazu, ob der Dialog mit Abbrechen oder OK beendet wurde sowie die
	 *         in den Feldern eingegebenen Strings.
	 */
	public UrlasserDialogInfo anzeigen(final Collection<ResultData> resultDatas) {

		((DefaultDatenSender) getDatenSender()).addResultData(resultDatas);
		final boolean dialogOk = open() == Window.OK;
		final UrlasserInfo info = getUrlasserInfo();
		return new UrlasserDialogInfo(dialogOk, info.getBenutzerName(),
				info.getPasswort(), info.getUrsache(), info.getVeranlasser(),
				"");
	}

	/**
	 * Diese Methode initialisiert die Shell dieses Dialogs und bernimmt die zu
	 * versendenden Daten.
	 * 
	 * @param resultDatas
	 *            Zu versendende {@link ResultData}s. Knnen z.B. in einer
	 *            LinkedList geliefert werden. Im {@link ResultData} muss der
	 *            korrekte Aspekt fr das Versenden in der DataDescription
	 *            eingetragen sein.
	 * @param benutzer
	 *            Name des Benutzers, mit dem das Feld Benutzer vorbelegt werden
	 *            soll.
	 * @param passwort
	 *            Passwort, mit dem das Feld Passwort vorbelegt werden soll.
	 * @param ursache
	 *            Ursache, mit der das Feld Ursache vorbelegt werden soll.
	 * @param veranlasser
	 *            Veranlasser, mit dem das Feld Veranlasser vorbelegt werden
	 *            soll.
	 * @return Liefert in einer {@link UrlasserDialogInfo}-Instanz Informationen
	 *         dazu, ob der Dialog mit Abbrechen oder OK beendet wurde sowie die
	 *         in den Feldern eingegebenen Strings.
	 */
	public UrlasserDialogInfo anzeigen(
			final Collection<ResultData> resultDatas, final String benutzer,
			final String passwort, final String ursache,
			final String veranlasser) {

		((DefaultDatenSender) getDatenSender()).addResultData(resultDatas);
		setInitialBenutzer(benutzer);
		setInitialUrsache(ursache);
		setInitialVeranlasser(veranlasser);

		final boolean dialogOk = open() == Window.OK;
		final UrlasserInfo info = getUrlasserInfo();
		return new UrlasserDialogInfo(dialogOk, info.getBenutzerName(),
				info.getPasswort(), info.getUrsache(), info.getVeranlasser(),
				"");
	}
}
