/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.menu;

import java.util.Map;

import org.eclipse.jface.action.IContributionItem;

/**
 * Schnittstelle fr ein Element, das in die Menstruktur (Men, Toolbar,
 * Statusleiste) der Rahmenwerkapplikation eingebaut werden kann.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwMenuControl.java 45393 2013-08-07 21:32:57Z peuker $
 */
public interface RwMenuControl extends IContributionItem {

	/**
	 * bergibt der Instanz die im Meneditor eingestellten Parameter fr das
	 * Element.
	 * 
	 * @param parameter
	 *            die Parameter als Zurodnungsliste (ID, Wert)
	 */
	void setParameter(Map<String, String> parameter);

	/**
	 * ermittelt, ob das Element im Hauptmen des Rahmenwerks verwendet werden
	 * kann.
	 * 
	 * @return den Status
	 */
	boolean useInMenue();

	/**
	 * ermittelt, ob das Element in der Symbolleiste des Rahmenwerks verwendet
	 * werden kann.
	 * 
	 * @return den Status
	 */
	boolean useInToolbar();

	/**
	 * ermittelt, ob das Element in der Statusleiste des Rahmenwerks verwendet
	 * werden kann.
	 * 
	 * @return den Status
	 */
	boolean useInStatusLine();
}
