/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.basislib.datenidentifikation.Datenidentifikation;
import de.bsvrz.buv.rw.basislib.datenidentifikation.Datenidentifikation.Darstellung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse ermglicht die Objektzusammenstellung. <br>
 * Die Klasse erfllt folgende Technische Anforderungen des Segments Bedienung
 * und Visualisierung (BUV)<br>
 * - TBuV-96 und 97
 * 
 * Einschrnkungen: In einer Eclipse RCP Anwendung sind Dialoge modal, d.h. sie
 * berlagern alle anderen Dialoge und damit das gesamte Rahmenwerk, wenn sie
 * aktiv sind. Dadurch ist es nicht mglich, per Drag & Drop SystemObjekte aus
 * der Komponente zur Datenidentifikation hinzuzufgen.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/14 13:15:29 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author Dambach-Werke GmbH
 * @author HofmannJ
 * 
 */
public class Objektzusammenstellung extends Dialog {

	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	private final ClientDavInterface connection;

	/*
	 * Standard GUI-Elemente
	 */
	private Shell shell;

	private Group groupObjekte;

	private Composite compositeMitte;

	private Button checkBoxObjekttypEinschraenken;

	private Label labelObjekttypenEinschraenken;

	private Group groupBeschreibung;

	private Text textAreaBeschreibung;

	private Button buttonHinzufuegen;

	private Button buttonLoeschen;

	private Button buttonHoch;

	private Button buttonRunter;

	private Button buttonOk;

	private Button buttonAbbrechen;

	private Button buttonHilfe;

	/**
	 * SWT-Liste, die im Dialog sichtbar ist.
	 */
	private List listeSWT;

	/**
	 * Enspricht den Indices der in der Liste ListeSWT selektierten Elementen.
	 */
	private int[] selektion;

	/**
	 * ber die Check-Box "Objekttypen einschrnken" wird vorgegeben, dass nur
	 * Datenidentifikationen in die Objektzusammenstellung bernommen werden
	 * knnen, die von einem Objekttyp sind, der bereits in der
	 * Objektzusammenstellung vorkommt. Wenn z.B. nur Objekte vom Typ MQ in der
	 * Objektzusammenstellung enthalten sind und die Check-Box aktiviert ist,
	 * dann knnen auch nur weitere Objekte vom Typ MQ hinzugefht werden.
	 * Dieser Umstand muss sich direkt fr die Datenidentifikations-Eingabe
	 * auswirken. D.h. wenn in diesem Fall "Hinzufgen" bettigt wird, muss die
	 * Datenidentifikations-Eingabe bereits auf Objekte vom Typ "MQ" filtern.
	 * Drag und Drop wird nur fr Objekte der entsprechenden Objekttypen
	 * zugelassen Die Check-Box ist optional. Fr die Zusammenstellung eines
	 * Streckenprofils ist die Check-Box nicht erforderlich. In diesen Fllen
	 * bestimmt die Anwendung bereits welche Objekttypen zulssig sind. Checkbox
	 * ja/nein
	 * 
	 * Gibt an, ob nach ObjektTyp eingeschrnkt werden soll oder nicht.
	 */
	private boolean einschraenkenObjektTyp;

	/**
	 * Liste von Listenobjekte, die der Objektreprsentation der in der Liste
	 * listObjektListe enthaltenen Elementen entspricht.
	 */
	private java.util.List<ListenObjekt> listeDaten;

	/**
	 * Hier werden die Erfassten SystemObjekte an die Klasse
	 * Objektzusammenstellung zurckgegeben.
	 */
	private final java.util.List<SystemObject> listeRueckgabeSystemObjekte;

	/**
	 * Konstruktor der Klasse {@link Objektzusammenstellung}. <br>
	 * Erezeugt eine neue Instanz dieser Klasse, die Darstellung erfolgt auf der
	 * Basis der bergebenen Eltern-Shell. <br>
	 * 
	 * @param parent
	 *            Eltern-Shell, auf deren Basis die Darstellung erfolgen soll.
	 * @param connection
	 *            Verbindung zum DaV. Kann auch <code>null</code> sein, dann
	 *            kann jedoch kein {@link SystemObject} ber den Dialog zur
	 *            Datenidentifikation hinzugefgt werden.
	 * @param rueckListe
	 *            Liste von Systemobjekten, in der die Objektzusammenstellung
	 *            erfolgen soll. Sind in dieser Liste schon Systemobjekte
	 *            vorhanden, werden diese im Dialog angezeigt.
	 */
	public Objektzusammenstellung(final Shell parent,
			final ClientDavInterface connection,
			final java.util.List<SystemObject> rueckListe) {
		super(parent);
		this.connection = connection;
		listeRueckgabeSystemObjekte = rueckListe;
	}

	/**
	 * Konstruktor der Klasse {@link Objektzusammenstellung}. Erezeugt eine neue
	 * Instanz dieser Klasse, die Darstellung erfolgt auf der Basis der
	 * bergebenen Eltern-Shell. <br>
	 * 
	 * @param parent
	 *            Eltern-Shell, auf deren Basis die Darstellung erfolgen soll.
	 * @param connection
	 *            Verbindung zum DaV. Kann auch <code>null</code> sein, dann
	 *            kann jedoch kein {@link SystemObject} ber den Dialog zur
	 *            Datenidentifikation hinzugefgt werden.
	 * @param rueckListe
	 *            Liste von Systemobjekten, in der die Objektzusammenstellung
	 *            erfolgen soll. Sind in dieser Liste schon Systemobjekte
	 *            vorhanden, werden diese im Dialog angezeigt.
	 * @param filtern
	 *            Wenn <code>true</code>, dann wird nach den Systemobjekttypen
	 *            gefiltert, bei <code>false</code> nicht.
	 */
	public Objektzusammenstellung(final Shell parent,
			final ClientDavInterface connection,
			final java.util.List<SystemObject> rueckListe, final boolean filtern) {
		super(parent);
		this.connection = connection;
		listeRueckgabeSystemObjekte = rueckListe;
		einschraenkenObjektTyp = filtern;
	}

	// Hier mssen noch die RckgabeWerte als Array von SystemObject
	// eingebaut werden.
	// Prfung auf bereits in der Liste vorhandene SystemObjekte hinzufgen.

	/**
	 * ffnet den Dialog zur Objektzusammenstellung.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn der erfolgreich geffnet
	 *         wurde.
	 */
	public boolean open() {
		final Shell parent = getParent();
		shell = new Shell(parent, SWT.TITLE | SWT.BORDER
				| SWT.APPLICATION_MODAL);

		// 1. Erezugen der grafischen Darstellung.
		erzeugeShell();

		// 2. Initialisieren der Datenstrukturen.
		initDatenstrukturen();

		// 3. ffnen der Shell.
		shell.open();

		final Display display = parent.getDisplay();

		// Dialog zentrieren
		final Monitor primary = display.getPrimaryMonitor();
		final Rectangle bounds = primary.getBounds();
		final Rectangle rect = shell.getBounds();
		final int x = bounds.x + ((bounds.width - rect.width) / 2);
		final int y = bounds.y + ((bounds.height - rect.height) / 2);
		shell.setLocation(x, y);

		// Displayloop.
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		return true;
	}

	/**
	 * Initialisiert die hier bentigten Datenstrukturen.
	 * 
	 */
	private void initDatenstrukturen() {
		System.out
				.println("ObjektzusammenstellungDialog::Init => Datenstrukturen werden initialisiert.");
		listeDaten = new LinkedList<ListenObjekt>();

		// Fllen der Liste mit den bentigten Elementen
		fuelleObjektListe();

		// Fllten der Listen mit den bergebenen Elementen.
		uebergabeListeAngleichen();

		checkBoxObjekttypEinschraenken.setEnabled(true);

		listeSWT.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(final Event e) {
				if (listeSWT.getSelectionIndex() > -1) {
					// Beschreibung des jeweiligen Objektes in
					// textAreaBeschreibung anzeigen
					textAreaBeschreibung.setText("Beschreibung: "
							+ getListenObjektBeschreibung(listeSWT
									.getSelectionIndex()));

					// (Mehrfach-)Selektion als Index der jeweils selektierten
					// Elemente
					// im Array Selektion ablegen.
					selektion = listeSWT.getSelectionIndices();

					Objektzusammenstellung.LOGGER
							.fine("Anzahl selektierter Elemente: "
									+ selektion.length);
				}

			}
		});

	}

	/**
	 * Initialisierung von shell.
	 */
	private void erzeugeShell() {
		shell.setText("Objektzusammenstellung");
		shell.setLayout(null);
		erzeugeGruppeObjekte();
		erzeugeCompositeMitte();
		erzeugeGruppeBeschreibung();
		shell.setSize(new Point(481, 483));
		buttonOk = new Button(shell, SWT.NONE);
		buttonOk.setBounds(new Rectangle(60, 420, 101, 23));
		buttonOk.setText("OK");
		buttonOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				if ((selektion != null) && (selektion.length > 0)) {
					String elementUebersicht = "";
					for (final int element : selektion) {
						elementUebersicht = elementUebersicht + "E:" + element
								+ ", ";
					}
					Objektzusammenstellung.LOGGER.fine("Selektierte Elemente: "
							+ elementUebersicht);
				}

				shell.dispose();
			}
		});

		buttonAbbrechen = new Button(shell, SWT.NONE);
		buttonAbbrechen.setBounds(new Rectangle(180, 420, 111, 23));
		buttonAbbrechen.setText("Abbrechen");
		buttonAbbrechen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				shell.dispose();
			}
		});

		buttonHilfe = new Button(shell, SWT.NONE);
		buttonHilfe.setBounds(new Rectangle(310, 420, 101, 23));
		buttonHilfe.setText("Hilfe");
		buttonHilfe.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				Objektzusammenstellung.LOGGER.fine("Hilfe wurde geklickt.");
			}
		});
	}

	/**
	 * Initialisierung von groupObjekte.
	 * 
	 */
	private void erzeugeGruppeObjekte() {
		groupObjekte = new Group(shell, SWT.NONE);
		groupObjekte.setLayout(null);
		groupObjekte.setText("Objekte");
		groupObjekte.setBounds(new Rectangle(10, 10, 451, 191));
		listeSWT = new List(groupObjekte, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL
				| SWT.H_SCROLL);
		listeSWT.setBounds(new Rectangle(13, 23, 291, 151));

		/*
		 * Hinzufgen
		 */
		buttonHinzufuegen = new Button(groupObjekte, SWT.NONE);
		buttonHinzufuegen.setBounds(new Rectangle(323, 23, 121, 31));
		buttonHinzufuegen.setText("Hinzufgen");
		buttonHinzufuegen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				hinzufuegenObjekt();
			}
		});

		/*
		 * Lschen
		 */
		buttonLoeschen = new Button(groupObjekte, SWT.NONE);
		buttonLoeschen.setBounds(new Rectangle(323, 73, 121, 31));
		buttonLoeschen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				entferneSelektierteObjekteAusListe();
			}
		});

		/*
		 * Hoch
		 */
		buttonHoch = new Button(groupObjekte, SWT.ARROW | SWT.UP);
		buttonHoch.setBounds(new Rectangle(339, 133, 25, 25));
		buttonHoch.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				Objektzusammenstellung.LOGGER.fine("Hoch wurde geklickt.");
				// ListeSWT.getSelectionIndex()
				if (listeSWT.getSelectionIndex() > -1) {
					verschiebeNachOben(listeSWT.getSelectionIndex());
				}
			}
		});

		/*
		 * Runter
		 */
		buttonRunter = new Button(groupObjekte, SWT.ARROW | SWT.DOWN);
		buttonRunter.setBounds(new Rectangle(393, 133, 25, 25));
		buttonRunter.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				Objektzusammenstellung.LOGGER.fine("Runter wurde geklickt.");
				if (listeSWT.getSelectionIndex() > -1) {
					verschiebeNachUnten(listeSWT.getSelectionIndex());
				}
			}
		});
	}

	/**
	 * Initialisierung von compositeMitte.
	 * 
	 */
	private void erzeugeCompositeMitte() {
		compositeMitte = new Composite(shell, SWT.NONE);
		compositeMitte.setBounds(160, 212, 200, 30);

		checkBoxObjekttypEinschraenken = new Button(compositeMitte, SWT.CHECK);
		checkBoxObjekttypEinschraenken.setBounds(new Rectangle(10, 10, 13, 16));
		checkBoxObjekttypEinschraenken
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						if (e.getSource() instanceof Button) {
							final Button b = (Button) e.getSource();
							if (b.getSelection()) {
								Objektzusammenstellung.LOGGER
										.fine("Objekttypeinschrnken wurde aktiviert");
								einschraenkenObjektTyp = true;
								final String[] tempString = bestimmeVorhandeneObjektTypen();
								if (tempString.length > 0) {
									for (int i = 0; i < tempString.length; i++) {
										Objektzusammenstellung.LOGGER
												.fine("ObjektTyp #" + i + ": "
														+ tempString[i]);
									}
								}
							} else {
								Objektzusammenstellung.LOGGER
										.fine("Objekttypeinschrnken wurde deaktiviert");
								einschraenkenObjektTyp = false;
							}
						}
					}
				});

		// _checkBoxObjekttypEinschraenken.setEnabled(false);

		// Checkbox ObjekttypEinschraenken abhaken, wen eingeschrnkt werden
		// soll.
		if (einschraenkenObjektTyp) {
			checkBoxObjekttypEinschraenken.setSelection(true);
		} else {
			checkBoxObjekttypEinschraenken.setSelection(false);
		}

		labelObjekttypenEinschraenken = new Label(compositeMitte, SWT.NONE);
		labelObjekttypenEinschraenken.setText("Objekttyp einschrnken");
		labelObjekttypenEinschraenken.setBounds(new Rectangle(30, 10, 114, 13));

		compositeMitte.setSize(new Point(161, 34));
		compositeMitte.setLayout(null);

	}

	/**
	 * Initialisierung von groupBeschreibung.
	 * 
	 */
	private void erzeugeGruppeBeschreibung() {
		groupBeschreibung = new Group(shell, SWT.NONE);
		groupBeschreibung.setLayout(null);
		groupBeschreibung.setText("Beschreibung");
		groupBeschreibung.setBounds(new Rectangle(10, 250, 451, 151));
		textAreaBeschreibung = new Text(groupBeschreibung, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL);
		textAreaBeschreibung.setEditable(false);
		textAreaBeschreibung.setBounds(5, 15, 440, 132);
	}

	/**
	 * Fllt/Aktuallisiert listObjektListe mit den entsprechenden Werten aus
	 * Liste li der ListenObjekte.
	 * 
	 */
	private void fuelleObjektListe() {
		final ListIterator<ListenObjekt> it = listeDaten.listIterator();
		while (it.hasNext()) {
			final ListenObjekt obj = it.next();
			listeSWT.add(obj.getName());
		}
	}

	/**
	 * Liefert die Beschreibung fr ein selektiertes ListenObjekt.
	 * 
	 * @return
	 */
	private String getListenObjektBeschreibung(final int index) {
		return (listeDaten.get(index).getBeschreibung());
	}

	/**
	 * Entfernt die in listObjektListe selektierten Elemente.
	 * 
	 */
	private void entferneSelektierteObjekteAusListe() {
		if ((selektion != null) && (selektion.length > 0)) {
			String elemente = "";
			final int[] loeschen = listeSWT.getSelectionIndices();

			for (int i = 0; i < loeschen.length; i++) {
				elemente = elemente + loeschen[i] + ", ";
				listeSWT.remove(loeschen[i] - i);
				// -i, weill schon i Elemente aus
				// der Liste entfernt wurden
				listeDaten.remove(loeschen[i] - i);
				// -i, weill schon i Elemente aus
				// der Liste entfernt wurden
			}

			selektion = null;

			Objektzusammenstellung.LOGGER.fine("Die Elemente " + elemente
					+ " wurden aus der Liste entfernt.");
		} else {
			System.out
					.println("Es sind keine Objekte selektiert worden. Diese knnen nicht aus der Liste entfernt werden.");
		}

		// Wenn die Liste kein Element mehr enthltz, dann
		// checkBoxObjekttypEinschraenken deaktiviern
		if ((listeDaten.size() == 0) && (listeDaten.size() == 0)) {
			if (checkBoxObjekttypEinschraenken.getSelection()) {
				checkBoxObjekttypEinschraenken.setSelection(false);
			}
			checkBoxObjekttypEinschraenken.setEnabled(false);
		}

		rueckgabeListeAngleichen();
	}

	/**
	 * Fgt ein Objekt zur Liste hinzu.
	 * 
	 */
	private void hinzufuegenObjekt() {
		if (connection != null) {
			ListenObjekt listenObjekt = null;

			// An dieser Stelle muss die Datenidentifikationseingabe
			// geffnet werden
			// und das entsprechend zurckgegebene Objekt eingefgt werden.
			final Shell diShell = new Shell(Display.getDefault());
			final Datenidentifikation dialogDatenIdent = new Datenidentifikation(
					diShell, connection);
			// dialogDatenIdent.setConnection(_connection);

			SystemObject[] objects = null;
			if (einschraenkenObjektTyp) {
				// objects = dialogDatenIdent
				// .oeffne(
				// bestimmeVorhandeneObjektTypen(),
				// Datenidentifikation.Darstellung.OBJEKTELISTE_OBJEKTTYP_ATG_ASP_SIMUVAR);
				objects = dialogDatenIdent
						.oeffne(bestimmeVorhandeneObjektTypen(),
								Datenidentifikation.Darstellung.OBJEKTELISTE_OBJEKTTYP_ATG_ASP);
				Objektzusammenstellung.LOGGER
						.fine("ffnete eingeschrnkte ObjektTypen");

			} else {
				// objects = dialogDatenIdent
				// .oeffne(Darstellung.OBJEKTELISTE_OBJEKTTYP_ATG_ASP_SIMUVAR);
				objects = dialogDatenIdent
						.oeffne(Darstellung.OBJEKTELISTE_OBJEKTTYP_ATG_ASP);
				Objektzusammenstellung.LOGGER.fine("ffnete alle ObjektTypen");
			}

			Objektzusammenstellung.LOGGER.fine("Anzahl SystemObjekte: "
					+ objects.length);

			listeSWT.setVisible(false);
			for (final SystemObject object : objects) {
				listenObjekt = new ListenObjekt(object);
				listeDaten.add(listenObjekt);
				listeSWT.add(listenObjekt.getName());
			}
			listeSWT.setVisible(true);

			rueckgabeListeAngleichen();
		} else {
			final MessageBox box = new MessageBox(shell, SWT.OK
					| SWT.ICON_ERROR);
			box.setText("Keine Verbindung zum DaV");
			box.setMessage("Es wurde keine Verbindung zum DaV bergeben. \nDer Dialog zur Datenidentifikation wird nicht angezeigt.");
			box.open();
		}

	}

	// /**
	// * Fgt ein Array von Systemobjekten zur Liste hinzu
	// *
	// * @param objekt
	// * Array von Systemobjekten, dass zur Liste hinzugefgt werden soll
	// */
	// public void hinzufuegenSystemObjekte(SystemObject[] objekt)
	// {
	// ListenObjekt listenObjekt = null;
	// for (int i = 0; i < objekt.length; i++)
	// {
	// listenObjekt = new ListenObjekt();
	// listenObjekt.setSystemObject(objekt[i]);
	// _ListeDaten.add(listenObjekt);
	// _ListeSWT.add(listenObjekt.getName());
	// }
	// }

	// /**
	// * Setzt den Haken ObjekttypEinschrnken
	// *
	// * @param einschraenken
	// * true oder false
	// */
	// public void setObjekttypEinschraenken(boolean einschraenken)
	// {
	// if (einschraenken)
	// {
	// _checkBoxObjekttypEinschraenken.setEnabled(true);
	// } else
	// {
	// _checkBoxObjekttypEinschraenken.setEnabled(false);
	// }
	// }

	/**
	 * Verschiebt ein Element in der ListeObjekt/ListeSWT um eine Position nach
	 * oben.
	 * 
	 * @param index
	 *            Index des Elements, dass nach oben verschoben werden soll.
	 */
	private void verschiebeNachOben(final int index) {
		if (index == 0) {
			@SuppressWarnings("unused")
			boolean ok = false;
			final MessageBox box = new MessageBox(shell, SWT.OK
					| SWT.ICON_ERROR);
			box.setText("Fehler");
			box.setMessage("Das Element kann nicht nach oben verschoben werden, \ndenn es ist schon das letzte Element in der Liste.");
			if (box.open() == SWT.OK) {
				ok = true;
			}
		} else {
			// Verschieben
			// 1. In der ListeDaten
			final ListenObjekt tempListenObjekt = listeDaten.get(index);
			listeDaten.remove(index);
			listeDaten.add(index - 1, tempListenObjekt);

			// 2. In der ListeSWT
			final String tempListeSWTElement = listeSWT.getItem(index);
			listeSWT.remove(index);
			listeSWT.add(tempListeSWTElement, index - 1);

			// 3. Erneutes selektieren des verschobenen Elementes in ListeSWT
			listeSWT.select(index - 1);

		}

		rueckgabeListeAngleichen();
	}

	/**
	 * Verschiebt ein Element in der ListeObjekt/ListeSWT um eine Position nach
	 * unten.
	 * 
	 * @param index
	 *            Index des Elements, dass nach unten verschoben werden soll.
	 */
	private void verschiebeNachUnten(final int index) {
		if (index == (listeDaten.size() - 1)) {
			@SuppressWarnings("unused")
			boolean ok = false;
			final MessageBox box = new MessageBox(shell, SWT.OK
					| SWT.ICON_ERROR);
			box.setText("Fehler");
			box.setMessage("Das Element kann nicht nach unten verschoben werden, \ndenn es ist schon das letzte Element in der Liste.");
			if (box.open() == SWT.OK) {
				ok = true;
			}
		} else {
			// Verschieben
			// 1. In der ListeDaten
			final ListenObjekt tempListenObjekt = listeDaten.get(index);
			listeDaten.remove(index);
			listeDaten.add(index + 1, tempListenObjekt);

			// 2. In der ListeSWT
			final String tempListeSWTElement = listeSWT.getItem(index);
			listeSWT.remove(index);
			listeSWT.add(tempListeSWTElement, index + 1);

			// 3. Erneutes selektieren des verschobenen Elementes in ListeSWT
			listeSWT.select(index + 1);
		}

		rueckgabeListeAngleichen();
	}

	/*
	 * Hilfsfunktionen
	 */

	/**
	 * Bestimmt die in der Liste vorhandenen ObjektTypen und gibt deren Namen in
	 * einem Array von Strings zurck. Die wird bentigt wenn der der Haken fr
	 * die Einschrnkung gesetzt wird, dass nur Objekte vom gleichen Typ ber
	 * den Datenidentifiaktions- Dialog aufgenommen werden sollen.
	 */
	private String[] bestimmeVorhandeneObjektTypen() {
		// Rckgabe String
		String[] tempString = null;

		final TreeMap<String, String> tempMap = new TreeMap<String, String>();
		final Iterator<ListenObjekt> it = listeDaten.iterator();
		while (it.hasNext()) {
			final ListenObjekt listenObjekt = it.next();
			final SystemObject tempSO = listenObjekt.getSystemObject();
			final String insertKey = tempSO.getType().getPid();
			final String insertValue = tempSO.getType().getName();
			if (!tempMap.containsKey(insertKey)) {
				tempMap.put(insertKey, insertValue);
			}
		}

		if (tempMap.size() > 0) {
			int i = 0;
			tempString = new String[tempMap.size()];
			final Iterator<Entry<String, String>> itTempMap = tempMap
					.entrySet().iterator();
			while (itTempMap.hasNext()) {
				tempString[i] = itTempMap.next().getKey();
				i++;
			}
		}

		return tempString;
	}

	/**
	 * Fgt in die Darstellung die bergebenen SystemObjekte ein.
	 * 
	 */
	private void uebergabeListeAngleichen() {
		if (listeRueckgabeSystemObjekte != null) {
			final Iterator<SystemObject> it = listeRueckgabeSystemObjekte
					.iterator();
			while (it.hasNext()) {
				final SystemObject systemObject = it.next();
				final ListenObjekt neuesListenObjekt = new ListenObjekt(
						systemObject);
				listeDaten.add(neuesListenObjekt);
				listeSWT.add(neuesListenObjekt.getName());
			}
		}

	}

	/**
	 * Gleicht die RckgabeListe mit der hier gefhrten _ListeDaten an.
	 * 
	 */
	private void rueckgabeListeAngleichen() {
		if (listeDaten.size() > 0) {
			listeRueckgabeSystemObjekte.clear();
			final Iterator<ListenObjekt> it = listeDaten.iterator();
			while (it.hasNext()) {
				final ListenObjekt objekt = it.next();
				final SystemObject systemObject = objekt.getSystemObject();
				listeRueckgabeSystemObjekte.add(systemObject);
			}
		}

	}

	/**
	 * Reprsentiert die Objekte, die in listObjektListe angezeigt werden
	 * sollen, dient somit als Datencontainer.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class ListenObjekt {
		/**
		 * Name des Objektes, taucht damit auch i.d. Liste auf.
		 */
		private String name;

		/**
		 * Beschreibung des Objekts.
		 */
		private String beschreibung;

		/**
		 * Das SystemObject.
		 */
		private SystemObject systemObject;

		/*
		 * An dieser Stelle nicht vergessen, das eigentliche Objekt ber eine
		 * entsprechende Instanz zu "verlinken".
		 */

		public ListenObjekt(final SystemObject sysObj) {
			this.setSystemObject(sysObj);
		}

		/**
		 * Liefert den Namen des Objekts.
		 * 
		 * @return Objektname
		 */
		public String getName() {
			return name;
		}

		/**
		 * Setzt den Namen des Objekts.
		 * 
		 * @param name
		 *            Name des Objekts
		 */
		public void setName(final String name) {
			this.name = name;
		}

		/**
		 * Liefert die Beschreibung des Objekts.
		 * 
		 * @return
		 */
		public String getBeschreibung() {
			return beschreibung;
		}

		/**
		 * Setzt die Beschreibung des Objekts.
		 * 
		 * @param beschreibung
		 */
		public void setBeschreibung(final String beschreibung) {
			this.beschreibung = beschreibung;
		}

		/**
		 * Liefert das SystemObject dieser Klasse.
		 * 
		 * @return SystemObject
		 */
		public SystemObject getSystemObject() {
			return systemObject;
		}

		/**
		 * Setzt das Systemobject dieser Klasse. Gleichzeitig wird Name und
		 * Beschreibung fr die Darstellung in der Liste gesetzt.
		 * 
		 * @param systemObject
		 *            Zu setzendes Systemobjekt
		 */
		public void setSystemObject(final SystemObject systemObject) {
			this.systemObject = systemObject;
			// this.setName(systemObject.getName());
			String tempName = "";
			if ((systemObject.getName() != null)
					&& (!systemObject.getName().isEmpty())) {
				tempName = tempName + systemObject.getName() + " ";
			}
			if ((systemObject.getPid() != null)
					&& (!systemObject.getPid().isEmpty())) {
				tempName = tempName + "(" + systemObject.getPid() + ") ";
			}
			if (systemObject.getId() > 0) {
				tempName = tempName + "["
						+ String.valueOf(systemObject.getId()) + "]";
			}

			this.setName(tempName);
			this.setBeschreibung((systemObject.getInfo()).getShortInfo());
		}

	}

}
