/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse speichert das Element, das via Drag and Drop eingefgt werden
 * soll. <br>
 * Die realisierung erfolgt als Singleton.
 * 
 * @author HofmannJ
 * 
 */
public final class SpeicherTransfer {

	/** Der Debugger. */
	private static final Debug LOGGER = Debug.getLogger();

	/** speichert die Singleton-Instanz. */
	private static SpeicherTransfer instanz;

	/** speichert das {@link SystemObject} fr den DND-Vorgang. */
	private SystemObject speicherSystemObject;

	/** speichert die altePosition. */
	private int altePosition = -1;

	/** speichert die neue Position. */
	private int neuePosition = -1;

	/**
	 * liefert die Singleton-Instanz der Klasse {@link SpeicherTransfer}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link SpeicherTransfer}.
	 */
	public static SpeicherTransfer getInstanz() {
		if (SpeicherTransfer.instanz == null) {
			SpeicherTransfer.instanz = new SpeicherTransfer();
		}

		return SpeicherTransfer.instanz;
	}

	/**
	 * Privater Konstruktor.
	 * 
	 */
	private SpeicherTransfer() {
	}

	/**
	 * Setzt das zu speichernde {@link SystemObject}.
	 * 
	 * @param systemObject
	 *            Zu speicherndes {@link SystemObject}.
	 */
	public void setSystemObjekt(final SystemObject systemObject) {
		this.speicherSystemObject = systemObject;
	}

	/**
	 * Liefert das gespeicherte {@link SystemObject}.
	 * 
	 * @return Gespeichertes {@link SystemObject}.
	 */
	public SystemObject getSystemObjekt() {
		return speicherSystemObject;
	}

	/**
	 * Liefert die Pid des gespeicherten {@link SystemObject}.
	 * 
	 * @return Pid des gespeicherten {@link SystemObject}.
	 */
	public String getPid() {
		if (speicherSystemObject != null) {
			return speicherSystemObject.getPidOrNameOrId();
		}
		return null;
	}

	/**
	 * Setzt die Alte Position.
	 * 
	 * @param altePosition
	 */
	public void setAltePosition(final int altePosition) {
		this.altePosition = altePosition;
		SpeicherTransfer.LOGGER.fine("ALTE POSITION: " + altePosition);

	}

	public int getAltePosition() {
		return altePosition;
	}

	public void setNeuePosition(final int neuePosition) {
		this.neuePosition = neuePosition;
	}

	public int getNeuePosition() {
		return neuePosition;
	}

	/**
	 * Setzt den Speicher fr das {@link SystemObject} auf <code>null</code>
	 * zurck.
	 * 
	 */
	public void zuruecksetzenSpeicher() {
		speicherSystemObject = null;
		neuePosition = -1;
		altePosition = -1;
	}

}
