/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.List;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class ZugZuhoererSystemObjekt extends DragSourceAdapter {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Viewer, auf dem gearbeitet wird.
	 */
	private final Viewer tableViewer;

	/**
	 * Liste von Systemobjekten.
	 */
	private final List<SystemObject> listeDaten;

	/**
	 * Hlt die Singleton-Instanz der Klasse SpeicherTransfer vor.
	 */
	private SpeicherTransfer speicherTransfer;

	/**
	 * Konsturktor.
	 * 
	 * @param listViewer
	 * @param listeDaten
	 */
	public ZugZuhoererSystemObjekt(final Viewer tabelle,
			final List<SystemObject> listeDaten) {
		this.tableViewer = tabelle;
		this.listeDaten = listeDaten;
	}

	@Override
	public void dragStart(final DragSourceEvent event) {
		ZugZuhoererSystemObjekt.LOGGER.fine(getClass().getName()
				+ "::dragStart => Start");

		event.doit = !tableViewer.getSelection().isEmpty();
	}

	@Override
	public void dragFinished(final DragSourceEvent event) {
		if (!event.doit) {
			return;
		}
		// if the gadget was moved, remove it from the source viewer
		// if (event.detail == DND.DROP_MOVE) {
		// IStructuredSelection selection =
		// (IStructuredSelection)_viewer.getSelection();
		// for (Iterator it = selection.iterator(); it.hasNext();) {
		// //Entfernen der Elemente
		// }
		// _viewer.refresh();
		// }
		if (event.detail == DND.DROP_MOVE) {
			System.out.println(getClass().getName() + " => Lschen.");
			// IStructuredSelection selection
			// =(IStructuredSelection)_viewer.getSelection();
			// Iterator und aktualisieren der Elemente
			tableViewer.refresh();
		}

		ZugZuhoererSystemObjekt.LOGGER.fine(getClass().getName()
				+ " => dragFinished");

	}

	@Override
	public void dragSetData(final DragSourceEvent event) {
		ZugZuhoererSystemObjekt.LOGGER.fine(getClass().getName()
				+ "::dragSetData => begin.");

		final IStructuredSelection selection = (IStructuredSelection) tableViewer
				.getSelection();

		// SpeicherDNDTransfer.getInstanz().setAltePosition(_tableViewer.getList().getSelectionIndex());

		final Object selectedObject = selection.getFirstElement();
		speicherTransfer = SpeicherTransfer.getInstanz();
		speicherTransfer.setAltePosition(listeDaten.indexOf(selectedObject));

		// SystemObject?
		if (selectedObject instanceof SystemObject) {
			final SystemObject systemObject = (SystemObject) selectedObject;
			speicherTransfer.setSystemObjekt(systemObject);
			final String daten = "[PID]" + systemObject.getPidOrNameOrId();
			event.data = daten;
		}

	}
}
