/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort;

import java.util.IllegalFormatException;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.ort.FilterKnoten.Richtung;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.AsbKnotenKombiniert;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.BereichsInfoAsbKnotenKombiniert;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.EnumAsbVerkehrsRichtung;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Strasse;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenKnoten;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenSegment;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.debug.Debug;

public class OrtsangabeComposite extends Composite {

	/** der Debugger. */
	private static final Debug LOGGER = Debug.getLogger();

	private TabFolder pultOrdner;

	private Label labelStrassenSegmentStrasseLaengenwarnung;

	private Combo comboStrassenSegmentStrasse;

	/**
	 * {@link ComboViewer} fr Ortstangabe ber Straensegment - Straenauswahl.
	 */
	private ComboViewer cvStrassenSegmentStrasse;

	private Combo comboStrassenSegmentStrassenSegment;

	/**
	 * {@link ComboViewer} fr Ortstangabe ber Straensegment - StraenSegment
	 * auswahl.
	 */
	private ComboViewer cvStrassenSegmentStrassenSegment;

	private Text textStrassenSegmentKilometer;

	private Composite compositeBetriebskilometer;

	/**
	 * Dieses Label signalisiert, dass der angegebene Betriebskilometer nicht im
	 * LCL-Segment dieser Strae liegt und wird auch nur dann VISIBLE gesetzt.
	 */
	private Label labelBetriebskilometerWarnung;

	private Combo comboBetriebskilometerStrasse;

	/**
	 * ComboViewer der Straen der Betriebskilometer.
	 */
	private ComboViewer cvBetriebsKilometerStrasse;

	private Combo comboBetriebskilometerRichtung;

	private ComboViewer cvBetriebsKilometerRichtung;

	private Text textBetriebskilometerBetriebskilometer;

	/**
	 * Listener fr die Selektionen in der ComboBox Strassen der
	 * BetriebsKilometer.
	 */
	private ISelectionChangedListener listenerBetriebsKilometerStrasse;

	private ISelectionChangedListener listenerBetriebskilometerRichtung;

	/**
	 * Listener, der die Eingabe im Feld Betriebskilometer darauf hin prft, ob
	 * nur Zahlen eingetragen wurden.
	 */
	private Listener listenerTextBetriebskilometerBetriebskilometer;

	/**
	 * Listener, der die Eingabe im Feld Betriebskilometer Auswerter.
	 */
	private ModifyListener modifyListenerTextBetriebskilometerBetriebskilometer;

	private Combo comboBetriebskilometerBlocknummer;

	/**
	 * ComboViewer fr die BlockNummern.
	 */
	private ComboViewer cvBetriebsKilometerBlockNummer;

	/**
	 * Composite, auf dem die ASB-Ortsangabe aufgebaut wird.
	 */
	private Composite compositeAsbOrtsangabe;

	private Label labelAsbStrasse;

	private Label labelAsbVonNetzknoten;

	private Label labelAsbNachNetzknoten;

	private Label labelAsbStationierung;

	/**
	 * Label zum Anzeigen, dass sich der angegebene Stationierungswert nicht im
	 * gltigen Bereich befindet.
	 */
	private Label labelAsbStationierungAusserhalbBereich;

	/**
	 * Label Stationierungsrichtung der ASB-Ortsangabe.
	 */
	private Label labelAsbStationierungStationierungsRichtung;

	/**
	 * ComboBox zur Auswahl der Strae bei der ASB-Stationierung.
	 */
	private Combo comboAsbStrasse;

	/**
	 * {@link ComboViewer} zur Auswahl der Strae (ASB).
	 */
	private ComboViewer cvAsbStrasse;

	/**
	 * ComboBox zur Auswahl des Anfangsknotens bei der ASB-Stationierung.
	 */
	private Combo comboAsbAnfangsKnoten;

	/**
	 * {@link ComboViewer} zur Auswahl des Von-Netzknoten/Astes (ASB).
	 */
	private ComboViewer cvAsbAnfangsKnoten;

	/**
	 * ComboBox zur Auswahl des Endknotens bei der ASB-Stationierung.
	 */
	private Combo comboAsbEndKnoten;

	/**
	 * {@link ComboViewer} zur Auswahl des Nach-Netzknoten/Astes (ASB).
	 */
	private ComboViewer cvAsbEndKnoten;

	private Combo comboAsbStationierungsrichtung;
	private ComboViewer cvAsbStationierungsrichtung;

	/**
	 * Textfeld zum Eingeben der ASB-Stationierung in Metern.
	 */
	private Text textAsbStationierung;

	private Composite compositeKnotenOffset;

	private Composite compositeKnotenOffsetOben;

	private Group gruppeKnotenOffsetUnten;

	private Label labelKnotenOffsetStrasse;

	private Label labelKnotenOffsetKnoten;

	private Label labelKnotenOffsetKilometer;

	private Button radioButtonKnotenOffsetPositiv;

	private Button radioButtonKnotenOffsetNegativ;

	private Combo comboKnotenOffsetStrasse;

	/**
	 * {@link ComboViewer} fr comboKnotenOffsetStrasse.
	 */
	private ComboViewer cvKnotenOffsetStrasse;

	private Combo comboKnotenOffsetKnoten;

	/**
	 * {@link ComboViewer} fr comboKnotenOffsetKnoten.
	 */
	private ComboViewer cvKnotenOffsetKnoten;

	private Text textKnotenOffsetKilometer;

	/*
	 * TabBezeichner
	 */
	/**
	 * TabBezeichner Straensegment-Offset.
	 */
	private static final String BEZ_STRASSENSEGMENT_OFFSET = "Straensegment-Offset";

	/**
	 * TabBezeichner Betriebskilometer.
	 */
	private static final String BEZ_BETRIEBSKILOMETER = "Betriebskilometer";

	/**
	 * TabBezeichner ASB-Ortsangabe.
	 */
	private static final String BEZ_ASB_ORTSANGABE = "ASB-Ortsangabe";

	/**
	 * TabBezeichner Knoten-Offset.
	 */
	private static final String BEZ_KNOTEN_OFFSET = "Knoten-Offset";

	/**
	 * Composite zur Darstellung des Straensegments.
	 */
	private Composite pultStrassenSegment;

	/**
	 * Composite zur Darstellung der Betriebskilometerauswahl.
	 */
	private Composite pultBetriebskilometer;

	/**
	 * Composite zur Orstangabe ber ASB.
	 */
	private Composite pultAsbOrtsangabe;

	/**
	 * Composite zur Orstangabe ber einen Knoten.
	 */
	private Composite pultKnotenOffset;

	/**
	 * Speichert, welche Art der Ortsangabe gewhlt wurde.
	 */
	private OrtsangabeEnumeratorAuswahl auswahl = OrtsangabeEnumeratorAuswahl.strassensegment;

	/**
	 * Filter fr die Angabe ber Straensegment & Offset.
	 */
	private FilterStrassenSegmentOffset filterStrassenSegmentOffset;

	/**
	 * Filter fr die Angabe ber ASB.
	 */
	private FilterAsbOrtsangabe filterAsbOrtsangabe;

	/**
	 * Filter fr die Angabe ber Betriebskilometer.
	 */
	private FilterBetriebskilometer filterBetriebskilometer;

	/**
	 * Filter fr die Angabe ber Knoten.
	 */
	private FilterKnoten filterKnoten;

	/**
	 * Verbindung zum DaV.
	 */
	private final ClientDavInterface connection;

	private Composite compositeStrassenSegment;

	/**
	 * Liefert ein Composite zur Erfassung von Ortsangaben.
	 * 
	 * @param parent
	 *            Eltern-Composite fr die Darstellung
	 * @param style
	 *            SWT-Stil
	 * @param connection
	 *            Verbindung zum DaV.
	 */
	public OrtsangabeComposite(final Composite parent, final int style,
			final ClientDavInterface connection) {
		super(parent, style);
		this.connection = connection;
		initialize();
	}

	private void initialize() {
		setSize(new Point(300, 200));
		final GridLayout layout = new GridLayout();
		setLayout(layout);

		if (connection != null) {
			erzeugePultOrdner(this);
		} else {
			final Label labelInfo = new Label(this, SWT.NONE);
			labelInfo.setForeground(new Color(Display.getCurrent(), 255, 0, 0));
			labelInfo.setText("Es wurde keine Verbindung zum DaV bergeben!");
			erzeugePultOrdner(this);
		}

		fuellePultordner();

		/*
		 * "Selektieren" des ersten Tabs, damit sich die Strassen in diesem
		 * aktualisieren.
		 */
		selektiereAuswahlArt(OrtsangabeComposite.BEZ_STRASSENSEGMENT_OFFSET);

		this.pack(true);
	}

	/**
	 * Erzeugung des pultOrdner zur Auswahl der Ortsangaben.
	 * 
	 * @param elternComposite
	 *            {@link Composite}, auf dessen Basis die Darstellung erfolgen
	 *            soll.
	 */
	private void erzeugePultOrdner(final Composite elternComposite) {
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		pultOrdner = new TabFolder(elternComposite, SWT.NONE);
		pultOrdner.setLayoutData(gridData);

		pultOrdner.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				final String selektion = ((TabItem) (e.item)).getText();

				OrtsangabeComposite.LOGGER.fine("Selektiertes Tab: "
						+ selektion);

				selektiereAuswahlArt(selektion);

			}
		});
	}

	/**
	 * Selektiert einen {@link TabFolder} an Hand einer Instanz des Enumerators
	 * {@link OrtsangabeEnumeratorAuswahl}.
	 * 
	 * @param auswahlArt
	 *            {@link OrtsangabeEnumeratorAuswahl}
	 */
	private void selektierePultOrdner(
			final OrtsangabeEnumeratorAuswahl auswahlArt) {
		/*
		 * Selektion des PultOrdners
		 */
		switch (auswahlArt) {
		case strassensegment:
			pultOrdner.setSelection(0);
			selektiereAuswahlArt(OrtsangabeComposite.BEZ_STRASSENSEGMENT_OFFSET);
			break;
		case betriebskilometer:
			pultOrdner.setSelection(1);
			selektiereAuswahlArt(OrtsangabeComposite.BEZ_BETRIEBSKILOMETER);
			break;
		case asborstangabe:
			pultOrdner.setSelection(2);
			selektiereAuswahlArt(OrtsangabeComposite.BEZ_ASB_ORTSANGABE);
			break;
		case knotenoffset:
			pultOrdner.setSelection(3);
			selektiereAuswahlArt(OrtsangabeComposite.BEZ_KNOTEN_OFFSET);
			break;
		default:
			OrtsangabeComposite.LOGGER.fine("IMMER AUSGEFHRT");
			break;
		}

	}

	/**
	 * Bestimmt nach dem als Parameter bergebenen Tab die Auswahlart, die sich
	 * mit {@link OrtsangabeComposite#getAuswahlArt()} erfragen lsst. <br>
	 * 
	 * Die Filter werden dementsprechend selektiert.<br>
	 * 
	 * Diese Methode wird vom {@link SelectionAdapter} der Methode
	 * {@link OrtsangabeComposite#erzeugePultOrdner(Composite)} aufgerufen.
	 * 
	 * @param selektion
	 */
	private void selektiereAuswahlArt(final String selektion) {
		/*
		 * Straensegment-Offset
		 */
		if (selektion.equals(OrtsangabeComposite.BEZ_STRASSENSEGMENT_OFFSET)) {
			aktionenStrassenSegmentOffset();

		}

		/*
		 * Betriebskilometer
		 */
		if (selektion.equals(OrtsangabeComposite.BEZ_BETRIEBSKILOMETER)) {
			aktionenBetriebskilometer();

		}

		/*
		 * ASB-Ortsangabe
		 */
		if (selektion.equals(OrtsangabeComposite.BEZ_ASB_ORTSANGABE)) {
			// _auswahl = OrtsangabeEnumeratorAuswahl.asborstangabe;
			aktionenAsbOrtsangabe();
		}

		/*
		 * Knoten-Offset
		 */
		if (selektion.equals(OrtsangabeComposite.BEZ_KNOTEN_OFFSET)) {
			aktionenKnotenOffset();

		}
	}

	/**
	 * Versorgt die Elemente des Tabs Straensegment-Offset mit den notwendigen
	 * Funktionalitten.
	 */
	private void aktionenStrassenSegmentOffset() {
		auswahl = OrtsangabeEnumeratorAuswahl.strassensegment;
		if ((filterStrassenSegmentOffset == null) && (connection != null)) {
			filterStrassenSegmentOffset = new FilterStrassenSegmentOffset(
					(ClientDavConnection) connection);

			/*
			 * ComboViwer Straen
			 */
			if (cvStrassenSegmentStrasse == null) {
				cvStrassenSegmentStrasse = new ComboViewer(
						comboStrassenSegmentStrasse);
				cvStrassenSegmentStrasse
						.setContentProvider(new ArrayContentProvider());
				cvStrassenSegmentStrasse.setLabelProvider(new LabelProvider());
			}

			cvStrassenSegmentStrasse.setInput(filterStrassenSegmentOffset
					.getStrassen());

			/*
			 * SelectionListener Straen
			 */
			final ISelectionChangedListener selectionChangedListenerStrasse = new ISelectionChangedListener() {

				@Override
				public void selectionChanged(final SelectionChangedEvent event) {
					final ISelection selection = event.getSelection();
					if (selection instanceof StructuredSelection) {
						final StructuredSelection ss = (StructuredSelection) selection;
						final Object[] objects = ss.toArray();
						if (objects.length > 0) {
							final Object object = objects[0];
							if (object instanceof Strasse) {
								final Strasse strasse = (Strasse) object;
								filterStrassenSegmentOffset
										.setPidSelektierteStrasse(strasse
												.getPid());
								cvStrassenSegmentStrassenSegment
										.setInput(filterStrassenSegmentOffset
												.getStrassenSegmenteGefiltert(strasse
														.getPid()));

								// LinkedList<StrassenSegment>
								// strassenSegmenteGefiltert =
								// _filterStrassenSegmentOffset
								// .getStrassenSegmenteGefiltert(strasse.getPid());

							}
						}
					}
				}
			};

			cvStrassenSegmentStrasse
					.addSelectionChangedListener(selectionChangedListenerStrasse);

			/*
			 * ComboViewer StraenSegmente
			 */

			if (cvStrassenSegmentStrassenSegment == null) {
				cvStrassenSegmentStrassenSegment = new ComboViewer(
						comboStrassenSegmentStrassenSegment);
				cvStrassenSegmentStrassenSegment
						.setContentProvider(new ArrayContentProvider());
				cvStrassenSegmentStrassenSegment
						.setLabelProvider(new LabelProvider());
			}

			/*
			 * SelectionListener StraenSegmente
			 */
			final ISelectionChangedListener selectionChangedListenerStrassenSegmente = new ISelectionChangedListener() {

				@Override
				public void selectionChanged(final SelectionChangedEvent event) {
					final ISelection selection = event.getSelection();
					if (selection instanceof StructuredSelection) {
						final StructuredSelection ss = (StructuredSelection) selection;
						final Object[] objects = ss.toArray();
						if (objects.length > 0) {
							final Object object = objects[0];
							if (object instanceof StrassenSegment) {
								final StrassenSegment strassenSegment = (StrassenSegment) object;
								filterStrassenSegmentOffset
										.setPidSelektiertesStrassenSegment(strassenSegment
												.getPid());
							}
						}
					}

				}

			};

			cvStrassenSegmentStrassenSegment
					.addSelectionChangedListener(selectionChangedListenerStrassenSegmente);

			/*
			 * TEXTFELD
			 */
			final Listener listenerEingabe = new Listener() {

				@Override
				public void handleEvent(final Event e) {
					final String string = e.text;
					final char[] chars = new char[string.length()];
					string.getChars(0, chars.length, chars, 0);
					for (int i = 0; i < chars.length; i++) {
						if (!((('0' <= chars[i]) && (chars[i] <= '9')) || (',' == chars[i]))) {
							e.doit = false;
							return;
						}

						// Es wird die Eingabe mehrerer Kommas verhindert
						final String text = textStrassenSegmentKilometer
								.getText() + string;
						final int first = text.indexOf(",");
						final int last = text.lastIndexOf(",");
						OrtsangabeComposite.LOGGER.fine(text + " -> " + first
								+ " : " + last);
						if (first != last) {
							e.doit = false;
							return;
						}
					}

				}

			};

			textStrassenSegmentKilometer.addListener(SWT.Verify,
					listenerEingabe);

			textStrassenSegmentKilometer
					.addModifyListener(new ModifyListener() {

						@Override
						public void modifyText(final ModifyEvent e) {
							// Long l = new Long(0);
							Float f;

							if (textStrassenSegmentKilometer.getText().length() > 0) {
								// l =
								// Long.valueOf(textStrassenSegmentKilometer.getText());

								final String text = textStrassenSegmentKilometer
										.getText();

								f = Float.valueOf(text.replace(",", "."));

								// _filterStrassenSegmentOffset.setKm(l);
								filterStrassenSegmentOffset.setKm(f);
							} else {
								// l = new Long(0);
								f = new Float(0.0);
							}

							// _debug.finest(getClass().getName() + " LONG: " +
							// l.toString());
							OrtsangabeComposite.LOGGER.finest(getClass()
									.getName() + " FLOAT: " + f.toString());

							if (filterStrassenSegmentOffset != null) {
								final ISelection selection = cvStrassenSegmentStrassenSegment
										.getSelection();
								if (selection instanceof StructuredSelection) {
									final StructuredSelection s = (StructuredSelection) selection;
									final Object firstElement = s
											.getFirstElement();
									if (firstElement instanceof StrassenSegment) {
										final StrassenSegment strassenSegment = (StrassenSegment) firstElement;
										// if
										// (_filterStrassenSegmentOffset.pruefeKilometerSegment(l,
										// strassenSegment.getPid()))
										if (filterStrassenSegmentOffset
												.pruefeKilometerSegment(f,
														strassenSegment
																.getPid())) {
											// _filterStrassenSegmentOffset.setKm(l);
											filterStrassenSegmentOffset
													.setKm(f);
											labelStrassenSegmentStrasseLaengenwarnung
													.setText(" ");
											labelStrassenSegmentStrasseLaengenwarnung
													.pack(true);
										} else {
											OrtsangabeComposite.LOGGER
													.fine(getClass().getName()
															+ " => KM passt nicht ins Segment");

											labelStrassenSegmentStrasseLaengenwarnung
													.setForeground(Display
															.getDefault()
															.getSystemColor(
																	SWT.COLOR_RED));
											labelStrassenSegmentStrasseLaengenwarnung
													.setText("Die Maximallnge des Segments wurde berschritten!");
											labelStrassenSegmentStrasseLaengenwarnung
													.pack(true);
										}
									}
								}

							}

						}

					});

		}
	}

	/**
	 * Versorgt die Elemente des Tabs Betriebskilometer mit den notwendigen
	 * Funktionalitten.
	 */
	private void aktionenBetriebskilometer() {
		auswahl = OrtsangabeEnumeratorAuswahl.betriebskilometer;

		if (filterBetriebskilometer == null) {
			filterBetriebskilometer = new FilterBetriebskilometer(
					(ClientDavConnection) connection);
		}

		/*
		 * ComboViewer fr die Strae
		 */
		if (cvBetriebsKilometerStrasse == null) {
			cvBetriebsKilometerStrasse = new ComboViewer(
					comboBetriebskilometerStrasse);
			cvBetriebsKilometerStrasse
					.setContentProvider(new ArrayContentProvider());
			cvBetriebsKilometerStrasse.setLabelProvider(new LabelProvider());

			final List<Strasse> strassen = filterBetriebskilometer
					.getStrassen();
			cvBetriebsKilometerStrasse.setInput(strassen);
		}

		/*
		 * Betriebskilometer fr die Strae erstellen
		 */
		if (listenerBetriebsKilometerStrasse == null) {

			listenerBetriebsKilometerStrasse = new ISelectionChangedListener() {
				@Override
				public void selectionChanged(final SelectionChangedEvent event) {
					final ISelection selection = event.getSelection();
					if (selection instanceof StructuredSelection) {
						final StructuredSelection ss = (StructuredSelection) selection;
						final List<Object> list = ss.toList();
						if (list.size() > 0) {
							final Object object = list.get(0);
							OrtsangabeComposite.LOGGER.fine("Object: "
									+ object.getClass().getName());
							if (object instanceof Strasse) {
								final Strasse strasseBk = (Strasse) object;
								filterBetriebskilometer
										.setSelektierteStrasse(strasseBk
												.getPid());

								cvBetriebsKilometerRichtung.setSelection(null);

								if ((strasseBk.getNummer() % 2) == 0) {
									// Gerade == OST + West
									final LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung> richtung = new LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung>();

									final de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung west = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.WEST;
									final de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung ost = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.OST;

									richtung.add(west);
									richtung.add(ost);
									cvBetriebsKilometerRichtung
											.setInput(richtung);
								} else {
									// Ungerade == NORD + SD

									final LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung> richtung = new LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung>();

									final de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung nord = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.NORD;
									final de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung sued = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.SD;

									richtung.add(nord);
									richtung.add(sued);

									cvBetriebsKilometerRichtung
											.setInput(richtung);
								}

								textBetriebskilometerBetriebskilometer
										.setText("");
								cvBetriebsKilometerBlockNummer
										.setSelection(null);

							}
						}
					}
				}

			};

			cvBetriebsKilometerStrasse
					.addSelectionChangedListener(listenerBetriebsKilometerStrasse);
		}

		/*
		 * ComboViewer fr die Richtung
		 */

		if (cvBetriebsKilometerRichtung == null) {
			cvBetriebsKilometerRichtung = new ComboViewer(
					comboBetriebskilometerRichtung);
			cvBetriebsKilometerRichtung
					.setContentProvider(new ArrayContentProvider());
			cvBetriebsKilometerRichtung.setLabelProvider(new LabelProvider());

			final LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung> richtung = new LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung>();

			final de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung nord = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.NORD;
			final de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung sued = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.SD;
			final de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung west = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.WEST;
			final de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung ost = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.OST;

			richtung.add(nord);
			richtung.add(sued);
			richtung.add(west);
			richtung.add(ost);
			cvBetriebsKilometerRichtung.setInput(richtung);
		}

		/*
		 * Listener fr nderungen in der ComboBox Richtung erstellen
		 */
		if (listenerBetriebskilometerRichtung == null) {
			listenerBetriebskilometerRichtung = new ISelectionChangedListener() {

				@Override
				public void selectionChanged(final SelectionChangedEvent event) {
					final ISelection selection = event.getSelection();
					if (selection instanceof StructuredSelection) {
						final StructuredSelection ss = (StructuredSelection) selection;
						final Object[] objects = ss.toArray();
						if (objects.length > 0) {
							final Object object = objects[0];
							if (object instanceof de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung) {
								final de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung ri = (de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung) object;
								// _debug.fine("Richtung: " + ri.toString());
								filterBetriebskilometer
										.setSelektierteRichtung(ri);
							}
						}
					}

				}

			};

			cvBetriebsKilometerRichtung
					.addSelectionChangedListener(listenerBetriebskilometerRichtung);
		}

		/*
		 * Prfung der Eingabe der Betriebskilometer. Es knnen nur ZAHLEN
		 * eingegeben werden.
		 */
		if (listenerTextBetriebskilometerBetriebskilometer == null) {
			listenerTextBetriebskilometerBetriebskilometer = new Listener() {

				@Override
				public void handleEvent(final Event e) {
					final String string = e.text;
					final char[] chars = new char[string.length()];
					string.getChars(0, chars.length, chars, 0);
					for (int i = 0; i < chars.length; i++) {
						if (!(('0' <= chars[i]) && (chars[i] <= '9'))) {
							e.doit = false;
							return;
						}
					}

				}

			};

			textBetriebskilometerBetriebskilometer.addListener(SWT.Verify,
					listenerTextBetriebskilometerBetriebskilometer);

		}

		/*
		 * ComboViewer fr die Combobox BetriebsKilometerBlockNummer
		 */
		if (cvBetriebsKilometerBlockNummer == null) {
			cvBetriebsKilometerBlockNummer = new ComboViewer(
					comboBetriebskilometerBlocknummer);
			cvBetriebsKilometerBlockNummer
					.setContentProvider(new ArrayContentProvider());
			cvBetriebsKilometerBlockNummer
					.setLabelProvider(new LabelProvider());

			cvBetriebsKilometerBlockNummer
					.addSelectionChangedListener(new ISelectionChangedListener() {

						@Override
						public void selectionChanged(
								final SelectionChangedEvent event) {
							final ISelection selection = event.getSelection();
							if (selection instanceof StructuredSelection) {
								final StructuredSelection structuredSelection = (StructuredSelection) selection;
								final Object object = structuredSelection
										.getFirstElement();
								if (object != null) {
									if (object instanceof String) {
										final String wert = (String) object;
										filterBetriebskilometer
												.setSelektierteBlockNummer(wert);

										// Ausblenden der Warnung, dass man
										// etwas selektieren muss.
										labelBetriebskilometerWarnung
												.setVisible(false);
									}
								}
							}

						}

					});

		}

		/*
		 * Listener zur bernahme des Genderten Textes im Feld
		 * Betriebskilometer
		 */
		if (modifyListenerTextBetriebskilometerBetriebskilometer == null) {
			modifyListenerTextBetriebskilometerBetriebskilometer = new ModifyListener() {

				@Override
				public void modifyText(final ModifyEvent e) {
					final String text = textBetriebskilometerBetriebskilometer
							.getText();

					if ((text != null) && (text.length() > 0)) {
						final long bkWert = Long.valueOf(text);

						filterBetriebskilometer.setBetriebskilometer(bkWert);

						final List<String> vorhandeneBlockNummern = filterBetriebskilometer
								.getVorhandeneBlockNummern();

						cvBetriebsKilometerBlockNummer
								.setInput(vorhandeneBlockNummern);
						if (comboBetriebskilometerBlocknummer.getItemCount() == 1) {

							cvBetriebsKilometerBlockNummer
									.setSelection(new StructuredSelection(
											cvBetriebsKilometerBlockNummer
													.getElementAt(0)));
						}

						// Einblenden des Warnhinweises, wenn der
						// Betriebskilometer nicht
						// existiert.
						if (comboBetriebskilometerBlocknummer.getItemCount() < 1) {
							comboBetriebskilometerBlocknummer.setEnabled(false);
							labelBetriebskilometerWarnung
									.setText("Der angegebene Betriebskilometer ist im LCL-Segment der Strae nicht vorhanden!");
							labelBetriebskilometerWarnung.setVisible(true);

						} else if (comboBetriebskilometerBlocknummer
								.getItemCount() > 1) {
							comboBetriebskilometerBlocknummer.setEnabled(true);
							labelBetriebskilometerWarnung
									.setText("Es stehen mehrere Blocknummern zur Auswahl. Sie mssen einen auswhlen.");
							labelBetriebskilometerWarnung.setVisible(true);
						} else {
							comboBetriebskilometerBlocknummer.setEnabled(false);
							labelBetriebskilometerWarnung.setVisible(false);
						}
					}
				}

			};

			textBetriebskilometerBetriebskilometer
					.addModifyListener(modifyListenerTextBetriebskilometerBetriebskilometer);
		}

	}

	/**
	 * Versorgt die Elemente des Tabs ASB-Ortsangabe mit den notwendigen
	 * Funktionalitten.
	 */
	private void aktionenAsbOrtsangabe() {
		auswahl = OrtsangabeEnumeratorAuswahl.asborstangabe;

		if (filterAsbOrtsangabe == null) {
			filterAsbOrtsangabe = new FilterAsbOrtsangabe(
					(ClientDavConnection) connection);
		}

		/*
		 * ComboViewer fr ASB Strae
		 */
		if (cvAsbStrasse == null) {
			cvAsbStrasse = new ComboViewer(comboAsbStrasse);
			cvAsbStrasse.setContentProvider(new ArrayContentProvider());
			cvAsbStrasse.setLabelProvider(new LabelProvider());

			final List<Strasse> strassen = filterAsbOrtsangabe.getStrassen();
			cvAsbStrasse.setInput(strassen);
		}

		/*
		 * Listener fr den ComboViewer der Strae
		 */
		final ISelectionChangedListener listenerAsbStrasse = new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection) {
					final StructuredSelection ss = (StructuredSelection) selection;
					final List<Object> list = ss.toList();
					if (list.size() > 0) {
						final Object object = list.get(0);
						if (object instanceof Strasse) {
							final Strasse strasseAsb = (Strasse) object;
							OrtsangabeComposite.LOGGER.fine("PID: "
									+ strasseAsb.getPid());
							filterAsbOrtsangabe
									.setSelektierteStrasse(strasseAsb.getPid());

							final List<AsbKnotenKombiniert> verfuegbareAsbKnoten = filterAsbOrtsangabe
									.getAsbOrtsangabe()
									.getVonAuswahlfeldKombiniert();

							cvAsbAnfangsKnoten.setInput(verfuegbareAsbKnoten);

							cvAsbAnfangsKnoten.setSelection(null);
							cvAsbEndKnoten.setSelection(null);
							textAsbStationierung.setText("");
							labelAsbStationierungAusserhalbBereich
									.setVisible(false);
						}
					}
				}

			}

		};

		cvAsbStrasse.addSelectionChangedListener(listenerAsbStrasse);

		/*
		 * VON-Netrknoten/Ast
		 */
		if (cvAsbAnfangsKnoten == null) {
			cvAsbAnfangsKnoten = new ComboViewer(comboAsbAnfangsKnoten);
			cvAsbAnfangsKnoten.setContentProvider(new ArrayContentProvider());
			cvAsbAnfangsKnoten.setLabelProvider(new LabelProvider());
		}

		final ISelectionChangedListener listenerAsbVon = new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection) {
					final StructuredSelection ss = (StructuredSelection) selection;
					final List<Object> list = ss.toList();
					if (list.size() > 0) {
						final Object object = list.get(0);

						if (object instanceof AsbKnotenKombiniert) {
							final AsbKnotenKombiniert asbVonKnotenKombiniert = (AsbKnotenKombiniert) object;
							filterAsbOrtsangabe
									.setAnfangsKnoten(asbVonKnotenKombiniert
											.getKnoten());

							final List<AsbKnotenKombiniert> erreichbareAsbKnoten = filterAsbOrtsangabe
									.getAsbOrtsangabe()
									.bestimmeErreichbareAsbKnotenKombiniert(
											asbVonKnotenKombiniert)
									.getListeKombinierteKnoten();

							cvAsbEndKnoten.setInput(erreichbareAsbKnoten);
							cvAsbEndKnoten.setSelection(null);
							textAsbStationierung.setText("");
							labelAsbStationierungAusserhalbBereich
									.setVisible(false);
						}
					}
				}
			}

		};

		cvAsbAnfangsKnoten.addSelectionChangedListener(listenerAsbVon);

		/*
		 * Nach-Netzknoten/Ast
		 */
		if (cvAsbEndKnoten == null) {
			cvAsbEndKnoten = new ComboViewer(comboAsbEndKnoten);
			cvAsbEndKnoten.setContentProvider(new ArrayContentProvider());
			cvAsbEndKnoten.setLabelProvider(new LabelProvider());
		}

		final ISelectionChangedListener listenerAsbNach = new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection) {
					final StructuredSelection ss = (StructuredSelection) selection;
					final List<Object> list = ss.toList();
					if (list.size() > 0) {
						final Object object = list.get(0);

						if (object instanceof AsbKnotenKombiniert) {
							final AsbKnotenKombiniert endKnoten = (AsbKnotenKombiniert) object;

							filterAsbOrtsangabe.setEndKnoten(endKnoten
									.getKnoten());
							textAsbStationierung.setText("");
							labelAsbStationierungAusserhalbBereich
									.setVisible(false);
						}

					}
				}
			}

		};

		cvAsbEndKnoten.addSelectionChangedListener(listenerAsbNach);

		/*
		 * Sicherstellen, dass im TextFeld textAsbStationierung nur Zahlen
		 * eingegeben werden
		 */
		final Listener listenerTextAsbStationierung = new Listener() {

			@Override
			public void handleEvent(final Event e) {
				final String string = e.text;
				final char[] chars = new char[string.length()];
				string.getChars(0, chars.length, chars, 0);
				for (int i = 0; i < chars.length; i++) {
					if (!(('0' <= chars[i]) && (chars[i] <= '9'))) {
						e.doit = false;
						return;
					}
				}

			}
		};

		textAsbStationierung.addListener(SWT.Verify,
				listenerTextAsbStationierung);

		final ModifyListener modifyListenerTextAsbStationierung = new ModifyListener() {

			@Override
			public void modifyText(final ModifyEvent e) {
				pruefeAsbStationierungWaehrendTextfeldEingabe();

			}
		};

		textAsbStationierung
				.addModifyListener(modifyListenerTextAsbStationierung);

		if (cvAsbStationierungsrichtung == null) {
			cvAsbStationierungsrichtung = new ComboViewer(
					comboAsbStationierungsrichtung);
			cvAsbStationierungsrichtung
					.setContentProvider(new ArrayContentProvider());
			cvAsbStationierungsrichtung.setLabelProvider(new LabelProvider());
			final LinkedList<EnumAsbVerkehrsRichtung> asbVerkehrsRichtungen = new LinkedList<EnumAsbVerkehrsRichtung>();
			asbVerkehrsRichtungen
					.add(EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG);
			asbVerkehrsRichtungen
					.add(EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG);
			asbVerkehrsRichtungen.add(EnumAsbVerkehrsRichtung.BEIDERICHTUNGEN);
			asbVerkehrsRichtungen.add(EnumAsbVerkehrsRichtung.UNBEKANNT);
			cvAsbStationierungsrichtung.setInput(asbVerkehrsRichtungen);

			cvAsbStationierungsrichtung.setSelection(new StructuredSelection(
					EnumAsbVerkehrsRichtung.UNBEKANNT));

			final ISelectionChangedListener listenerAsbStationierung = new ISelectionChangedListener() {
				@Override
				public void selectionChanged(final SelectionChangedEvent event) {
					final ISelection selection = event.getSelection();
					if (selection instanceof StructuredSelection) {
						final StructuredSelection ss = (StructuredSelection) selection;
						final Object firstElement = ss.getFirstElement();
						if (firstElement instanceof EnumAsbVerkehrsRichtung) {
							final EnumAsbVerkehrsRichtung richtung = (EnumAsbVerkehrsRichtung) firstElement;
							filterAsbOrtsangabe
									.setAsbVerkehrsRichtung(richtung);
							pruefeAsbStationierung(richtung);

						}
					}
				}

			};
			cvAsbStationierungsrichtung
					.addSelectionChangedListener(listenerAsbStationierung);

		}
	}

	private void pruefeAsbStationierungWaehrendTextfeldEingabe() {
		final ISelection selection = cvAsbStationierungsrichtung.getSelection();
		if (selection instanceof StructuredSelection) {
			final StructuredSelection ss = (StructuredSelection) selection;
			final Object firstElement = ss.getFirstElement();
			if (firstElement instanceof EnumAsbVerkehrsRichtung) {
				final EnumAsbVerkehrsRichtung richtung = (EnumAsbVerkehrsRichtung) firstElement;
				pruefeAsbStationierung(richtung);
			}
		}

	}

	private void pruefeAsbStationierung(final EnumAsbVerkehrsRichtung richtung) {
		final String text = textAsbStationierung.getText();

		if ((text != null) && (text.length() > 0)) {

			try {
				final long wert = Long.valueOf(text);

				AsbKnotenKombiniert asbKnotenAnfangStationierung = null;
				AsbKnotenKombiniert asbKnotenEndeStationierung = null;

				final StructuredSelection ssAnfang = (StructuredSelection) cvAsbAnfangsKnoten
						.getSelection();
				final Object firstElementAnfang = ssAnfang.getFirstElement();
				if ((firstElementAnfang != null)
						&& (firstElementAnfang instanceof AsbKnotenKombiniert)) {
					asbKnotenAnfangStationierung = (AsbKnotenKombiniert) firstElementAnfang;
				}

				final StructuredSelection ssEnde = (StructuredSelection) cvAsbEndKnoten
						.getSelection();
				final Object firstElementEnde = ssEnde.getFirstElement();
				if ((firstElementEnde != null)
						&& (firstElementEnde instanceof AsbKnotenKombiniert)) {
					asbKnotenEndeStationierung = (AsbKnotenKombiniert) firstElementEnde;
				}

				/*
				 * Prfen des Wertes im Stationierungsfeld, wenn Anfangs und
				 * Endknoten ausgewhlt wurden.
				 */
				if ((asbKnotenAnfangStationierung != null)
						&& (asbKnotenEndeStationierung != null)) {
					final BereichsInfoAsbKnotenKombiniert bereichsInfoStationierungKombiniert = filterAsbOrtsangabe
							.getAsbOrtsangabe()
							.pruefeAsbStationierungKombiniert(
									asbKnotenAnfangStationierung,
									asbKnotenEndeStationierung, wert);

					if (richtung
							.equals(EnumAsbVerkehrsRichtung.BEIDERICHTUNGEN)) {
						final boolean[] erg = bereichsInfoStationierungKombiniert
								.istInBereichGefiltert(richtung);
						if (erg.length == 2) {
							final boolean inStatRichtung = erg[0];
							final boolean gegenStatRichung = erg[1];

							if (inStatRichtung || gegenStatRichung) {
								filterAsbOrtsangabe.setStationierung(wert);
								labelAsbStationierungAusserhalbBereich
										.setVisible(false);
							} else {
								final long[] minMax = FilterAsbOrtsangabe
										.ermittleMinMax(
												bereichsInfoStationierungKombiniert
														.getBereichsInfoAsbKnotenTmcPos(),
												bereichsInfoStationierungKombiniert
														.getBereichsInfoAsbKnotenTmcNeg(),
												richtung);

								if (minMax != null) {
									labelAsbStationierungAusserhalbBereich
											.setText("Die Stationierung muss zwischen "
													+ minMax[0]
													+ " und "
													+ minMax[1] + " m liegen!");

								} else {
									labelAsbStationierungAusserhalbBereich
											.setText("Bereich fr Stationierung nicht ermittelbar!");
								}

								labelAsbStationierungAusserhalbBereich.pack();
								labelAsbStationierungAusserhalbBereich
										.setVisible(true);

							}

						}
					} else if ((richtung
							.equals(EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG))
							|| (richtung
									.equals(EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG))) {
						auswertungRichtung(bereichsInfoStationierungKombiniert,
								richtung, wert);

					} else if (richtung
							.equals(EnumAsbVerkehrsRichtung.UNBEKANNT)) {
						if (bereichsInfoStationierungKombiniert.istInBereich()) {
							filterAsbOrtsangabe.setStationierung(wert);
							labelAsbStationierungAusserhalbBereich
									.setVisible(false);
						} else {
							final long[] minMax = FilterAsbOrtsangabe
									.ermittleMinMax(
											bereichsInfoStationierungKombiniert
													.getBereichsInfoAsbKnotenTmcPos(),
											bereichsInfoStationierungKombiniert
													.getBereichsInfoAsbKnotenTmcNeg(),
											richtung);

							if (minMax != null) {
								labelAsbStationierungAusserhalbBereich
										.setText("Die Stationierung muss zwischen "
												+ minMax[0]
												+ " und "
												+ minMax[1] + " m liegen!");

							} else {
								labelAsbStationierungAusserhalbBereich
										.setText("Bereich fr Stationierung nicht ermittelbar!");
							}

							labelAsbStationierungAusserhalbBereich.pack();
							labelAsbStationierungAusserhalbBereich
									.setVisible(true);

						}

					}

				}

			} catch (final IllegalFormatException ife) {
				ife.printStackTrace();
			}

		}
	}

	/**
	 * bernimmt die Auswertung der Richtung fr
	 * {@link EnumAsbVerkehrsRichtung#INSTATIONIERUNGSRICHTUNG} und
	 * {@link EnumAsbVerkehrsRichtung#GEGENSTATIONIERUNGSRICHTUNG}.
	 * 
	 * @param bereichsInfoStationierungKombiniert
	 *            Kombinierte {@link BereichsInfoAsbKnotenKombiniert} fr die
	 *            AsbKnoten in positiver und negativer TMC-Richtung.
	 * @param richtung
	 *            Zu prfende {@link EnumAsbVerkehrsRichtung}. Funktioniert nur
	 *            fr {@link EnumAsbVerkehrsRichtung#INSTATIONIERUNGSRICHTUNG}
	 *            und
	 *            {@link EnumAsbVerkehrsRichtung#GEGENSTATIONIERUNGSRICHTUNG}.
	 * @param wertStationierung
	 *            Wert der Stationierung in Metern.
	 */
	private void auswertungRichtung(
			final BereichsInfoAsbKnotenKombiniert bereichsInfoStationierungKombiniert,
			final EnumAsbVerkehrsRichtung richtung, final long wertStationierung) {
		// String msgKeineBestimmung = "";

		final boolean[] erg = bereichsInfoStationierungKombiniert
				.istInBereichGefiltert(richtung);
		if (erg.length == 2) {
			final boolean inBereichTmcPos = erg[0];
			final boolean inBereichTmcNeg = erg[1];

			if (inBereichTmcPos || inBereichTmcNeg) {
				filterAsbOrtsangabe.setStationierung(wertStationierung);
				labelAsbStationierungAusserhalbBereich.setVisible(false);
			} else {

				if (!inBereichTmcPos && !inBereichTmcNeg) {
					final long[] minMax = FilterAsbOrtsangabe
							.ermittleMinMax(bereichsInfoStationierungKombiniert
									.getBereichsInfoAsbKnotenTmcPos(),
									bereichsInfoStationierungKombiniert
											.getBereichsInfoAsbKnotenTmcNeg(),
									richtung);

					String labelText = null;
					if (minMax != null) {
						labelText = "Die Stationierung muss zwischen "
								+ minMax[0] + " und " + minMax[1]
								+ " m liegen!";
					} else {
						labelText = "Es ist keine Bereichsbestimmung mglich da keine Infos in \""
								+ richtung + "\" vorhanden.";

					}

					labelAsbStationierungAusserhalbBereich.setText(labelText);
					labelAsbStationierungAusserhalbBereich.pack();
					labelAsbStationierungAusserhalbBereich.setVisible(true);
				}

				String info = "";
				info = info + "inBereichTmcPos: " + inBereichTmcPos + "\n";
				info = info + "inBereichTmcNeg: " + inBereichTmcNeg + "\n";

				System.out.println(info);

			}

		}
	}

	/**
	 * Versorgt die Elemente des Tabs Knoten-Offset mit den notwendigen
	 * Funktionalitten.
	 */
	private void aktionenKnotenOffset() {
		auswahl = OrtsangabeEnumeratorAuswahl.knotenoffset;

		if ((filterKnoten == null) && (connection != null)) {
			filterKnoten = new FilterKnoten((ClientDavConnection) connection);
		}

		/*
		 * Straen versorgen
		 */
		if (cvKnotenOffsetStrasse == null) {

			cvKnotenOffsetStrasse = new ComboViewer(comboKnotenOffsetStrasse);
			cvKnotenOffsetStrasse
					.setContentProvider(new ArrayContentProvider());
			cvKnotenOffsetStrasse.setLabelProvider(new LabelProvider());

			cvKnotenOffsetStrasse.setInput(filterKnoten.getStrassen());

		}

		final ISelectionChangedListener listenerKnotenOffsetStrasse = new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection) {
					final StructuredSelection ss = (StructuredSelection) selection;
					final List<Object> list = ss.toList();
					if (list.size() > 0) {
						final Object object = list.get(0);
						if (object instanceof Strasse) {
							final Strasse strasseKnoten = (Strasse) object;
							filterKnoten.setPidStrasse(strasseKnoten.getPid());

							if (cvKnotenOffsetKnoten != null) {
								cvKnotenOffsetKnoten.setInput(filterKnoten
										.getKnoten(strasseKnoten.getPid()));
							}

						}
					}
				}

			}

		};

		cvKnotenOffsetStrasse
				.addSelectionChangedListener(listenerKnotenOffsetStrasse);

		/*
		 * Knoten versorgen
		 */
		if (cvKnotenOffsetKnoten == null) {
			cvKnotenOffsetKnoten = new ComboViewer(comboKnotenOffsetKnoten);

			cvKnotenOffsetKnoten.setContentProvider(new ArrayContentProvider());
			cvKnotenOffsetKnoten.setLabelProvider(new LabelProvider());
		}

		final ISelectionChangedListener listenerKnotenOffsetKnoten = new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection) {
					final StructuredSelection ss = (StructuredSelection) selection;
					final List<Object> list = ss.toList();
					if (list.size() > 0) {
						final Object object = list.get(0);
						if (object instanceof StrassenKnoten) {
							final StrassenKnoten knoten = (StrassenKnoten) object;
							filterKnoten.setPidKnoten(knoten.getPid());
						}
					}
				}
			}

		};

		cvKnotenOffsetKnoten
				.addSelectionChangedListener(listenerKnotenOffsetKnoten);

		/*
		 * Erfassen der Kilometer
		 */
		textKnotenOffsetKilometer.addListener(SWT.Verify, new Listener() {

			@Override
			public void handleEvent(final Event event) {

				final String text = event.text;
				if (text.length() > 0) {
					final char c = text.charAt(text.length() - 1);
					if (c != '.') {
						if (!(('0' <= c) && (c <= '9'))) {
							event.doit = false;
							return;
						}
					}

					/*
					 * Prfen auf mehrfaches vorkommen von "."
					 */
					if (c == '.') {
						final String textFeld = textKnotenOffsetKilometer
								.getText();

						int count = 0;
						for (int i = 0; i < textFeld.length(); i++) {
							OrtsangabeComposite.LOGGER.fine("T: "
									+ textFeld.charAt(i));

							if (textFeld.charAt(i) == '.') {
								count++;
							}
						}

						OrtsangabeComposite.LOGGER.fine("C: " + count);

						if (count >= 1) {
							event.doit = false;
							return;
						}
					}

				}
			}

		});

		textKnotenOffsetKilometer.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(final ModifyEvent e) {
				final String text = textKnotenOffsetKilometer.getText();
				final Float fl = Float.valueOf(text);
				filterKnoten.setKm(fl);
			}

		});

		/*
		 * Listener fr die Radiobuttons
		 */
		radioButtonKnotenOffsetPositiv
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						if (radioButtonKnotenOffsetPositiv.getSelection()) {

							OrtsangabeComposite.LOGGER.fine(getClass()
									.getName() + " => POSITIV SELECTED");

							if (filterKnoten != null) {
								filterKnoten.setRichtung(Richtung.positiv);
								OrtsangabeComposite.LOGGER
										.fine(getClass().getName()
												+ " => in FilterKnoten wurde Richtugn auf Richtung.positiv gesetzt.");

							}
						}
					}
				});

		radioButtonKnotenOffsetNegativ
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						if (radioButtonKnotenOffsetNegativ.getSelection()) {
							OrtsangabeComposite.LOGGER.fine(getClass()
									.getName() + " => NEGATIV SELECTED");

							if (filterKnoten != null) {
								filterKnoten.setRichtung(Richtung.negativ);

								OrtsangabeComposite.LOGGER
										.fine(getClass().getName()
												+ " => in FilterKnoten wurde Richtugn auf Richtung.negativ gesetzt.");

							}
						}

					}
				});

	}

	/**
	 * Erzeugung von compositeStrassenSegment.
	 * 
	 */
	private void erzeugeCompositeStrassenSegment() {
		final GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.verticalAlignment = GridData.CENTER;
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.verticalAlignment = GridData.FILL;
		compositeStrassenSegment = new Composite(pultStrassenSegment, SWT.NONE);
		compositeStrassenSegment.setLayoutData(gridData1);
		compositeStrassenSegment.setLayout(gridLayout);
		final Label labelStrassenSegmentStrasse = new Label(
				compositeStrassenSegment, SWT.NONE);
		labelStrassenSegmentStrasse.setText("Strae:");
		erzeugeComboStrassenSegmentStrasse();
		final Label labelStrassenSegmentStrassenSegment = new Label(
				compositeStrassenSegment, SWT.NONE);
		labelStrassenSegmentStrassenSegment.setText("Straensegment:");
		erzeugeComboStrassenSegmentStrassenSegment();
		final Label labelStrassenSegmentKilometer = new Label(
				compositeStrassenSegment, SWT.NONE);
		labelStrassenSegmentKilometer.setText("Kilometer:");
		textStrassenSegmentKilometer = new Text(compositeStrassenSegment,
				SWT.BORDER);
		textStrassenSegmentKilometer.setLayoutData(gridData4);

		final Label labelStrassenSegmentStrasseFueller = new Label(
				compositeStrassenSegment, SWT.NONE);
		labelStrassenSegmentStrasseFueller.setText("");

		labelStrassenSegmentStrasseLaengenwarnung = new Label(
				compositeStrassenSegment, SWT.NONE);
		labelStrassenSegmentStrasseLaengenwarnung.setLayoutData(gridData1);

	}

	/**
	 * Erzeugung von comboStrassenSegmentStrasse.
	 * 
	 */
	private void erzeugeComboStrassenSegmentStrasse() {
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.verticalAlignment = GridData.CENTER;
		comboStrassenSegmentStrasse = new Combo(compositeStrassenSegment,
				SWT.NONE);
		comboStrassenSegmentStrasse.setLayoutData(gridData2);

	}

	/**
	 * Erzeugung von comboStrassenSegmentStrassenSegment.
	 * 
	 */
	private void erzeugeComboStrassenSegmentStrassenSegment() {
		final GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.verticalAlignment = GridData.CENTER;
		comboStrassenSegmentStrassenSegment = new Combo(
				compositeStrassenSegment, SWT.NONE);
		comboStrassenSegmentStrassenSegment.setLayoutData(gridData3);

	}

	/**
	 * Erzeugung von compositeBetriebskilometer.
	 * 
	 */
	private void erzeugeCompositeBetriebskilometer() {
		final GridData gridData8 = new GridData();
		gridData8.horizontalAlignment = GridData.FILL;
		gridData8.grabExcessHorizontalSpace = true;
		gridData8.verticalAlignment = GridData.CENTER;
		final GridLayout gridLayout1 = new GridLayout();
		gridLayout1.numColumns = 2;
		final GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = GridData.FILL;
		gridData5.grabExcessHorizontalSpace = true;
		gridData5.grabExcessVerticalSpace = true;
		gridData5.verticalAlignment = GridData.FILL;
		compositeBetriebskilometer = new Composite(pultBetriebskilometer,
				SWT.NONE);
		compositeBetriebskilometer.setLayoutData(gridData5);
		compositeBetriebskilometer.setLayout(gridLayout1);
		final Label labelBetriebskilometerStrasse = new Label(
				compositeBetriebskilometer, SWT.NONE);
		labelBetriebskilometerStrasse.setText("Strae:");
		erzeugeComboBetriebskilometerStrasse();
		final Label labelBetriebskilometerRichtung = new Label(
				compositeBetriebskilometer, SWT.NONE);
		labelBetriebskilometerRichtung.setText("Richtung:");
		erzeugeComboBetriebskilometerRichtung();
		final Label labelBetriebskilometerBetriebskilometer = new Label(
				compositeBetriebskilometer, SWT.NONE);
		labelBetriebskilometerBetriebskilometer.setText("Betriebskilometer:");
		textBetriebskilometerBetriebskilometer = new Text(
				compositeBetriebskilometer, SWT.BORDER);
		textBetriebskilometerBetriebskilometer.setLayoutData(gridData8);
		textBetriebskilometerBetriebskilometer
				.setToolTipText("Hier knnen die Betriebskilometer in Metern eingegeben werden.");

		final Label labelBetriebskilometerBlocknummer = new Label(
				compositeBetriebskilometer, SWT.NONE);
		labelBetriebskilometerBlocknummer.setText("Blocknummer:");
		erzeugeComboBetriebskilometerBlocknummer();

		final Label dummyLabel = new Label(compositeBetriebskilometer, SWT.NONE);
		dummyLabel.setVisible(false);

		labelBetriebskilometerWarnung = new Label(compositeBetriebskilometer,
				SWT.NONE);

		labelBetriebskilometerWarnung.setForeground(new Color(Display
				.getCurrent(), 255, 0, 0));
		labelBetriebskilometerWarnung
				.setText("Der angegebene Betriebskilometer ist im LCL-Segment der Strae nicht vorhanden!");
		labelBetriebskilometerWarnung.setVisible(false);

	}

	/**
	 * Erzeugung von comboBetriebskilometerStrae.
	 * 
	 */
	private void erzeugeComboBetriebskilometerStrasse() {
		final GridData gridData6 = new GridData();
		gridData6.horizontalAlignment = GridData.FILL;
		gridData6.grabExcessHorizontalSpace = true;
		gridData6.verticalAlignment = GridData.CENTER;
		comboBetriebskilometerStrasse = new Combo(compositeBetriebskilometer,
				SWT.NONE);
		comboBetriebskilometerStrasse.setLayoutData(gridData6);

	}

	/**
	 * Erzeugung von comboBetriebskilometerRichtung.
	 * 
	 */
	private void erzeugeComboBetriebskilometerRichtung() {
		final GridData gridData7 = new GridData();
		gridData7.horizontalAlignment = GridData.FILL;
		gridData7.grabExcessHorizontalSpace = true;
		gridData7.verticalAlignment = GridData.CENTER;
		comboBetriebskilometerRichtung = new Combo(compositeBetriebskilometer,
				SWT.NONE);
		comboBetriebskilometerRichtung.setLayoutData(gridData7);
	}

	/**
	 * Erzeugung von comboBetriebskilometerBlocknummer.
	 * 
	 */
	private void erzeugeComboBetriebskilometerBlocknummer() {
		final GridData gridData9 = new GridData();
		gridData9.horizontalAlignment = GridData.FILL;
		gridData9.grabExcessHorizontalSpace = true;
		gridData9.verticalAlignment = GridData.CENTER;
		comboBetriebskilometerBlocknummer = new Combo(
				compositeBetriebskilometer, SWT.NONE);
		comboBetriebskilometerBlocknummer.setLayoutData(gridData9);
	}

	/**
	 * Erzeugung von compositeAsbOrtsangabe.
	 * 
	 */
	@SuppressWarnings("unused")
	private void erzeugeCompositeAsbOrtsangabe() {
		final GridData gridDataTextAsbStationierung = new GridData();
		gridDataTextAsbStationierung.horizontalAlignment = GridData.FILL;
		gridDataTextAsbStationierung.grabExcessHorizontalSpace = true;
		gridDataTextAsbStationierung.verticalAlignment = GridData.CENTER;
		final GridLayout gridLayoutAsbOrtsangabe = new GridLayout();
		gridLayoutAsbOrtsangabe.numColumns = 2;
		final GridData gridDataAsbOrtsangabe = new GridData();
		gridDataAsbOrtsangabe.horizontalAlignment = GridData.FILL;
		gridDataAsbOrtsangabe.grabExcessHorizontalSpace = true;
		gridDataAsbOrtsangabe.grabExcessVerticalSpace = true;
		gridDataAsbOrtsangabe.verticalAlignment = GridData.FILL;
		compositeAsbOrtsangabe = new Composite(pultAsbOrtsangabe, SWT.NONE);
		compositeAsbOrtsangabe.setLayoutData(gridDataAsbOrtsangabe);
		compositeAsbOrtsangabe.setLayout(gridLayoutAsbOrtsangabe);
		labelAsbStrasse = new Label(compositeAsbOrtsangabe, SWT.NONE);
		labelAsbStrasse.setText("Strae:");
		erzeugeComboAsbStrasse();
		labelAsbVonNetzknoten = new Label(compositeAsbOrtsangabe, SWT.NONE);
		labelAsbVonNetzknoten.setText("Von Netzknoten/Ast:");
		erzeugeComboAsbAnfangsKnoten();
		labelAsbNachNetzknoten = new Label(compositeAsbOrtsangabe, SWT.NONE);
		labelAsbNachNetzknoten.setText("Nach Netzknoten/Ast:");
		erzeugeComboAsbEndKnoten();
		labelAsbStationierung = new Label(compositeAsbOrtsangabe, SWT.NONE);
		labelAsbStationierung.setText("Stationierung:");

		textAsbStationierung = new Text(compositeAsbOrtsangabe, SWT.BORDER);
		textAsbStationierung.setLayoutData(gridDataTextAsbStationierung);
		textAsbStationierung
				.setToolTipText("Hier knnen Sie einen Wert fr die Stationierung "
						+ "eingeben (in Metern). Dieser wird\ndaraufhin geprft, ob er sich im gltigen "
						+ "Bereich der zuvor ausgewhlten Knoten befindet.");

		labelAsbStationierungStationierungsRichtung = new Label(
				compositeAsbOrtsangabe, SWT.NONE);
		labelAsbStationierungStationierungsRichtung
				.setText("Stationierungsrichtung:");
		erzeugeComboAsbStationierung();

		/*
		 * FIXME sollte auch mit Layoutdaten gehen? Dummy-Label zum fllen des
		 * freien Platzes
		 */
		new Label(compositeAsbOrtsangabe, SWT.NONE);

		/*
		 * Label zur Anzeige, dass sich die Stationierung auerhalb des gltigen
		 * Bereiches befindet.
		 */
		labelAsbStationierungAusserhalbBereich = new Label(
				compositeAsbOrtsangabe, SWT.NONE);
		labelAsbStationierungAusserhalbBereich.setForeground(new Color(
				PlatformUI.getWorkbench().getDisplay(), 255, 0, 0));
		labelAsbStationierungAusserhalbBereich
				.setText("Die angegebene Stationierung liegt auerhalb des gltigen Bereichs!");
		labelAsbStationierungAusserhalbBereich.setVisible(false);

		/*
		 * TabListe setzen
		 */
		final Control[] tabList = new Control[] { comboAsbStrasse,
				comboAsbAnfangsKnoten, comboAsbEndKnoten, textAsbStationierung };
		compositeAsbOrtsangabe.setTabList(tabList);
	}

	/**
	 * Erzeugung von comboAsbStrasse.
	 * 
	 */
	private void erzeugeComboAsbStrasse() {
		final GridData gridDataComboAsbStrasse = new GridData();
		gridDataComboAsbStrasse.horizontalAlignment = GridData.FILL;
		gridDataComboAsbStrasse.grabExcessHorizontalSpace = true;
		gridDataComboAsbStrasse.verticalAlignment = GridData.CENTER;
		comboAsbStrasse = new Combo(compositeAsbOrtsangabe, SWT.NONE);
		comboAsbStrasse
				.setToolTipText("Hier knnen Sie eine Strae auswhlen.");
		comboAsbStrasse.setLayoutData(gridDataComboAsbStrasse);
	}

	/**
	 * Erzeugung von comboAsbVonNetzknoten.
	 * 
	 */
	private void erzeugeComboAsbAnfangsKnoten() {
		final GridData gridDataAsbAnfangsKnoten = new GridData();
		gridDataAsbAnfangsKnoten.horizontalAlignment = GridData.FILL;
		gridDataAsbAnfangsKnoten.grabExcessHorizontalSpace = true;
		gridDataAsbAnfangsKnoten.verticalAlignment = GridData.CENTER;
		comboAsbAnfangsKnoten = new Combo(compositeAsbOrtsangabe, SWT.READ_ONLY);
		comboAsbAnfangsKnoten.setLayoutData(gridDataAsbAnfangsKnoten);
		comboAsbAnfangsKnoten
				.setToolTipText("Hier knnen Sie einen Anfangs Knoten auswhlen.");
	}

	/**
	 * Erzeugung von comboNachNetzknoten.
	 * 
	 */
	private void erzeugeComboAsbEndKnoten() {
		final GridData gridDataComboAsbEndknoten = new GridData();
		gridDataComboAsbEndknoten.horizontalAlignment = GridData.FILL;
		gridDataComboAsbEndknoten.grabExcessHorizontalSpace = true;
		gridDataComboAsbEndknoten.verticalAlignment = GridData.CENTER;
		comboAsbEndKnoten = new Combo(compositeAsbOrtsangabe, SWT.READ_ONLY);
		comboAsbEndKnoten.setLayoutData(gridDataComboAsbEndknoten);
		comboAsbEndKnoten
				.setToolTipText("Hier knnen Sie einen End Knoten auswhlen.");
	}

	private void erzeugeComboAsbStationierung() {
		final GridData gridDataComboAsbStationierung = new GridData();
		gridDataComboAsbStationierung.horizontalAlignment = GridData.FILL;
		gridDataComboAsbStationierung.grabExcessHorizontalSpace = true;
		gridDataComboAsbStationierung.verticalAlignment = GridData.CENTER;

		comboAsbStationierungsrichtung = new Combo(compositeAsbOrtsangabe,
				SWT.READ_ONLY);
		comboAsbStationierungsrichtung
				.setLayoutData(gridDataComboAsbStationierung);
	}

	/**
	 * Erzeugung von compositeKnotenOffset.
	 * 
	 */
	private void erzeugeCompositeKnotenOffset() {
		final GridData gridData15 = new GridData();
		gridData15.horizontalAlignment = GridData.FILL;
		gridData15.grabExcessHorizontalSpace = true;
		gridData15.grabExcessVerticalSpace = true;
		gridData15.verticalAlignment = GridData.BEGINNING;
		compositeKnotenOffset = new Composite(pultKnotenOffset, SWT.NONE);
		compositeKnotenOffset.setLayout(new GridLayout());
		erzeugeCompositeKnotenOffsetOben();
		compositeKnotenOffset.setLayoutData(gridData15);
		erzeugeGruppeKnotenOffsetUnten();
	}

	/**
	 * Erzeugung von compositeKnotenOffsetOben.
	 * 
	 */
	private void erzeugeCompositeKnotenOffsetOben() {
		final GridData gridData19 = new GridData();
		gridData19.horizontalAlignment = GridData.FILL;
		gridData19.grabExcessHorizontalSpace = true;
		gridData19.verticalAlignment = GridData.CENTER;
		final GridLayout gridLayout4 = new GridLayout();
		gridLayout4.numColumns = 2;
		final GridData gridData16 = new GridData();
		gridData16.horizontalAlignment = GridData.FILL;
		gridData16.grabExcessHorizontalSpace = true;
		gridData16.grabExcessVerticalSpace = true;
		gridData16.verticalAlignment = GridData.FILL;
		compositeKnotenOffsetOben = new Composite(compositeKnotenOffset,
				SWT.NONE);
		compositeKnotenOffsetOben.setLayoutData(gridData16);
		compositeKnotenOffsetOben.setLayout(gridLayout4);
		labelKnotenOffsetStrasse = new Label(compositeKnotenOffsetOben,
				SWT.NONE);
		labelKnotenOffsetStrasse.setText("Strae:");
		erzeugeComboKnotenOffsetStrasse();
		labelKnotenOffsetKnoten = new Label(compositeKnotenOffsetOben, SWT.NONE);
		labelKnotenOffsetKnoten.setText("Knoten:");
		erzeugeComboKnotenOffsetKnoten();
		labelKnotenOffsetKilometer = new Label(compositeKnotenOffsetOben,
				SWT.NONE);
		labelKnotenOffsetKilometer.setText("Kilometer:");
		textKnotenOffsetKilometer = new Text(compositeKnotenOffsetOben,
				SWT.BORDER);
		textKnotenOffsetKilometer.setLayoutData(gridData19);
	}

	/**
	 * Erzeugung von gruppeKnotenOffsetUnten.
	 * 
	 */
	private void erzeugeGruppeKnotenOffsetUnten() {
		gruppeKnotenOffsetUnten = new Group(compositeKnotenOffset, SWT.NONE);
		gruppeKnotenOffsetUnten.setLayout(new GridLayout());
		gruppeKnotenOffsetUnten.setText("Richtung");
		radioButtonKnotenOffsetPositiv = new Button(gruppeKnotenOffsetUnten,
				SWT.RADIO);
		radioButtonKnotenOffsetPositiv.setText("positiv");
		radioButtonKnotenOffsetPositiv.setSelection(true);
		radioButtonKnotenOffsetNegativ = new Button(gruppeKnotenOffsetUnten,
				SWT.RADIO);
		radioButtonKnotenOffsetNegativ.setText("negativ");

	}

	/**
	 * Erzeugung von comboKnotenOffsetStrasse.
	 * 
	 */
	private void erzeugeComboKnotenOffsetStrasse() {
		final GridData gridData17 = new GridData();
		gridData17.horizontalAlignment = GridData.FILL;
		gridData17.grabExcessHorizontalSpace = true;
		gridData17.verticalAlignment = GridData.CENTER;
		comboKnotenOffsetStrasse = new Combo(compositeKnotenOffsetOben,
				SWT.NONE);
		comboKnotenOffsetStrasse.setLayoutData(gridData17);
	}

	/**
	 * Erzeugung von comboKnotenOffsetKnoten.
	 * 
	 */
	private void erzeugeComboKnotenOffsetKnoten() {
		final GridData gridData18 = new GridData();
		gridData18.horizontalAlignment = GridData.FILL;
		gridData18.grabExcessHorizontalSpace = true;
		gridData18.verticalAlignment = GridData.CENTER;
		comboKnotenOffsetKnoten = new Combo(compositeKnotenOffsetOben, SWT.NONE);
		comboKnotenOffsetKnoten.setLayoutData(gridData18);
	}

	/**
	 * Fllen des {@link TabFolder} mit den jeweiligen {@link TabItem}s der
	 * verschiedenen Mglichkeiten zur Ortsauswahl.
	 * 
	 */
	private void fuellePultordner() {
		final GridLayout pultLayout = new GridLayout();

		pultStrassenSegment = OrtsangabeComposite.erzeugeSeite(pultOrdner,
				OrtsangabeComposite.BEZ_STRASSENSEGMENT_OFFSET);
		pultStrassenSegment.setLayout(pultLayout);
		erzeugeCompositeStrassenSegment();

		pultBetriebskilometer = OrtsangabeComposite.erzeugeSeite(pultOrdner,
				OrtsangabeComposite.BEZ_BETRIEBSKILOMETER);
		pultBetriebskilometer.setLayout(pultLayout);
		erzeugeCompositeBetriebskilometer();

		pultAsbOrtsangabe = OrtsangabeComposite.erzeugeSeite(pultOrdner,
				OrtsangabeComposite.BEZ_ASB_ORTSANGABE);
		pultAsbOrtsangabe.setLayout(pultLayout);
		erzeugeCompositeAsbOrtsangabe();

		pultKnotenOffset = OrtsangabeComposite.erzeugeSeite(pultOrdner,
				OrtsangabeComposite.BEZ_KNOTEN_OFFSET);
		pultKnotenOffset.setLayout(pultLayout);
		erzeugeCompositeKnotenOffset();

	}

	/**
	 * Liefert die Art der Ortsauswahl als Instanz des Enumerator
	 * {@link OrtsangabeEnumeratorAuswahl}.
	 * 
	 * @return Art der Ortsauswahl
	 */
	public OrtsangabeEnumeratorAuswahl getAuswahlArt() {
		return auswahl;
	}

	/**
	 * Liefert das Strassensegment.
	 * 
	 * @return {@link FilterStrassenSegmentOffsetALT}
	 */
	public FilterStrassenSegmentOffset getFilterStrassenSegmentOffset() {
		return filterStrassenSegmentOffset;
	}

	/**
	 * Liefert die ASB-Ortsangabe.
	 * 
	 * @return {@link FilterAsbOrtsangabe}
	 */
	public FilterAsbOrtsangabe getFilterAsbOrtsangabe() {
		return filterAsbOrtsangabe;
	}

	/**
	 * Liefert die Betriebskilometer.
	 * 
	 * @return {@link FilterBetriebskilometer}
	 */
	public FilterBetriebskilometer getFilterBetriebskilometer() {
		return filterBetriebskilometer;
	}

	/**
	 * Liefert den Knoten.
	 * 
	 * @return {@link FilterKnotenAlt}
	 */
	public FilterKnoten getFilterKnoten() {
		return filterKnoten;
	}

	/**
	 * Setzt die Angabe via StraenSegment.
	 * 
	 * @param pidSelektierteStrasse
	 * @param pidStrassenSegment
	 * @param km
	 * @throws IllegalArgumentException
	 */
	public void setStrassenSegment(final String pidSelektierteStrasse,
			final String pidStrassenSegment, final long km) {
		selektierePultOrdner(OrtsangabeEnumeratorAuswahl.strassensegment);

		/*
		 * Strae selektieren
		 */
		boolean selektierteStrasseGefunden = false;
		final int anzStrassen = comboStrassenSegmentStrasse.getItemCount();

		for (int i = 0; i < anzStrassen; i++) {
			final Object object = cvStrassenSegmentStrasse.getElementAt(i);
			if (object instanceof Strasse) {
				final Strasse strasse = (Strasse) object;
				if (strasse.getPid().equals(pidSelektierteStrasse)) {

					selektierteStrasseGefunden = true;
					cvStrassenSegmentStrasse
							.setSelection(new StructuredSelection(strasse));
					filterStrassenSegmentOffset
							.setPidSelektierteStrasse(strasse.getPid());
					break;
				}
			}
		}

		if (!selektierteStrasseGefunden) {
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setBetriebsKilomert \n\t=> Die Strae mit der PID "
					+ pidSelektierteStrasse + " wurde nicht gefunden!"));
		}

		/*
		 * StraenSegment selektieren
		 */
		boolean selektiertesStrasseSegmentGefunden = false;
		final int anzSegmente = comboStrassenSegmentStrassenSegment
				.getItemCount();
		for (int i = 0; i < anzSegmente; i++) {
			final Object object = cvStrassenSegmentStrassenSegment
					.getElementAt(i);
			if (object instanceof StrassenSegment) {
				final StrassenSegment strassenSegment = (StrassenSegment) object;
				if (strassenSegment.getPid().equals(pidStrassenSegment)) {
					selektiertesStrasseSegmentGefunden = true;
					cvStrassenSegmentStrassenSegment
							.setSelection(new StructuredSelection(
									strassenSegment));
					filterStrassenSegmentOffset
							.setPidSelektiertesStrassenSegment(strassenSegment
									.getPid());
					break;
				}

			}
		}

		if (!selektiertesStrasseSegmentGefunden) {
			throw new IllegalArgumentException(getClass().getName()
					+ "::setStraenSegment \n\t=> Die Strae mit der PID "
					+ pidSelektierteStrasse + " wurde nicht gefunden!");
		}

		// Feld setzen
		// textStrassenSegmentKilometer.setText("" +
		// _filterStrassenSegmentOffset.getKm());

		// _filterStrassenSegmentOffset.getKm();

		final float fKm = (float) km / 1000;
		String sKm = "" + fKm;
		sKm = sKm.replace(".", ",");

		textStrassenSegmentKilometer.setText(sKm);
		filterStrassenSegmentOffset.setKm(km);

	}

	/**
	 * Methode zum Setzen der Daten eines Betriebskilometers.
	 * 
	 * @param pidSelektierteStrasse
	 *            PID der selektierten {@link Strasse} des Betriebskilometers.
	 * @param himmelsRichtung
	 *            Himmelsrichtung
	 *            {@link de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung}
	 * @param betriebsKilometer
	 *            Betriebskilometer in Metern
	 * @param selektierterBlock
	 *            Selektierter Block des Betriebskilometers. Wurde kein
	 *            BlockNummer selektiert, ist dies mit "KeineAngabe" sichtbar zu
	 *            machen. Wird <code>null</code> bergeben, wird automatisch auf
	 *            "KeineAngabe" gesetzt.
	 * @throws IllegalArgumentException
	 *             Wird geworfen, wenn keine Strae mit der angegebenen PID
	 *             existiert oder wenn der angegebene Betriebskilometer auf
	 *             dieser Strae nicht existiert oder wenn eine BlockNummer
	 *             angegeben wurde, die nicht existiert.
	 */
	public void setBetriebskilometer(final String pidSelektierteStrasse,
			final FilterBetriebskilometer.Richtung himmelsRichtung,
			final long betriebsKilometer, final String selektierterBlock) {
		// Tab selektieren
		selektierePultOrdner(OrtsangabeEnumeratorAuswahl.betriebskilometer);

		/*
		 * Strasse selektieren
		 */
		boolean selektierteStrasseGefunden = false;
		final int anzStrassen = comboBetriebskilometerStrasse.getItemCount();
		for (int i = 0; i < anzStrassen; i++) {
			final Object object = cvBetriebsKilometerStrasse.getElementAt(i);
			if (object instanceof Strasse) {
				final Strasse strasse = (Strasse) object;
				if (strasse.getPid().equals(pidSelektierteStrasse)) {
					selektierteStrasseGefunden = true;
					cvBetriebsKilometerStrasse
							.setSelection(new StructuredSelection(strasse));
					break;
				}
			}
		}
		if (!selektierteStrasseGefunden) {
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setBetriebsKilomert \n\t=> Die Strae mit der PID "
					+ pidSelektierteStrasse + " wurde nicht gefunden!"));
		}

		/*
		 * Himmelsrichtung selektieren
		 */
		final int anzRichtungen = comboBetriebskilometerRichtung.getItemCount();
		for (int i = 0; i < anzRichtungen; i++) {
			final Object object = cvBetriebsKilometerRichtung.getElementAt(i);
			if (object instanceof FilterBetriebskilometer.Richtung) {
				final FilterBetriebskilometer.Richtung richtung = (FilterBetriebskilometer.Richtung) object;
				if (richtung.equals(himmelsRichtung)) {
					cvBetriebsKilometerRichtung
							.setSelection(new StructuredSelection(richtung));
					break;
				}
			}
		}

		/*
		 * Betriebskilometer eintragen
		 */
		textBetriebskilometerBetriebskilometer.setText("" + betriebsKilometer);
		if (filterBetriebskilometer.getVorhandeneBlockNummern().size() == 0) {
			throw (new IllegalArgumentException(
					"::setBetriebsKilomert \n\t=> Der Betriebskilometerwert "
							+ betriebsKilometer + " m ist nicht vorhanden!"));

		}

		/*
		 * BlockNummer auswhlen
		 */
		boolean blockNummerGefunden = false;
		final int anzBloecke = comboBetriebskilometerBlocknummer.getItemCount();
		if (anzBloecke > 0) {
			for (int i = 0; i < anzBloecke; i++) {
				final Object object = cvBetriebsKilometerBlockNummer
						.getElementAt(i);
				if (object instanceof String) {
					final String block = (String) object;
					if (block.equalsIgnoreCase(selektierterBlock)) {
						blockNummerGefunden = true;
						cvBetriebsKilometerBlockNummer
								.setSelection(new StructuredSelection(block));
						break;
					}
				}
			}
		}
		if (!blockNummerGefunden) {
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setBetriebsKilomert \n\t=> Die BlockNummer "
					+ selektierterBlock + " wurde nicht gefunden!"));
		}

	}

	public void setAsb(final String pidSelektierteStrasse,
			final String anfangsKnoten, final String endKnoten,
			final long stationierung,
			final EnumAsbVerkehrsRichtung asbVerkehrsRichtung) {
		/*
		 * Tab selektieren
		 */
		selektierePultOrdner(OrtsangabeEnumeratorAuswahl.asborstangabe);

		/*
		 * Strasse selektieren
		 */
		boolean selektierteStrasseGefunden = false;
		final int anzStrassen = comboAsbStrasse.getItemCount();
		for (int i = 0; i < anzStrassen; i++) {
			final Object object = cvAsbStrasse.getElementAt(i);
			if (object instanceof Strasse) {
				final Strasse strasse = (Strasse) object;
				if (strasse.getPid().equals(pidSelektierteStrasse)) {
					selektierteStrasseGefunden = true;
					cvAsbStrasse.setSelection(new StructuredSelection(strasse));
					break;
				}
			}
		}
		if (!selektierteStrasseGefunden) {
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setAsb \n\t=> Die Strae mit der PID "
					+ pidSelektierteStrasse + " wurde nicht gefunden!"));
		}

		/*
		 * Anfangsknoten selektieren
		 */
		boolean anfangsKnotenGefunden = false;
		final int anzAnfangsKnoten = comboAsbAnfangsKnoten.getItemCount();
		for (int i = 0; i < anzAnfangsKnoten; i++) {
			final Object object = cvAsbAnfangsKnoten.getElementAt(i);
			// if (object instanceof AsbKnoten)
			// {
			// AsbKnoten knoten = (AsbKnoten) object;
			// if ((knoten.getKnoten().equals(anfangsKnoten))
			// && (knoten.getTmcRichtung().equals(richtungAnfangsKnoten)))
			// {
			// anfangsKnotenGefunden = true;
			// _cvAsbAnfangsKnoten.setSelection(new
			// StructuredSelection(knoten));
			// break;
			// }
			// }
			if (object instanceof AsbKnotenKombiniert) {
				final AsbKnotenKombiniert knoten = (AsbKnotenKombiniert) object;
				if (knoten.getKnoten().equals(anfangsKnoten)) {
					anfangsKnotenGefunden = true;
					cvAsbAnfangsKnoten.setSelection(new StructuredSelection(
							knoten));
					break;
				}
			}
		}
		if (!anfangsKnotenGefunden) {
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setAsb \n\t=> Der Anfangsknoten " + anfangsKnoten
					+ " wurde nicht gefunden!"));
		}

		/*
		 * Endknoten selektieren
		 */
		boolean endKnotenGefunden = false;
		final int anzEndKnoten = comboAsbEndKnoten.getItemCount();
		for (int i = 0; i < anzEndKnoten; i++) {
			final Object object = cvAsbEndKnoten.getElementAt(i);
			// if (object instanceof AsbKnoten)
			// {
			// AsbKnoten knoten = (AsbKnoten) object;
			// if ((knoten.getKnoten().equals(endKnoten))
			// && (knoten.getTmcRichtung().equals(richtungEndKnoten)))
			// {
			// endKnotenGefunden = true;
			// _cvAsbEndKnoten.setSelection(new StructuredSelection(knoten));
			// break;
			// }
			// }
			if (object instanceof AsbKnotenKombiniert) {
				final AsbKnotenKombiniert knoten = (AsbKnotenKombiniert) object;
				if (knoten.getKnoten().equals(endKnoten)) {
					endKnotenGefunden = true;
					cvAsbEndKnoten
							.setSelection(new StructuredSelection(knoten));
					break;
				}
			}
		}
		if (!endKnotenGefunden) {
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setAsb \n\t=> Der Endknoten " + endKnoten
					+ " wurde nicht gefunden!"));
		}

		if (stationierung >= 0) {
			textAsbStationierung.setText("" + stationierung);
		}

		/*
		 * Verkehrsrichtung
		 */
		for (int i = 0; i < comboAsbStationierungsrichtung.getItemCount(); i++) {
			final Object elementAt = cvAsbStationierungsrichtung
					.getElementAt(i);
			if (elementAt instanceof EnumAsbVerkehrsRichtung) {
				final EnumAsbVerkehrsRichtung richtung = (EnumAsbVerkehrsRichtung) elementAt;
				if (richtung.equals(asbVerkehrsRichtung)) {
					cvAsbStationierungsrichtung
							.setSelection(new StructuredSelection(richtung));
					break;
				}
			}
		}

	}

	public void setAsb(final String pidSelektierteStrasse,
			final String anfangsKnoten, final String endKnoten,
			final long stationierung) {
		setAsb(pidSelektierteStrasse, anfangsKnoten, endKnoten, stationierung,
				null);
	}

	/**
	 * Methode zum Setzen der Daten eines Knoten-Offsets.
	 * 
	 * @param pidSelektierteStrasse
	 *            PID der selektierten {@link Strasse}.
	 * @param pidSelektierterKnoten
	 *            PID des selektierten {@link StrassenKnoten}s.
	 * @param km
	 *            Kilometer.
	 * @param selektierteRichtung
	 *            {@link Richtung}.
	 * @throws IllegalArgumentException
	 *             Wird geworfen, wenn die angegebene {@link Strasse} oder der
	 *             angegebene {@link StrassenKnoten} nicht existiert.
	 */
	public void setKnotenOffset(final String pidSelektierteStrasse,
			final String pidSelektierterKnoten, final float km,
			final Richtung selektierteRichtung) {
		// Tab selektieren
		selektierePultOrdner(OrtsangabeEnumeratorAuswahl.knotenoffset);

		/*
		 * Strae selektieren
		 */
		boolean selektierteStrasseGefunden = false;
		final int anzStrassen = comboKnotenOffsetStrasse.getItemCount();
		for (int i = 0; i < anzStrassen; i++) {
			final Object object = cvKnotenOffsetStrasse.getElementAt(i);
			if (object instanceof Strasse) {
				final Strasse strasse = (Strasse) object;
				if (strasse.getPid().equals(pidSelektierteStrasse)) {
					selektierteStrasseGefunden = true;
					cvKnotenOffsetStrasse.setSelection(new StructuredSelection(
							strasse));
					break;
				}
			}
		}
		if (!selektierteStrasseGefunden) {
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setKnotenOffset \n\t=> Die Strae mit der PID "
					+ pidSelektierteStrasse + " wurde nicht gefunden!"));
		}

		/*
		 * Knoten selektieren
		 */
		boolean selektierterKnotenGefunden = false;
		final int anzKnoten = comboKnotenOffsetKnoten.getItemCount();
		for (int i = 0; i < anzKnoten; i++) {
			final Object object = cvKnotenOffsetKnoten.getElementAt(i);
			if (object instanceof StrassenKnoten) {
				final StrassenKnoten knoten = (StrassenKnoten) object;
				if (knoten.getPid().equals(pidSelektierterKnoten)) {
					selektierterKnotenGefunden = true;
					cvKnotenOffsetKnoten.setSelection(new StructuredSelection(
							knoten));
					break;
				}
			}
		}
		if (!selektierterKnotenGefunden) {
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setKnotenOffset \n\t=> Der Knoten mit der PID "
					+ pidSelektierterKnoten + " wurde nicht gefunden!"));
		}

		/*
		 * Textfeld Kilometer setzen
		 */
		textKnotenOffsetKilometer.setText("" + km);

		/*
		 * Richtung setzen
		 */
		if (selektierteRichtung.equals(Richtung.positiv)) {
			radioButtonKnotenOffsetNegativ.setSelection(false);
			radioButtonKnotenOffsetPositiv.setSelection(true);
		} else if (selektierteRichtung.equals(Richtung.negativ)) {
			radioButtonKnotenOffsetPositiv.setSelection(false);
			radioButtonKnotenOffsetNegativ.setSelection(true);
		}

	}

	/**
	 * Erzeugen der Seiten fr den {@link TabFolder}.
	 * 
	 * @param folder
	 *            {@link TabFolder}, zu dem die Seiten hinzugefgt werden
	 *            sollen.
	 * @param label
	 *            Bezeichnung der Seite im Reiter.
	 * @return
	 */
	private static Composite erzeugeSeite(final TabFolder folder,
			final String label) {
		// TabItem erzeugen + TabText setzen
		final TabItem tab = new TabItem(folder, SWT.NONE);
		tab.setText(label);

		// Composite erzeugen
		final Composite page = new Composite(folder, SWT.NONE);

		// Composite als Control dem Tab zuordnen
		tab.setControl(page);
		return page;
	}

}
