/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

public final class KonfigurationNeu {

	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Datenmodell.
	 */
	private DataModel dataModel;

	/**
	 * * Verbindung zum DaV.
	 */
	private ClientDavConnection davConnection;

	/**
	 * {@link HashMap} mit den Strae als Instanz der Klasse {@link Strasse}
	 * (Key = Pid, Value = Instanz).
	 */
	private final Map<String, Strasse> strassenMenge = new HashMap<String, Strasse>();

	/**
	 * Hashmap mit Instanzen der einzelnen Staenknoten (Key = Pid, Value =
	 * Instanz).
	 */
	private final Map<String, StrassenKnoten> strassenKnotenMenge = new HashMap<String, StrassenKnoten>();

	/**
	 * Hashmap mit Instanzen der einzelnen ueren Straensegmente (Key = Pid,
	 * Value = Instanz).
	 */
	private final Map<String, AeusseresStrassenSegment> aeusseresStrassenSegmentMenge = new HashMap<String, AeusseresStrassenSegment>();

	/**
	 * Hashmap mit Instanzen der einzelnen inneren Staensegmente (Key = Pid,
	 * Value = Instanz).
	 */
	private final Map<String, InneresStrassenSegment> inneresStrassenSegmentMenge = new HashMap<String, InneresStrassenSegment>();

	/**
	 * Hashmap mit Instanzen der einzelnen, gefilterten Staenknoten (Key = Pid,
	 * Value = Instanz).
	 */
	private final Map<String, StrassenKnoten> strassenKnotenGefiltert = new HashMap<String, StrassenKnoten>();

	/**
	 * Hashmap mit Instanzen der einzelnen <b>gefilterten</b> ueren
	 * Straensegmente (Key = Pid, Value = Instanz).
	 */
	private final Map<String, AeusseresStrassenSegment> aeusseresStrassenSegmentGefiltert = new HashMap<String, AeusseresStrassenSegment>();

	/**
	 * Hashmap mit Instanzen der einzelnen <b>gefilterten</b> inneren
	 * Staensegmente (Key = Pid, Value = Instanz).
	 */
	private final Map<String, InneresStrassenSegment> inneresStrassenSegmentGefiltert = new HashMap<String, InneresStrassenSegment>();

	/**
	 * Privater Konstruktor.
	 */
	private KonfigurationNeu() {
	}

	private static class Inner {
		private static final KonfigurationNeu INSTANZ = new KonfigurationNeu();
	}

	public static KonfigurationNeu getInstanz() {
		return Inner.INSTANZ;
	}

	/**
	 * Bestimmt die bentigten Objekte zu einer Strae.
	 * 
	 * @param connection
	 *            Verbindung zum DaV
	 */
	public void bestimmeObjekte(final ClientDavConnection connection) {
		davConnection = connection;

		dataModel = davConnection.getDataModel();

		bestimmeStrassen();
	}

	/**
	 * Methode zum Bestimmen aller Objekte vom Typ "typ.strae".
	 */
	private void bestimmeStrassen() {
		final List<SystemObject> strassen = bestimmeObjekte("typ.strae");

		dataModel.getConfigurationData(strassen,
				dataModel.getAttributeGroup("atg.strae"));

		final Iterator<SystemObject> it = strassen.iterator();
		while (it.hasNext()) {
			final SystemObject so = it.next();
			// System.out.println("PID: " + so.getPid());
			strassenMenge.put(so.getPid(), new Strasse(so, dataModel));
		}
	}

	/**
	 * Fhrt die Filterung nach der selektierten Strae durch.
	 * 
	 * @param pidSelektierteStrasse
	 */
	public void filtereNachStrasse(final String pidSelektierteStrasse) {
		strassenKnotenMenge.clear();
		bestimmeStrassenKnoten();
		aeusseresStrassenSegmentGefiltert.clear();
		bestimmeAeussereStrassenSegmenteGefiltert(pidSelektierteStrasse);
		inneresStrassenSegmentGefiltert.clear();
		bestimmeInnereStrassenSegmenteGefiltert(pidSelektierteStrasse);

		bestimmeStrassenKnotenGefiltert(pidSelektierteStrasse);

		// /*
		// * Lschen berflssiger StrassenKnoten
		// */
		// loescheUeberfluessigeStrassenKnoten();
		//
		// /*
		// * Entfernen der entsprechenden Strassenknoten aus den ueren
		// * Straensegmenten
		// */
		// loescheNichtVorhandeneStrassenKnotenAusAeusserenStrassenSegmenten();
	}

	/**
	 * Methode zum Bestimmen aller ueren Straensegmente, die zu einer
	 * bestimmten Strae gehren.
	 * 
	 * @param pidSelektierteStrasse
	 *            Pid der Strae, zu der die ueren Straensegmente bestimmt
	 *            werden sollen.
	 */
	private void bestimmeAeussereStrassenSegmenteGefiltert(
			final String pidSelektierteStrasse) {
		// LinkedList<SystemObject> aeussereStrassenSegmente = new
		// LinkedList<SystemObject>();

		final List<SystemObject> aeussereStrassenSegmente = bestimmeObjekte("typ.ueresStraenSegment");

		// PERFORMANCEVERBESSERUNGEN ANSEHEN
		dataModel.getConfigurationData(aeussereStrassenSegmente,
				dataModel.getAttributeGroup("atg.straenSegment"));
		dataModel.getConfigurationData(aeussereStrassenSegmente,
				dataModel.getAttributeGroup("atg.ueresStraenSegment"));

		final Iterator<SystemObject> it = aeussereStrassenSegmente.iterator();
		while (it.hasNext()) {
			final SystemObject so = it.next();
			final AttributeGroup attributeGroup = dataModel
					.getAttributeGroup("atg.straenSegment");
			final Data data = so.getConfigurationData(attributeGroup);
			final String valueText = data.getTextValue("gehrtZuStrae")
					.getValueText();

			// System.out.println("VAL: " + valueText);
			// System.out.println("PID: " + pidSelektierteStrasse);

			final AeusseresStrassenSegment aeusseresStrassenSegment = new AeusseresStrassenSegment(
					dataModel, so);
			// m_aeusseresStrassenSegment.put(so.getPid(),
			// new AeusseresStrassenSegment(m_dataModel, so));
			aeusseresStrassenSegmentMenge
					.put(aeusseresStrassenSegment.getPid(),
							aeusseresStrassenSegment);

			final StrassenKnoten vonKnoten = aeusseresStrassenSegment
					.getVonKnoten();
			final StrassenKnoten nachKnoten = aeusseresStrassenSegment
					.getNachKnoten();

			if (vonKnoten != null) {
				vonKnoten
						.addAbgehendesAeusseresStrassenSegment(aeusseresStrassenSegment);
			}

			if (nachKnoten != null) {
				nachKnoten
						.addHinfuehrendesAeusseresStrassenSegment(aeusseresStrassenSegment);
			}

			/*
			 * Gefiltertes ueresStraenSegment
			 */
			if (valueText.equals(pidSelektierteStrasse)) {
				aeusseresStrassenSegmentGefiltert.put(
						aeusseresStrassenSegment.getPid(),
						aeusseresStrassenSegment);

				/*
				 * Bestimmung des STS
				 */
				aeusseresStrassenSegment.initialisiere();

			}

		}

		// Iterator<SystemObject> itASS = aeussereStrassenSegmente.iterator();
		// while (itASS.hasNext())
		// {
		// SystemObject objekt = itASS.next();
		//
		// AeusseresStrassenSegment strassenSegment = new
		// AeusseresStrassenSegment(
		// m_dataModel, objekt);
		// strassenSegment.initialisiere();
		//
		// m_aeusseresStrassenSegment.put(objekt.getPid(), strassenSegment);
		//
		// StrassenKnoten vonKnoten = strassenSegment.getVonKnoten();
		// StrassenKnoten nachKnoten = strassenSegment.getNachKnoten();
		//
		// if (vonKnoten != null)
		// vonKnoten.addAbgehendesAeusseresStrassenSegment(strassenSegment);
		//
		// if (nachKnoten != null)
		// nachKnoten.addHinfuehrendesAeusseresStrassenSegment(strassenSegment);
		// }
	}

	/**
	 * Methode zum Bestimmen aller Objekte com Typ "typ.inneresStraenSegment".
	 * Zustzlich wird eine Zuordnung der Strassensegmente zu den Strassenknoten
	 * vorgenommen
	 */
	private void bestimmeInnereStrassenSegmenteGefiltert(
			final String pidSelektierteStrasse) {
		final List<SystemObject> innereStrassenSegmente = bestimmeObjekte("typ.inneresStraenSegment");

		dataModel.getConfigurationData(innereStrassenSegmente,
				dataModel.getAttributeGroup("atg.inneresStraenSegment"));

		final List<SystemObject> innereStrassenSegmenteGefiltert = new LinkedList<SystemObject>();

		/*
		 * Nur die innerenStrassensegmente verwenden, die zu einer Strae
		 * gehren
		 */
		final Iterator<SystemObject> itFi = innereStrassenSegmente.iterator();
		while (itFi.hasNext()) {
			final SystemObject so = itFi.next();
			final AttributeGroup attributeGroup = dataModel
					.getAttributeGroup("atg.straenSegment");
			final Data data = so.getConfigurationData(attributeGroup);
			final String valueText = data.getTextValue("gehrtZuStrae")
					.getValueText();
			if (valueText.equals(pidSelektierteStrasse)) {
				innereStrassenSegmenteGefiltert.add(so);
			}

			final InneresStrassenSegment strassenSegment = new InneresStrassenSegment(
					dataModel, so);

			inneresStrassenSegmentMenge.put(so.getPid(), strassenSegment);
		}

		// Iterator it = innereStrassenSegmente.iterator();
		final Iterator<SystemObject> it = innereStrassenSegmenteGefiltert
				.iterator();
		while (it.hasNext()) {

			final SystemObject objekt = it.next();

			final InneresStrassenSegment strassenSegment = new InneresStrassenSegment(
					dataModel, objekt);

			// m_inneresStrassenSegment.put(objekt.getPid(), strassenSegment);

			// Fllen der Gefiltert Liste
			if (innereStrassenSegmenteGefiltert.contains(objekt)) {
				inneresStrassenSegmentGefiltert.put(objekt.getPid(),
						strassenSegment);
			}

			final AeusseresStrassenSegment vonStrassenSegment = strassenSegment
					.getVonStrassenSegmemt();
			final AeusseresStrassenSegment nachStrassenSegment = strassenSegment
					.getNachStrassenSegment();

			if (((vonStrassenSegment) == null) && (nachStrassenSegment == null)) {
				inneresStrassenSegmentMenge.remove(objekt.getPid());
			} else {
				strassenSegment.initialisiere();

				if (vonStrassenSegment != null) {
					strassenSegment.setVorgaengerAufStrasse(vonStrassenSegment);

					final StrassenKnoten sk = strassenKnotenMenge
							.get(vonStrassenSegment.getNachKnoten().getPid());
					if (sk != null) {
						sk.addInneresStrassenSegment(strassenSegment);
					}

					if (vonStrassenSegment
							.liegtAufSelberStrasse(nachStrassenSegment)) {
						vonStrassenSegment
								.addNachfolgerAufStrasse(strassenSegment);
					} else {
						vonStrassenSegment.addNachfolger(strassenSegment);
					}
				}

				if (nachStrassenSegment != null) {
					strassenSegment
							.setNachfolgerAufStrasse(nachStrassenSegment);

					final StrassenKnoten sk = strassenKnotenMenge
							.get(nachStrassenSegment.getVonKnoten().getPid());
					if (sk != null) {
						sk.addInneresStrassenSegment(strassenSegment);
					}

					if (nachStrassenSegment
							.liegtAufSelberStrasse(vonStrassenSegment)) {
						nachStrassenSegment
								.addVorgaengerAufStrasse(strassenSegment);
					} else {
						nachStrassenSegment.addVorgaenger(strassenSegment);
					}
				}
			}
		}
	}

	/**
	 * Bestimmt die Knoten, die auf einer bestimmten Strae liegen und legt
	 * diese in der HashMap {@link KonfigurationNeu#strassenKnotenGefiltert} ab.
	 * 
	 * @param pidSelektierteStrasse
	 *            Pid der Strae, zu der die zugehrigen Knoten bestimmt werden
	 *            sollen.
	 */
	private void bestimmeStrassenKnotenGefiltert(
			final String pidSelektierteStrasse) {

		/*
		 * Ablauf
		 * 
		 * OK - lschen der alten gefilterten Map - durchlaufen aller Knoten der
		 * m_strassenKnoten - prfen ob Abgehende/Hinzufhrende uere
		 * Straensegmente auf der Strae liegen. -wenn ja, dann kommt der
		 * Knoten in die gefilterte Map, sonst nicht.
		 */
		strassenKnotenGefiltert.clear();

		final Iterator<Entry<String, StrassenKnoten>> itKnoten = strassenKnotenMenge
				.entrySet().iterator();
		while (itKnoten.hasNext()) {
			boolean gefunden = false;

			final Entry<String, StrassenKnoten> entryKnoten = itKnoten.next();
			final List<AeusseresStrassenSegment> abgehendeAeussereStrassenSegmente = entryKnoten
					.getValue().getAbgehendeAeussereStrassenSegmente();
			final List<AeusseresStrassenSegment> hinfuehrendeAeussereStrassenSegmente = entryKnoten
					.getValue().getHinfuehrendeAeussereStrassenSegmente();
			final List<InneresStrassenSegment> innereStrassenSegmente = entryKnoten
					.getValue().getInnereStrassenSegmente();

			/*
			 * Abgehende uereStraenSegmente
			 */
			if (!gefunden) {
				final Iterator<AeusseresStrassenSegment> itAngehendeASS = abgehendeAeussereStrassenSegmente
						.iterator();
				while (itAngehendeASS.hasNext()) {
					final AeusseresStrassenSegment aeusseresStrassenSegment = itAngehendeASS
							.next();
					if (aeusseresStrassenSegmentGefiltert
							.containsKey(aeusseresStrassenSegment.getPid())) {
						gefunden = true;
						break;
					}
				}
			}

			/*
			 * Hinzufhrende uereStraenSegmente
			 */
			if (!gefunden) {
				final Iterator<AeusseresStrassenSegment> itHinzufASS = hinfuehrendeAeussereStrassenSegmente
						.iterator();
				while (itHinzufASS.hasNext()) {
					final AeusseresStrassenSegment aeusseresStrassenSegment = itHinzufASS
							.next();
					if (aeusseresStrassenSegmentGefiltert
							.containsKey(aeusseresStrassenSegment.getPid())) {
						gefunden = true;
						break;
					}
				}
			}

			/*
			 * InnereStraenSegmente
			 */
			if (!gefunden) {
				final Iterator<InneresStrassenSegment> it = innereStrassenSegmente
						.iterator();
				while (it.hasNext()) {
					final InneresStrassenSegment inneresStrassenSegment = it
							.next();
					if (inneresStrassenSegmentGefiltert
							.containsKey(inneresStrassenSegment.getPid())) {
						gefunden = true;
						break;
					}
				}
			}

			if (gefunden) {
				strassenKnotenGefiltert.put(entryKnoten.getKey(),
						entryKnoten.getValue());
			}
		}

	}

	/**
	 * Methode zum Bestimmen aller Objekte vom Typ "typ.straenKnoten".
	 */
	private void bestimmeStrassenKnoten() {
		final List<SystemObject> strassenKnoten = bestimmeObjekte("typ.straenKnoten");

		dataModel.getConfigurationData(strassenKnoten,
				dataModel.getAttributeGroup("atg.straenKnoten"));

		final Iterator<SystemObject> it = strassenKnoten.iterator();
		while (it.hasNext()) {
			final SystemObject objekt = it.next();

			strassenKnotenMenge.put(objekt.getPid(), new StrassenKnoten(
					davConnection, objekt));
		}
	}

	/**
	 * Methode zum Bestimmen der Objekte, die zu einer bestimmten PID gehren
	 * (z.B. typ.de).
	 * 
	 * @param objPid
	 *            Pid der Objekttypen
	 * @return Liste mit den Systemobjekten
	 */
	private List<SystemObject> bestimmeObjekte(final String objPid) {
		KonfigurationNeu.LOGGER.fine("bestimmeObjekte --> " + objPid);

		final List<SystemObject> objekte = new ArrayList<SystemObject>();

		// Systemobjekt erzeugen

		SystemObjectType typeSysObj = null;

		typeSysObj = davConnection.getDataModel().getType(objPid);

		if (typeSysObj == null) {
			return objekte;
		}

		// Elemente dieses Systemobjekts einlesen

		final List<SystemObject> typobjekte = typeSysObj.getElements();

		final Iterator<SystemObject> iteratorTypen = typobjekte.iterator();

		while (iteratorTypen.hasNext()) {
			final SystemObject sysObj = iteratorTypen.next();

			objekte.add(sysObj);
		}

		return objekte;
	}

	/**
	 * Methode liefert ein ueres Straensegment.
	 * 
	 * @param pid
	 *            Pid des Straensegments
	 * @return Instanz des Straensegments, im Fehlerfall null
	 */
	public AeusseresStrassenSegment getAeusseresStrassenSegment(final String pid) {
		if (aeusseresStrassenSegmentMenge.containsKey(pid)) {
			return aeusseresStrassenSegmentMenge.get(pid);
		}

		return null;
	}

	/**
	 * Methode liefert ein inneres Straensegment.
	 * 
	 * @param pid
	 *            Pid des Straensegments
	 * @return Instanz des Straensegments, im Fehlerfall null
	 */
	public InneresStrassenSegment getInneresStrassenSegment(final String pid) {
		if (inneresStrassenSegmentMenge.containsKey(pid)) {
			return inneresStrassenSegmentMenge.get(pid);
		}
		return null;
	}

	/**
	 * Methode liefert einen Straenknoten zu einer Pid, existiert der
	 * Stasenknoten noch nicht und handlet es sich bei dem Objekt das zu der
	 * Pid gehrt um ein Objekt vom Typ 'typ.straenKnoten', so wird ein neuer
	 * StraenKnoten angelegt.
	 * 
	 * @param pid
	 *            Pid des Straenknotens
	 * @return Instanz des Straenknotens, im Fehlerfall null
	 */
	public StrassenKnoten getStrassenKnoten(final String pid) {
		if (strassenKnotenMenge.containsKey(pid)) {
			return strassenKnotenMenge.get(pid);
		}
		final ConfigurationObject objekt = (ConfigurationObject) dataModel
				.getObject(pid);
		if (objekt != null) {
			if (objekt.isOfType("typ.straenKnoten")) {
				final StrassenKnoten sk = new StrassenKnoten(davConnection,
						objekt);
				strassenKnotenMenge.put(sk.getPid(), sk);
				return sk;
			}
		}

		return null;
	}

	/**
	 * Liefert die HashMap, welche alle {@link Strasse}n beinhaltet.
	 * 
	 * @return HashMap der {@link Strasse}n.
	 */
	public Map<String, Strasse> getStrassen() {
		return strassenMenge;
	}

	public Map<String, AeusseresStrassenSegment> getAeussereStrassenSegmente() {
		return aeusseresStrassenSegmentMenge;
	}

	public Map<String, AeusseresStrassenSegment> getAeussereStrassenSegmenteGefiltert() {
		return aeusseresStrassenSegmentGefiltert;
	}

	public Map<String, InneresStrassenSegment> getInnereStrassenSegmente() {
		return inneresStrassenSegmentMenge;
	}

	public Map<String, InneresStrassenSegment> getInnereStrassenSegmenteGefiltert() {
		return inneresStrassenSegmentGefiltert;
	}

	public Map<String, StrassenKnoten> getStrassenKnoten() {
		return strassenKnotenMenge;
	}

	/**
	 * Liefert die {@link HashMap} der gefilterten StraenKnoten.
	 * 
	 * @return {@link HashMap} der gefilterten StraenKnoten
	 */
	public Map<String, StrassenKnoten> getStrasseKnotenGefiltert() {
		return strassenKnotenGefiltert;
	}

	@Override
	public String toString() {
		String re = "";

		re = re + "Anzahl Straen:                             "
				+ strassenMenge.size() + "\n";
		re = re + "Anzahl uerer Straensegmente:             "
				+ aeusseresStrassenSegmentMenge.size() + "\n";

		re = re + "Anzahl uerer Straensegmente (Gefiltert): "
				+ aeusseresStrassenSegmentGefiltert.size() + "\n";

		re = re + "Anzahl innerer Straensegmente:             "
				+ inneresStrassenSegmentMenge.size() + "\n";

		re = re + "Anzahl innerer Straensegmente (Gefiltert): "
				+ inneresStrassenSegmentGefiltert.size() + "\n";

		re = re + "Anzahl StraenKnoten:                       "
				+ strassenKnotenMenge.size() + "\n";

		re = re + "Anzahl StraenKnoten (Gefiltert):           "
				+ strassenKnotenGefiltert.size() + "\n";

		return re;
	}

}
