/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse zur Reprsentation einer Strae.
 * 
 * @author HofmannJ
 * 
 */
public class Strasse {

	/**
	 * Zur Strae gehrendes Systemobjekt.
	 */
	private final SystemObject systemObject;

	/**
	 * Name der Strae.
	 */
	private final String name;

	/**
	 * Typ der Strae.
	 */
	private final String typ;

	/**
	 * Nummer der Strae.
	 */
	private final long nummer;

	/**
	 * Konstruktor.
	 * 
	 * @param systemObjekt
	 *            Systemobjekt der Strae
	 * @param dataModel
	 *            Datenmodell
	 */
	public Strasse(final SystemObject systemObjekt, final DataModel dataModel) {
		this.systemObject = systemObjekt;

		/*
		 * Name der Strae bestimmen
		 */
		name = systemObject.getName();

		final AttributeGroup atg = dataModel.getAttributeGroup("atg.strae");
		final Data data = systemObject.getConfigurationData(atg);

		/*
		 * Typ der Autobahn bestimmen
		 */
		typ = data.getTextValue("Typ").getText();

		/*
		 * Nummer der Autobahn bestimmen
		 */
		nummer = data.getUnscaledValue("Nummer").longValue();

		// System.out.println(this.toString());

	}

	/**
	 * Liefert die Pid der Strae zurck.
	 * 
	 * @return Pid
	 */
	public String getPid() {
		return systemObject.getPid();
	}

	/**
	 * Liefert das {@link SystemObject} der Strae zurck.
	 * 
	 * @return {@link SystemObject} der Strae.
	 */
	public SystemObject getSystemObject() {
		return systemObject;
	}

	/**
	 * Liefert den Namen der Strae zurck.
	 * 
	 * @return Name der Strae
	 */
	public String getName() {
		return name;
	}

	/**
	 * Liefert den Typ der Strae zurck.
	 * 
	 * @return Typ der Strae
	 */
	public String getTyp() {
		return typ;
	}

	/**
	 * Liefert die Nummer der Strae zurck.
	 * 
	 * @return Nummer der Strae
	 */
	public long getNummer() {
		return nummer;
	}

	@Override
	public String toString() {
		return name + " (Typ: " + typ + " Nummer: " + nummer + ")";
	}

}
