/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.simulationsvariante;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

public class SichtSimulationsvarianteDarstellenEinstellen extends ViewPart {

	public static final String ID = "de.bsvrz.buv.rw.basislib.simulationsvariante.SichtSimulationsvarianteDarstellenEinstellen";

	private Composite basis;

	/**
	 * Textfeld zur Anzeige der aktuellen Simulationsvariante.
	 */
	private Text textAktuelleSimulationsvariante;

	/**
	 * {@link Spinner} zum Einstellen der aktuellen Simulationsvariante.
	 */
	private Spinner spinnerSimulationsvariante;

	private final Simulationsvariante simVariante = Simulationsvariante
			.getInstanz();

	private IZuhoererSimulationsvariante zuhoerer;

	@Override
	public void createPartControl(final Composite parent) {
		basis = new Composite(parent, SWT.NONE);
		basis.setLayout(new GridLayout());
		erzeugeGruppeAktuelleSimulationsvariante();
		erzeugeGruppeSimulationsvarianteWaehlen();
		erzeugeGruppeKontrolle();

		init();

		PlatformUI.getWorkbench().getHelpSystem()
				.setHelp(basis, "de.bsvrz.buv.rw.basislib.simulationsvariante");
	}

	private void init() {
		textAktuelleSimulationsvariante.setText(Integer.toString(simVariante
				.getSimulationsvariante()));

		if (zuhoerer == null) {
			zuhoerer = new IZuhoererSimulationsvariante() {
				@Override
				public void aktuelleSimulationsvariante(
						final int simulationsvariante) {
					textAktuelleSimulationsvariante.setText(Integer
							.toString(+simulationsvariante));
					textAktuelleSimulationsvariante.pack();
				}
			};
			simVariante.hinzufuegenZuhoererSimulationsvariante(zuhoerer);
		}

	}

	/**
	 * Schliet diese View.
	 * 
	 */
	private void schliessen() {
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
				.hideView(this);

		if ((simVariante != null) && (zuhoerer != null)) {
			simVariante.entferneZuhoererSimulationsvariante(zuhoerer);
		}
	}

	@Override
	public void setFocus() {
		if (basis != null) {
			basis.setFocus();
		}
	}

	/**
	 * This method initializes gruppeAktuelleSimulationsvariante.
	 * 
	 */
	private void erzeugeGruppeAktuelleSimulationsvariante() {
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		final Group gruppeAktuelleSimulationsvariante = new Group(basis,
				SWT.NONE);
		gruppeAktuelleSimulationsvariante.setLayout(new RowLayout());
		gruppeAktuelleSimulationsvariante.setLayoutData(gridData);
		gruppeAktuelleSimulationsvariante
				.setText("Aktuelle Simulationsvariante");
		final Label labelAktuelleSimulationsvariante = new Label(
				gruppeAktuelleSimulationsvariante, SWT.NONE);
		labelAktuelleSimulationsvariante
				.setText("Aktuelle Simulationsvariante:");
		textAktuelleSimulationsvariante = new Text(
				gruppeAktuelleSimulationsvariante, SWT.BORDER);
		textAktuelleSimulationsvariante.setEditable(false);
	}

	/**
	 * This method initializes gruppeSimulationsvarianteWaehlen.
	 * 
	 */
	private void erzeugeGruppeSimulationsvarianteWaehlen() {
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		final Group gruppeSimulationsvarianteWaehlen = new Group(basis,
				SWT.NONE);
		gruppeSimulationsvarianteWaehlen.setLayoutData(gridData1);
		gruppeSimulationsvarianteWaehlen.setLayout(gridLayout);
		gruppeSimulationsvarianteWaehlen.setText("Simulationsvariante whlen");
		final Label labelNeueSimulationsvariante = new Label(
				gruppeSimulationsvarianteWaehlen, SWT.NONE);
		labelNeueSimulationsvariante
				.setText("Neue Simulationsvariante einstellen:");
		spinnerSimulationsvariante = new Spinner(
				gruppeSimulationsvarianteWaehlen, SWT.NONE);
		spinnerSimulationsvariante.setDigits(0);
		spinnerSimulationsvariante.setMaximum(999);

		if (simVariante.getSimulationsvariante() > 0) {
			spinnerSimulationsvariante.setSelection(simVariante
					.getSimulationsvariante());
		}
	}

	/**
	 * This method initializes gruppeKontrolle.
	 * 
	 */
	private void erzeugeGruppeKontrolle() {
		final GridLayout gridLayout1 = new GridLayout();
		gridLayout1.numColumns = 3;
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.verticalAlignment = GridData.CENTER;
		final Group gruppeKontrolle = new Group(basis, SWT.NONE);
		gruppeKontrolle.setLayoutData(gridData2);
		gruppeKontrolle.setLayout(gridLayout1);
		gruppeKontrolle.setText("Einstellungen bernehmen");

		/*
		 * bernehmen
		 */
		final Button buttonUebernehmen = new Button(gruppeKontrolle, SWT.NONE);
		buttonUebernehmen.setText("bernehmen");
		buttonUebernehmen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				simVariante.setSimulationsvariante(spinnerSimulationsvariante
						.getSelection());
			}
		});

		/*
		 * bernehmen und Schlieen
		 */
		final Button buttonUebernehmenSchliessen = new Button(gruppeKontrolle,
				SWT.NONE);
		buttonUebernehmenSchliessen.setText("bernehmen + Schlieen");
		buttonUebernehmenSchliessen
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						simVariante
								.setSimulationsvariante(spinnerSimulationsvariante
										.getSelection());
						schliessen();
					}
				});

		/*
		 * Schlieen, ohne die nderungen zu bernehmen.
		 */
		final Button buttonSchliessen = new Button(gruppeKontrolle, SWT.NONE);
		buttonSchliessen.setText("Schlieen");
		buttonSchliessen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				schliessen();
			}
		});

	}

} // @jve:decl-index=0:visual-constraint="10,10,593,306"
