/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TransferData;

public class DropAdapter extends ViewerDropAdapter {
	/**
	 * Speichert die {@link TabellenDatenStruktur}, auf die sich dieser
	 * {@link ViewerDropAdapter} bezieht.
	 */
	private final TabellenDatenStruktur tds;

	/**
	 * Speichert das Ziel fr die Drop-Operation.
	 */
	private ITabellenZeile ziel;

	/**
	 * Konstruktor.
	 * 
	 * @param viewer
	 *            Viewer, fr den die Drop-Funktionalitt bereitgestellt wird.
	 * @param tds
	 *            {@link TabellenDatenStruktur}, auf welche sich die Operation
	 *            bezieht.
	 */
	public DropAdapter(final Viewer viewer, final TabellenDatenStruktur tds) {
		super(viewer);
		this.tds = tds;
	}

	@Override
	public boolean validateDrop(final Object target, final int operation,
			final TransferData transferType) {
		boolean re = false;

		if (target instanceof ITabellenZeile) {
			ziel = (ITabellenZeile) target;
			re = true;
		} else if (target == null) {
			ziel = null;
			re = true;
		}

		return re;
	}

	@Override
	public boolean performDrop(final Object data) {
		boolean re = false;

		if (data instanceof String) {
			final String pruefen = (String) data;
			if ("[ITabellenZeile]".equals(pruefen)) {
				if (tds.dropZeile(ziel)) {
					getViewer().setInput(tds);
					re = true;
				}
			}
		}
		return re;
	}

}
