/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import org.jdom.Attribute;
import org.jdom.Element;

/**
 * Klasse zur Beschreibung eines regulren Ausdrucks.
 * 
 * @author HofmannJ
 * 
 */
public class ElementRegEx implements Cloneable {
	/**
	 * ID des Elementes.
	 */
	private int id = -1;

	/**
	 * Bezeichnung.
	 */
	private String bezeichnung;

	/**
	 * Regulrer Ausdruck.
	 */
	private String ausdruck;

	/**
	 * Das Element selbst.
	 */
	private Element xmlElement;

	/**
	 * Konstuktor, initialisiert ein {@link ElementRegEx} mit ID und
	 * Bezeichnung.
	 * 
	 * @param id
	 *            ID des Elementes
	 * @param bezeichnung
	 *            Bezeichnung
	 * @param regEx
	 *            Regulrer Ausdruck
	 */
	public ElementRegEx(final int id, final String bezeichnung,
			final String regEx) {
		initialisiere(id, bezeichnung, regEx);
	}

	/**
	 * Konstruktor, initialisiert ein {@link ElementRegEx} an Hand des JDOM-
	 * {@link Element}es.
	 * 
	 * @param element
	 */
	public ElementRegEx(final Element element) {
		initialisiere(element);
	}

	/**
	 * Fhrt die Initialisierung durch.
	 * 
	 * @param initialId
	 *            ID zur Initialisierung
	 * @param initialBezeichnung
	 *            Bezeichnung zur Initialisierung
	 * @param regEx
	 *            Regulrer Ausdruck
	 */
	public void initialisiere(final int initialId,
			final String initialBezeichnung, final String regEx) {
		this.id = initialId;
		this.bezeichnung = initialBezeichnung;
		xmlElement = new Element(XmlFilterVerwaltung.XML_ELEMENT_AUSDRUCK);
		final Attribute attId = new Attribute(XmlFilterVerwaltung.XML_ATT_ID,
				"" + initialId);
		xmlElement.setAttribute(attId);
		final Attribute attBez = new Attribute(
				XmlFilterVerwaltung.XML_ATT_BEZEICHNUNG, initialBezeichnung);
		xmlElement.setAttribute(attBez);
		ausdruck = regEx;
		xmlElement.setText(regEx);
	}

	/**
	 * Initialisiert das Element an Hand eines JDOM-{@link Element}es.
	 * 
	 * @param element
	 *            JDOM-{@link Element}
	 */
	public void initialisiere(final Element element) {
		if (element.getName().equals(XmlFilterVerwaltung.XML_ELEMENT_AUSDRUCK)) {
			xmlElement = element;

			/*
			 * ID Holen
			 */
			try {
				this.id = Integer.valueOf(xmlElement
						.getAttributeValue(XmlFilterVerwaltung.XML_ATT_ID));
			} catch (final NumberFormatException e) {
				e.printStackTrace();
			}

			/*
			 * Bezeichnung Holen
			 */
			this.bezeichnung = xmlElement
					.getAttributeValue(XmlFilterVerwaltung.XML_ATT_BEZEICHNUNG);

			/*
			 * RegEx holen
			 */
			this.ausdruck = element.getText();

		}

	}

	public Element getElement() {
		initialisiere(id, bezeichnung, ausdruck);
		return xmlElement;
	}

	/**
	 * Liefert die ID dieses Elementes.
	 * 
	 * @return ID
	 */
	public int getId() {
		return id;
	}

	/**
	 * Setzt die ID dieses Elementes.
	 * 
	 * @param id
	 *            ID
	 */
	public void setId(final int id) {
		this.id = id;
	}

	/**
	 * Liefert die Bezeichnung dieses Elementes.
	 * 
	 * @return Bezeichnung
	 */
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * Setzt die Bezeichnung dieses Elementes.
	 * 
	 * @param bezeichnung
	 *            Zu setzende Bezeichnung
	 */
	public void setBezeichnung(final String bezeichnung) {
		this.bezeichnung = bezeichnung;
	}

	/**
	 * Liefert den regulren Ausdruck.
	 * 
	 * @return regulrer Ausdruck
	 */
	public String getRegEx() {
		return ausdruck;
	}

	/**
	 * Setzt den regulren Ausdruck.
	 * 
	 * @param regEx
	 *            regulrer Ausdruck
	 */
	public void setRegEx(final String regEx) {
		this.ausdruck = regEx;
	}

	@Override
	public ElementRegEx clone() {
		try {
			super.clone();
		} catch (final CloneNotSupportedException e) {
			throw new IllegalStateException(e);
		}
		final ElementRegEx re = new ElementRegEx(id, bezeichnung, ausdruck);
		return re;
	}

	@Override
	public String toString() {
		String re = "";
		re = re + "  RegEx ID: " + id + " Bezeichnung: " + bezeichnung
				+ " RegEx: " + ausdruck;
		return re;
	}
}
