/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

/**
 * Speichert die Sortierung einer Spalte.
 * 
 * @author HofmannJ
 * 
 */
public class Sortierung {
	/**
	 * Spalte, auf welche sich die Sortierung bezieht.
	 */
	private Integer spalte = Integer.valueOf(-1);

	/**
	 * Gibt an, ob die Spalte aufsteigend (<code>true</code>) oder absteigend(
	 * <code>false</code>) sortiert sein soll.
	 */
	private boolean aufsteigend = true;

	/**
	 * Konstruktor. Setzt den Index dieser Spalte.
	 * 
	 * @param spalte
	 *            Index der Spalte.
	 */
	public Sortierung(final Integer spalte) {
		this.spalte = spalte;
	}

	/**
	 * Konstruktor. Setzt Index und Sortierrichtung dieser Spalte.
	 * 
	 * @param spalte
	 *            Index der Spalte.
	 * @param aufsteigend
	 *            Sortierrichtung der Spate. <code>true</code> == aufsteigend,
	 *            <code>false</code> == absteigend.
	 */
	public Sortierung(final Integer spalte, final boolean aufsteigend) {
		this.spalte = spalte;
		this.aufsteigend = aufsteigend;
	}

	/**
	 * Liefert den Index dieser Spalte.
	 * 
	 * @return Index dieser Spalte
	 */
	public Integer getSpalte() {
		return spalte;
	}

	/**
	 * Setzt den Index dieser Spalte.
	 * 
	 * @param spalte
	 *            Index dieser Spalte
	 */
	public void setSpalte(final Integer spalte) {
		this.spalte = spalte;
	}

	/**
	 * Prft, ob diese Spalte aufsteigend oder absteigend sortiert werden soll.
	 * 
	 * @return <code>true</code> == aufsteigend, <code>false</code> ==
	 *         absteigend
	 */
	public boolean istAufsteigend() {
		return aufsteigend;
	}

	/**
	 * Setzt die Richtung der Sortierung dieser Spalte.
	 * 
	 * @param aufsteigend
	 *            <code>true</code> == aufsteigend, <code>false</code> ==
	 *            absteigend
	 */
	public void setAufsteigend(final boolean aufsteigend) {
		this.aufsteigend = aufsteigend;
	}
}
