/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.urlasser;

import de.bsvrz.dav.daf.main.ClientDavInterface;

/**
 * Schnittstelle fr eine Klasse, die eine Operationen mit einem Datenverteiler
 * ber eine besttigte mit UrlasserInformationen erstellte Verbindung ausfhren
 * kann.
 * 
 * Die Instanz der Klasse wird optional dem UrlasserDatenDialog bergeben, nach der
 * erfolgreichen Identifikation des Urlassers wird optional eine neue
 * Datenverteilerverbindung erstellt und der Klasse bergeben.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: UrlasserDatenSender.java 46015 2013-09-15 19:47:41Z peuker $
 */
public interface UrlasserDatenSender {

	/**
	 * Callback-Funktion wird aufgerufen, wenn ein Urlasser ber die
	 * Urlasserabfrage identifiziert wurde und optional eine entsprechende
	 * Datenverteilerverbindung erstellt wurde.
	 * 
	 * Nach der Ausfhrung der gewnschten Operationen wird die Verbindung
	 * automatisch wieder geschlossen.
	 * 
	 * @param verbindung
	 *            die Verbindung zum Datenverteiler
	 * @param urlasser
	 *            die Urlasserinformationen
	 */
	void execute(ClientDavInterface verbindung, UrlasserInfo urlasser);
}
