/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw;

import java.util.Timer;

import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import de.bsvrz.buv.rw.rw.perspective.RwPerspectives;
import de.bsvrz.buv.rw.rw.preferences.RahmenwerkPreference;

/**
 * Der Aktivator des Hauptplugins des Rahmenwerks.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RahmenwerkActivator.java 46800 2013-10-23 14:35:48Z peuker $
 */
public final class RahmenwerkActivator extends AbstractUIPlugin {

	/** die Id des Plug-ins. */
	public static final String PLUGIN_ID = "de.bsvrz.buv.rw.rw";

	private static RahmenwerkActivator plugin;
	private MApplication application;
	private static final Timer RAHMENWERK_TIMER = new Timer(
			"RahmenwerkMainTimer", true);

	private RwPerspectives rwPerspectives;

	/**
	 * liefert die globale Instanz des Plug-ins.
	 * 
	 * @return die Instanz
	 */
	public static RahmenwerkActivator getDefault() {
		return RahmenwerkActivator.plugin;
	}

	/**
	 * liefert einen globalen Timer fr alle Klassen des Plug-ins.
	 * 
	 * @return den Timer
	 */
	public static Timer getRahmenwerkTimer() {
		return RahmenwerkActivator.RAHMENWERK_TIMER;
	}

	/**
	 * liefert das Applikationsmodell.
	 * 
	 * Das Modell muss vorher von einem Prozessor gesetzt worden sein und kann
	 * daher <code>null</code> sein.
	 * 
	 * @return die Applikation oder <code>null</code>
	 */
	public MApplication getApplication() {
		return application;
	}

	/**
	 * liefert ein Image fr den bergebenen Pfad und nimmt dieses in die
	 * Image-Registry des Plug-ins auf.
	 * 
	 * Wenn kein entsprechendes Image erzeugt werden konnte, wird
	 * <code>null</code> geliefert.
	 * 
	 * @param pluginPath
	 *            der Pfad
	 * @return das Image oder <code>null</code>
	 */
	public Image getImage(final String pluginPath) {
		final ImageDescriptor descriptor = getImageDescriptor(pluginPath);
		if (descriptor == null) {
			return null;
		}
		Image image = getImageRegistry().get(descriptor.toString());
		if (image == null) {
			image = descriptor.createImage();
			getImageRegistry().put(descriptor.toString(), image);
		}

		return image;
	}

	/**
	 * liefert den ImageDescriptor fr das Plugin und den bergebenen Pfad.
	 * 
	 * @param pluginPath
	 *            der Pfad
	 * @return der Deskriptor
	 */
	public ImageDescriptor getImageDescriptor(final String pluginPath) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(
				RahmenwerkActivator.PLUGIN_ID, pluginPath);
	}

	/**
	 * setzt das Applikationsobjekt (Modell).
	 * 
	 * @param application
	 *            das Objekt
	 */
	public void setApplication(final MApplication application) {
		this.application = application;
	}

	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		for (final RahmenwerkPreference preference : RahmenwerkPreference
				.values()) {

			final Object defaultValue = preference.getDefaultValue();
			if (defaultValue == null) {
				continue;
			}

			if (defaultValue instanceof Boolean) {
				getPreferenceStore().setDefault(preference.getId(),
						(Boolean) defaultValue);
			} else if (defaultValue instanceof Integer) {
				getPreferenceStore().setDefault(preference.getId(),
						(Integer) defaultValue);
			} else if (defaultValue instanceof Double) {
				getPreferenceStore().setDefault(preference.getId(),
						(Double) defaultValue);
			} else {
				getPreferenceStore().setDefault(preference.getId(),
						defaultValue.toString());
			}
		}

		RahmenwerkActivator.plugin = this;
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		RahmenwerkActivator.plugin = null;
		super.stop(context);
	}

	/**
	 * liefert die Verwaltung der Perspektiven.
	 * 
	 * @return die Verwaltung
	 */
	public RwPerspectives getRwPerspectives() {
		if (rwPerspectives == null) {
			rwPerspectives = new RwPerspectives();
		}

		return rwPerspectives;
	}

	/**
	 * liefert die Standard-Perspektive-ID des Plugins ( Rahmenwerks).
	 * 
	 * @return die ID
	 */
	public String getDefaultPerspectiveId() {
		String result = getPreferenceStore().getString(
				RahmenwerkPreference.START_PERSPEKTIVE.getId());

		if (!getRwPerspectives().hasPerspective(result)) {
			result = (String) RahmenwerkPreference.START_PERSPEKTIVE
					.getDefaultValue();
		}

		return result;
	}
}
