/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw;

import java.util.Arrays;

import org.eclipse.core.runtime.Platform;
import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.EclipseContextFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.osgi.service.datalocation.Location;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.ui.internal.RahmenwerkWorkbenchAdvisor;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hauptklasse der Rahmenwerk-Applikation.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RahmenwerkApplication.java 50088 2014-04-09 12:30:25Z peuker $
 */
public final class RahmenwerkApplication implements IApplication {

	@Override
	public Object start(final IApplicationContext context) {

		final String[] applicationArgs = Platform.getApplicationArgs();
		Debug.init(
				"Rahmenwerk",
				new ArgumentList(Arrays.copyOf(applicationArgs,
						applicationArgs.length)));

		final Display display = PlatformUI.createDisplay();

		/* Zugriff auf den PreferenceStore der Platform. */
		final IPreferenceStore praeferenzen = PlatformUI.getPreferenceStore();
		final Location installLocation = Platform.getInstallLocation();

		/* Installationspfad des Rahmenwerkes ermitteln. */
		final String path = installLocation.getURL().getPath();

		/* Verzeichnis fr Voreinstellungen setzen. */
		final String einstellungenPfad = path + "einstellungen" + "/";
		praeferenzen.setDefault("PfadEinstellungen", einstellungenPfad);

		try {
			final IEclipseContext eclipseContext = EclipseContextFactory
					.getServiceContext(RahmenwerkActivator.getDefault()
							.getBundle().getBundleContext());

			final int returnCode = PlatformUI.createAndRunWorkbench(display,
					ContextInjectionFactory.make(
							RahmenwerkWorkbenchAdvisor.class, eclipseContext));

			if (returnCode == PlatformUI.RETURN_RESTART) {
				return IApplication.EXIT_RESTART;
			}
			return IApplication.EXIT_OK;
		} finally {
			if (!display.isDisposed()) {
				display.dispose();
			}
		}
	}

	@Override
	public void stop() {
		// Keine Aktionen erforderlich, die werden im
		// RahmenwerkWorkbenchAdvisor ausgefhrt
	}
}
