/*
 * Rahmenwerk-Plug-in "Systemkalender"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen;

import de.bsvrz.buv.rw.basislib.berechtigung.Berechtigungen;
import de.bsvrz.buv.rw.basislib.berechtigung.FunktionMitBerechtigung;

/**
 * Die Definition der Funktionen, fr die Berechtigungen innerhalb des
 * Rahmenwerkes festgelegt werden knnen. Die Funktionen sind erst ausfhrbar,
 * nachdem das Plug-in aktiviert wurde. (Definiert per plugin.xml)
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * 
 * @version $Id: BerechtigungenBerechtigungsFunktion.java 48338 2014-01-24 10:06:42Z peuker $
 */
public enum BerechtigungenBerechtigungsFunktion {

	/** Aktion zum ndern der Berechtigungen. */
	BERECHTIGUNG_AENDERN("de.bsvrz.buv.rw.rw.berechtigungen.aendern");

	/** die ID mit der Funktion registriert wird. */
	private String id;

	/**
	 * Konstruktor, erzeugt eine Funktionsdefinition mit der gegebenen ID.
	 * 
	 * Die ID wird ber den Extension-Point um die Plugin-ID erweitert (Prfix),
	 * deshalb wird diese hier ergnzt.
	 * 
	 * @param id
	 *            die ID
	 */
	private BerechtigungenBerechtigungsFunktion(final String id) {
		this.id = RwBerechtigungen.PLUGIN_ID + "." + id;
	}

	/**
	 * liefert die Funktion aus der Berechtigungsverwaltung.
	 * 
	 * @return die Funktion oder <code>null</code>, wenn keine solche existiert
	 */
	public FunktionMitBerechtigung getFunktionMitBerechtigung() {
		return BerechtigungenImpl.getInstance().getFunktion(id);
	}

	/**
	 * liefert die Id der Funktion.
	 * 
	 * @return die Id
	 */
	public String getId() {
		return id;
	}

	/**
	 * liefert den Freigabestatus der Funktion fr den angemeldeten Nutzer.
	 * 
	 * @return den Status
	 */
	public boolean isFreigegeben() {
		final Berechtigungen berechtigungen = BerechtigungenImpl.getInstance();
		return berechtigungen.hasBerechtigung(berechtigungen.getFunktion(id));
	}
}
