/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen.editor;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;

/**
 * Sortierer fr die Funktionen in der Berechtigungsmatrix.
 * 
 * Die Pseudofunktion "Alle" befindet sich immer an erster Position, der
 * Container fr nicht zuordenbare Funktionen wird ans Ende der Liste gesetzt.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungenSorter.java 49945 2014-04-02 12:44:01Z peuker $
 */
public final class BerechtigungenSorter extends ViewerSorter {
	@Override
	public int compare(final Viewer viewer, final Object e1, final Object e2) {
		if (e1 instanceof WildcardFunktion) {
			return -1;
		}

		if (e2 instanceof WildcardFunktion) {
			return 1;
		}

		if ((e1 instanceof PluginMitBerechtigungen)
				&& (e2 instanceof PluginMitBerechtigungen)) {
			if (!((PluginMitBerechtigungen) e1).isPlugin()) {
				return 1;
			} else if (!((PluginMitBerechtigungen) e2).isPlugin()) {
				return -1;
			}
		}

		return super.compare(viewer, e1, e2);
	}
}
