/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen.editor;

/**
 * Schlssel zum Zugriff auf die Berechtigungsmatrix.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungsKey.java 45978 2013-09-13 11:41:11Z peuker $
 */
public class BerechtigungsKey implements Comparable<BerechtigungsKey> {
	private final BerechtigungsKlasse klasse;
	private final Funktion funktion;

	/**
	 * Konstruktor, erzeugt einen Schlssel dfr die bergebene Kompination aus
	 * Berchtigungsklasse und Funktion.
	 * 
	 * @param klasse
	 *            die Klasse
	 * @param funktion
	 *            die Funktion
	 */
	public BerechtigungsKey(final BerechtigungsKlasse klasse,
			final Funktion funktion) {
		super();
		this.klasse = klasse;
		this.funktion = funktion;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result)
				+ ((funktion == null) ? 0 : funktion.hashCode());
		result = (prime * result) + ((klasse == null) ? 0 : klasse.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final BerechtigungsKey other = (BerechtigungsKey) obj;
		if (funktion == null) {
			if (other.funktion != null) {
				return false;
			}
		} else if (!funktion.equals(other.funktion)) {
			return false;
		}
		if (klasse == null) {
			if (other.klasse != null) {
				return false;
			}
		} else if (!klasse.equals(other.klasse)) {
			return false;
		}
		return true;
	}

	@Override
	public int compareTo(final BerechtigungsKey o) {
		int result = klasse.getId().compareTo(o.klasse.getId());
		if (result == 0) {
			result = funktion.getId().compareTo(o.funktion.getId());
		}
		return result;
	}

	/**
	 * liefert die zugeordete Berechtigungsklasse.
	 * 
	 * @return die Klasse
	 */
	public BerechtigungsKlasse getKlasse() {
		return klasse;
	}

	/**
	 * liefert die zugeordnete Funktion.
	 * 
	 * @return die Funktion
	 */
	public Funktion getFunktion() {
		return funktion;
	}

	@Override
	public String toString() {
		return klasse.toString() + ":" + funktion.toString();
	}
}
