/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen.editor;

import de.bsvrz.buv.rw.basislib.berechtigung.FunktionMitBerechtigung;

/**
 * Reprsentiert einer Funktion in den Contentprovidern zur Darstellung der
 * aktuellen Berechtigungen und im Berechtigungseditor.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: Funktion.java 45978 2013-09-13 11:41:11Z peuker $
 */
public class Funktion {

	private final String id;
	private final FunktionMitBerechtigung funktionMitBerechtigung;

	/**
	 * Konstruktor, erzeugt eine Reprsentation fr die Funktion mit der
	 * bergebenen Id.
	 * 
	 * @param id
	 *            die Id
	 */
	public Funktion(final String id) {
		this.id = id;
		this.funktionMitBerechtigung = null;
	}

	/**
	 * Konstruktor, erzeugt eine Reprsentation fr die bergebene Funktion aus
	 * der Berechtigungsverwaltung.
	 * 
	 * @param funktion
	 *            die Funktion
	 */
	public Funktion(final FunktionMitBerechtigung funktion) {
		this.id = funktion.getId();
		funktionMitBerechtigung = funktion;
	}

	/**
	 * liefert die Id der Funktion.
	 * 
	 * @return die Id
	 */
	public String getId() {
		return id;
	}

	/**
	 * liefert die in der Berechtigungsverwaltung regsitrierte Funktion oder
	 * <code>null</code>, wenn keine registriert wurde.
	 * 
	 * @return die Funktion oder <code>null</code>
	 */
	public FunktionMitBerechtigung getFunktionMitBerechtigung() {
		return funktionMitBerechtigung;
	}

	@Override
	public String toString() {
		if (funktionMitBerechtigung == null) {
			return id;
		}

		return funktionMitBerechtigung.getBezeichnung();
	}

	/**
	 * die Funktion ist verfgbar, wenn eine entsprechende Funktion in der
	 * Berechtigungsverwaltung hinterlegt ist.
	 * 
	 * @return die Verfgbarkeit
	 */
	public boolean isAvailable() {
		return funktionMitBerechtigung != null;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result) + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Funktion other = (Funktion) obj;
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		return true;
	}
}
