/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen.views;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.ColumnViewerEditor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.nebula.jface.gridviewer.GridTreeViewer;
import org.eclipse.nebula.jface.gridviewer.GridViewerColumn;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbenchCommandConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.CollapseAllHandler;
import org.eclipse.ui.handlers.ExpandAllHandler;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.berechtigung.BerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.Berechtigungen;
import de.bsvrz.buv.rw.basislib.berechtigung.IBerechtigungListener;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DefaultDavVerbindungsListener;
import de.bsvrz.buv.rw.rw.berechtigungen.BerechtigungenImpl;
import de.bsvrz.buv.rw.rw.berechtigungen.RwBerechtigungen;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.BerechtigungenFilter;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.BerechtigungenSorter;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.BerechtigungsGruppenHeaderRenderer;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Ansicht zur Darstellung der aktuell von der {@link Berechtigungen}-Verwaltung
 * verffentlichten Berechtigungen.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: AktuelleBerechtigungenView.java 51067 2014-07-30 09:23:27Z peuker $
 */
public class AktuelleBerechtigungenView extends ViewPart {

	/** Handler fr spezielle Tastaturcodes innerhalb des Grids. */
	private final class BerechtigungenAnsichtKeyHandler extends KeyAdapter {

		private Object zeilenObjekt;
		private int columnIndex = -1;

		@Override
		public void keyPressed(final KeyEvent e) {

			zeilenObjekt = null;
			columnIndex = -1;

			final ColumnViewerEditor editor = matrix.getColumnViewerEditor();
			if ((editor != null) && (editor.getFocusCell() != null)) {
				columnIndex = editor.getFocusCell().getColumnIndex();
				zeilenObjekt = ((IStructuredSelection) matrix.getSelection())
						.getFirstElement();
			}

			if ((columnIndex == 0)
					&& ((e.keyCode == SWT.SPACE) || (e.keyCode == SWT.CR))) {
				if (matrix.isExpandable(zeilenObjekt)
						&& (!matrix.getExpandedState(zeilenObjekt))) {
					matrix.expandToLevel(zeilenObjekt,
							AbstractTreeViewer.ALL_LEVELS);
				} else {
					matrix.collapseToLevel(zeilenObjekt,
							AbstractTreeViewer.ALL_LEVELS);
				}
			}
		}
	}

	/** die ID des Views. */
	public static final String VIEW_ID = "de.bsvrz.buv.rw.rw.berechtigungen.views.aktuell";

	private BerechtigungsGruppenHeaderRenderer headerRenderer;
	private GridTreeViewer matrix;

	@Override
	public void createPartControl(final Composite parent) {

		final Composite panel = new Composite(parent, SWT.NONE);
		panel.setLayout(new GridLayout());

		final Label label = new Label(panel, SWT.NONE);
		label.setText("Berechtigungsverwaltung: "
				+ (BerechtigungenImpl.getInstance()
						.isBerechtigungsKonzeptAktiv() ? "AKTIV"
						: "DEAKTIVIERT"));
		GridDataFactory.fillDefaults().grab(true, false).applyTo(label);

		matrix = new GridTreeViewer(panel);
		GridDataFactory.fillDefaults().grab(true, true)
				.applyTo(matrix.getControl());

		matrix.getGrid().setHeaderVisible(true);
		matrix.getGrid().setLinesVisible(true);
		matrix.getGrid().setCellSelectionEnabled(true);
		matrix.setSorter(new BerechtigungenSorter());
		matrix.setFilters(new ViewerFilter[] { new BerechtigungenFilter() });
		matrix.setContentProvider(new BerechtigungenContentProvider());

		matrix.getGrid().addKeyListener(new BerechtigungenAnsichtKeyHandler());

		BerechtigungenImpl.getInstance().addOberflaechenFunktionsListener(
				new IBerechtigungListener() {

					@Override
					public void sperrung(final BerechtigungEreignis e) {
						updateMatrix();
					}

					@Override
					public void freigabe(final BerechtigungEreignis e) {
						updateMatrix();
					}
				});

		BerechtigungenImpl.getInstance().getRahmenwerk()
				.addDavVerbindungsListener(new DefaultDavVerbindungsListener() {

					@Override
					public void verbindungHergestellt(
							final DavVerbindungsEvent event) {
						createColumns();
					}

					@Override
					public void verbindungGetrennt(
							final DavVerbindungsEvent event) {
						createColumns();
					}
				});

		headerRenderer = new BerechtigungsGruppenHeaderRenderer();
		createColumns();
		updateMatrix();

		final IHandlerService service = (IHandlerService) getSite().getService(
				IHandlerService.class);
		if (service != null) {
			service.activateHandler(IWorkbenchCommandConstants.NAVIGATE_EXPAND_ALL,
					new ExpandAllHandler(matrix));
			service.activateHandler(IWorkbenchCommandConstants.NAVIGATE_COLLAPSE_ALL,
					new CollapseAllHandler(matrix));
		}

		PlatformUI
				.getWorkbench()
				.getHelpSystem()
				.setHelp(matrix.getControl(),
						RwBerechtigungen.PLUGIN_ID + ".berechtigungen_ansicht");

	}

	private void createColumns() {
		if ((matrix != null) && (!matrix.getControl().isDisposed())) {
			matrix.getControl().getDisplay().asyncExec(new Runnable() {
				@Override
				public void run() {
					if ((matrix != null) && (!matrix.getControl().isDisposed())) {

						final BerechtigungenImpl verwaltung = BerechtigungenImpl
								.getInstance();

						for (final GridColumn column : matrix.getGrid()
								.getColumns()) {
							column.dispose();
						}

						GridViewerColumn column = new GridViewerColumn(matrix,
								SWT.NONE);

						column.getColumn().setWidth(300);
						column.getColumn().setText("Funktionen");
						column.getColumn()
								.setHeaderTooltip(
										"Die Funktionen, denen Berechtigungen zugewiesen werden knnen");
						column.setLabelProvider(new BerechtigungsLabelProvider());
						column.getColumn().setTree(true);

						final List<SystemObject> berechtigungsKlassen = new ArrayList<SystemObject>(
								verwaltung.getBerechtigungsKlassen());
						Collections.sort(berechtigungsKlassen,
								new Comparator<SystemObject>() {

									@Override
									public int compare(final SystemObject o1,
											final SystemObject o2) {
										return o1.getNameOrPidOrId().compareTo(
												o2.getNameOrPidOrId());
									}
								});

						for (final SystemObject klasse : berechtigungsKlassen) {
							column = new GridViewerColumn(matrix, SWT.NONE);
							column.getColumn().setWidth(50);
							column.getColumn()
									.setHeaderRenderer(headerRenderer);
							column.getColumn().getCellRenderer()
									.setAlignment(SWT.CENTER);

							column.getColumn().setText(klasse.getName());
							column.getColumn().setHeaderTooltip(
									klasse.toString());
							column.setLabelProvider(new BerechtigungsLabelProvider(
									klasse));
						}

						matrix.getGrid().recalculateHeader();
						matrix.getGrid().getParent().layout();

						matrix.setInput(BerechtigungenImpl.getInstance());
					}
				}
			});
		}
	}

	@Override
	public void setFocus() {
		// TODO Auto-generated method stub
	}

	private void updateMatrix() {
		if ((matrix != null) && (!matrix.getControl().isDisposed())) {
			matrix.getControl().getDisplay().asyncExec(new Runnable() {
				@Override
				public void run() {
					if ((matrix != null) && (!matrix.getControl().isDisposed())) {
						matrix.refresh();
					}
				}
			});
		}
	}
}
