/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen.views;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;

import de.bsvrz.buv.rw.basislib.berechtigung.FunktionMitBerechtigung;
import de.bsvrz.buv.rw.rw.berechtigungen.BerechtigungenImpl;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.PluginMitBerechtigungen;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Labelprovider fr die Darstellung der aktuellen Berechtigungen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungsLabelProvider.java 46020 2013-09-16 08:42:05Z peuker $
 */
public class BerechtigungsLabelProvider extends ColumnLabelProvider {

	private final SystemObject klasse;

	BerechtigungsLabelProvider() {
		this(null);
	}

	BerechtigungsLabelProvider(final SystemObject klasse) {
		this.klasse = klasse;
	}

	@Override
	public void update(final ViewerCell cell) {

		final BerechtigungenImpl verwaltung = BerechtigungenImpl.getInstance();

		final Object element = cell.getElement();

		if (klasse == null) {
			if (element == null) {
				cell.setText("???");
			} else if (element instanceof FunktionMitBerechtigung) {
				cell.setText(((FunktionMitBerechtigung) element)
						.getBezeichnung());
			} else if (element instanceof PluginMitBerechtigungen) {
				cell.setText(((PluginMitBerechtigungen) element).getNameOrId());
			} else {
				cell.setText(element.toString());
			}
		} else {
			cell.setText("");
			if (element instanceof FunktionMitBerechtigung) {
				final FunktionMitBerechtigung funktion = (FunktionMitBerechtigung) element;
				final boolean freigabe = verwaltung.hasKlasseBerechtigung(
						klasse, funktion);
				cell.setForeground(cell.getControl().getDisplay()
						.getSystemColor(SWT.COLOR_WHITE));
				if (freigabe) {
					cell.setBackground(cell.getControl().getDisplay()
							.getSystemColor(SWT.COLOR_DARK_GREEN));
					cell.setText("O");
				} else {
					cell.setBackground(cell.getControl().getDisplay()
							.getSystemColor(SWT.COLOR_DARK_RED));
					cell.setText("X");
				}
			} else {
				cell.setBackground(cell.getControl().getDisplay()
						.getSystemColor(SWT.COLOR_WHITE));
			}
		}
	}
}
