/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.dav;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Singleton-Klasse verwaltet die Verbindung zum DaV. <br>
 * Es knnen sich Listener vom Typ {@link IDaVVerbindungZuhoerer} registrieren,
 * um ber nderungen an der Verbindung zum DaV informiert zu werden.
 * 
 * @version $Revision: 1.10 $ / $Date: 2012/11/23 10:38:26 $ / ($Author:
 *          HofmannJ $)
 * 
 * @author HofmannJ
 * @deprecated die Klasse sollte durch das Interface {@link Verbindung} ersetzt
 *             werden, welches ber einen entsprechenden Service bereitgestellt
 *             wird.
 */
@Deprecated
public final class DaVVerbindung {

	private static class VerbindungZuhoererProxy implements
			DavVerbindungsListener {

		private final IDaVVerbindungZuhoerer zuhoerer;

		public VerbindungZuhoererProxy(final IDaVVerbindungZuhoerer zuhoerer) {
			this.zuhoerer = zuhoerer;
		}

		@Override
		public void verbindungHergestellt(final DavVerbindungsEvent event) {
			zuhoerer.verbindungAufgebaut(new DaVVerbindungEreignis(event
					.getDavVerbindung(), ""));
		}

		@Override
		public void verbindungGetrennt(final DavVerbindungsEvent event) {
			zuhoerer.verbindungAbgebaut(new DaVVerbindungEreignis(event
					.getSource(), ""));
		}

		@Override
		public boolean verbindungHalten(final DavVerbindungsEvent event) {
			return false;
		}

	}

	private static class VerbindungWartenProxy implements
			DavVerbindungsListener {

		private final IDaVVerbindungsAbbauWarten warten;

		public VerbindungWartenProxy(final IDaVVerbindungsAbbauWarten warten) {
			this.warten = warten;
		}

		@Override
		public void verbindungHergestellt(final DavVerbindungsEvent event) {
			// wird hier ignoriert
		}

		@Override
		public void verbindungGetrennt(final DavVerbindungsEvent event) {
			// wird hier ignoriert
		}

		@Override
		public boolean verbindungHalten(final DavVerbindungsEvent event) {
			return warten.davVerbindungsAbbauOk();
		}

	}

	private static DaVVerbindung instance;
	private static final Debug LOGGER = Debug.getLogger();

	private final Map<IDaVVerbindungZuhoerer, DavVerbindungsListener> zuhoererMap = new HashMap<IDaVVerbindungZuhoerer, DavVerbindungsListener>();
	private final Map<IDaVVerbindungsAbbauWarten, DavVerbindungsListener> warterMap = new HashMap<IDaVVerbindungsAbbauWarten, DavVerbindungsListener>();

	/**
	 * @return liefert eine Instanz der Datenverteilerverbindung des
	 *         Rahmenwerks.
	 * @deprecated ist nur aus Kompatibilittsgrnden noch vorhanden. Die
	 *             Verbindung sollte per Service ber ein verfgbares Rahmenwerk
	 *             ermittelt werden.
	 */
	@Deprecated
	public static DaVVerbindung getInstanz() {
		if (DaVVerbindung.instance == null) {
			DaVVerbindung.instance = new DaVVerbindung();
		}
		return DaVVerbindung.instance;
	}

	private DaVVerbindung() {
	}

	/**
	 * Verbindungsaufbau zum DaV.
	 * 
	 * @return Liefert <code>true</code>, wenn die Verbindung zum DaV aufgebaut
	 *         werden konnte, sonst <code>false</code>.
	 * @deprecated die Datenverteilerverbindung wird vom Rahmenwerk geliefert
	 */
	@Deprecated
	public boolean baueVerbindungMitDaVAuf() {
		DaVVerbindung.LOGGER
				.warning("Die Datenverteilerverbindung wird nur intern vom Rahmenwerk aufgebaut!");
		return false;
	}

	/**
	 * Urlasserverbindungen werden nur innerhalb des Rahmenwerks selbst
	 * verwendet und sollten kein Bestandteil der ffentlichen Schnittstelle
	 * sein.
	 * 
	 * @deprecated der Aufbau der Urlasserverbindung erfolgt nur intern und
	 *             sollte nict vom Client ausgefhrt werden
	 */
	@Deprecated
	public boolean baueUrlasserVerbindungMitDaVAuf() {
		DaVVerbindung.LOGGER
				.warning("Urlasserverbindungen sollten nur intern vom Rahmenwerk aufgebaut werden!");
		return false;
	}

	/**
	 * Trennt die Verbindung zum DaV. Dies ist nur mglich, wenn keine
	 * Urlasser-Verbindung zum DaV aktiv ist.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn keine Benutzerverbindung
	 *         zum DaV mehr existiert bzw. wenn alle Verbindungen zum DaV
	 *         geschlossen werden konnten.
	 * @deprecated die Datenverteilerverbindung wird vom Rahmenwerk geliefert
	 *             und gegebenfalls getrennt
	 */
	@Deprecated
	public boolean trenneDaVVerbindung() {
		DaVVerbindung.LOGGER
				.warning("Die Datenverteilerverbindung wird nur intern vom Rahmenwerk getrennt!");
		return false;
	}

	/**
	 * Trennt die Verbindung zum DaV. Dies ist nur mglich, wenn keine
	 * Urlasser-Verbindung zum DaV aktiv ist.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn keine Benutzerverbindung
	 *         zum DaV mehr existiert bzw. wenn alle Verbindungen zum DaV
	 *         geschlossen werden konnten.
	 * 
	 * @deprecated SWT-Thread-Probleme sollten ohnehin behandelt werden eine
	 *             spezielle Funktion dafr zu verwenden ist nicht erforderlich
	 */
	@Deprecated
	public boolean trenneDaVVerbindungSWTThreadsicher() {
		return trenneDaVVerbindung();
	}

	/**
	 * Trennt die Urlasserverbindung zum DaV.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn die Urlasser-Verbindung
	 *         zum DaV beendet wurde.
	 * 
	 * @deprecated Urlasserverbindungen sollten nur intern verwendet werden und
	 *             nicht Bestandteil der ffentlichen Schnittstelle sein
	 */
	@Deprecated
	public boolean trenneUrlasserDaVVerbindung() {
		DaVVerbindung.LOGGER
				.warning("Urlasserverbindungen sollten nur intern vom Rahmenwerk getrennt werden!");
		return false;
	}

	/**
	 * Trennt die Urlasserverbindung zum DaV.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn die Urlasser-Verbindung
	 *         zum DaV beendet wurde.
	 * @deprecated SWT-Thread-Probleme sollten ohnehin behandelt werden eine
	 *             spezielle Funktion dafr zu verwenden ist nicht erforderlich
	 */
	@Deprecated
	public boolean trenneUrlasserDaVVerbindungSWTThreadsicher() {
		return trenneUrlasserDaVVerbindung();
	}

	/**
	 * Trennt alle Verbindungen zum DaV.
	 * 
	 * @return Liefert <code>true</code>, wenn Benutzer- und Urlasserverbindung
	 *         zum DaV getrennt werden konnten, sonst <code>false</code>.
	 * 
	 * @deprecated wenn Urlasserbverbindungen nur intern behandelt werden, ist
	 *             diese Funktion nicht erforderlich
	 */
	@Deprecated
	public boolean trenneAlleVerbindungen() {
		return trenneDaVVerbindung();
	}

	/**
	 * Trennt alle Verbindungen zum DaV.
	 * 
	 * @return Liefert <code>true</code>, wenn Benutzer- und Urlasserverbindung
	 *         zum DaV getrennt werden konnten, sonst <code>false</code>.
	 * @deprecated SWT-Thread-Probleme sollten ohnehin behandelt werden eine
	 *             spezielle Funktion dafr zu verwenden ist nicht erforderlich
	 */
	@Deprecated
	public boolean trenneAlleVerbindungenSWTThreadsicher() {
		return trenneAlleVerbindungen();
	}

	/**
	 * Prft, ob die Verbindung zum DaV aktiv ist.
	 * 
	 * @return Liefert <code>true</code>, wenn die Verbindung zum DaV aktiv ist,
	 *         sonst <code>false</code>.
	 */

	public boolean istDaVVerbindungAktiv() {
		if (RahmenwerkService.getService() != null) {
			return RahmenwerkService.getService().getRahmenWerk().isOnline();
		}

		return false;
	}

	/**
	 * Liefert die Verbindung zum DaV oder <code>null</code>, wenn keine
	 * Verbindung zum DaV exitiert.
	 * 
	 * @return DaV-Verbindung.
	 * @deprecated wird durch {@link #getClientDavInterface()} ersetzt, da keine
	 *             Instanz einer {@link DaVVerbindung} geliefert wird.
	 */
	@Deprecated
	public ClientDavInterface getDaVVerbindung() {
		return getClientDavInterface();
	}

	/**
	 * Liefert die Verbindung zum DaV oder <code>null</code>, wenn keine
	 * Verbindung zum DaV exitiert.
	 * 
	 * @return DaV-Verbindung.
	 */

	public ClientDavInterface getClientDavInterface() {
		if (RahmenwerkService.getService() != null) {
			return RahmenwerkService.getService().getRahmenWerk()
					.getDavVerbindung();
		}
		return null;
	}

	/**
	 * Fgt einen Zuhrer, der ber den Status der Verbindung zum DaV informiert
	 * werden mchte, zur Liste der Zuhrer hinzu.
	 * 
	 * @param zuhoerer
	 *            Zuhrer, der zur Liste der Zuhrer hinzugefgt werden soll.
	 */

	public void hinzufuegenZuhoerer(final IDaVVerbindungZuhoerer zuhoerer) {
		final VerbindungZuhoererProxy proxy = new VerbindungZuhoererProxy(
				zuhoerer);
		zuhoererMap.put(zuhoerer, proxy);
		RahmenwerkService.getService().getRahmenWerk()
				.addDavVerbindungsListener(proxy);
	}

	/**
	 * Entfernt einen Zuhrer von der Liste der Zuhrer.
	 * 
	 * @param zuhoerer
	 *            Zu entfernender Zuhrer.
	 */

	public void entferneZuhoerer(final IDaVVerbindungZuhoerer zuhoerer) {
		final DavVerbindungsListener listener = zuhoererMap.remove(zuhoerer);
		if (listener != null) {
			RahmenwerkService.getService().getRahmenWerk()
					.removeDavVerbindungsListener(listener);
		}
	}

	/**
	 * Fgt einen Klasse zur Liste der Klassen hinzu die geprft werden mssen,
	 * bevor die Verbindung abgebaut werden kann.
	 * 
	 * @param warten
	 *            Instanz einer Klasse, die das Interface
	 *            {@link IDaVVerbindungsAbbauWarten} implementiert.
	 */

	public void hinzufuegenVerbindungsabbauWarten(
			final IDaVVerbindungsAbbauWarten warten) {
		final VerbindungWartenProxy proxy = new VerbindungWartenProxy(warten);
		warterMap.put(warten, proxy);
		RahmenwerkService.getService().getRahmenWerk()
				.addDavVerbindungsListener(proxy);
	}

	/**
	 * Entfernt eine Klasse von der Liste der Klassen die geprft werdne mssen,
	 * bevor die Verbindung abgebaut werden kann.
	 * 
	 * @param warten
	 *            Instanz einer Klasse, die das Interface
	 *            {@link IDaVVerbindungsAbbauWarten} implementiert.
	 */

	public void entfernenVerbindungsabbauWarten(
			final IDaVVerbindungsAbbauWarten warten) {
		final DavVerbindungsListener listener = warterMap.remove(warten);
		if (listener != null) {
			RahmenwerkService.getService().getRahmenWerk()
					.removeDavVerbindungsListener(listener);
		}
	}

	/**
	 * Holt fr die DaV-Verbindung automatisch die Verbindungsdaten aus
	 * {@link BenutzerAngemeldet}.
	 * 
	 * @return Liefert <code>false</code>, falls aus {@link BenutzerAngemeldet}
	 *         die notwendigen Daten nicht erfolgreich bezogen werden konnten
	 *         oder bereits eine Benutzerverbindung zum DaV besteht, sonst
	 *         <code>true</code>.
	 * 
	 * @deprecated die Funktion kann entfallen, weil das Singleton
	 *             {@link BenutzerAngemeldet} obsolet wird, da es gengt die
	 *             Anmeldungsdaten in der entsprechenden Instanz der
	 *             {@link DaVVerbindung} vorzuhalten, ansonsten mssen stndig
	 *             mindestens zwei Instanzen mit Benutzerinformationen
	 *             konsistent gehalten werden.
	 */
	@Deprecated
	public boolean holeBenutzerDaVVerbindung() {
		return true;
	}

	/**
	 * Holt fr die Urlasser-DaV-Verbindung automatisch die Verbindungsdaten aus
	 * {@link UrlasserAngemeldet}.
	 * 
	 * @return Liefert <code>flase</code>, falls aus {@link UrlasserAngemeldet}
	 *         die notwendigen Daten nicht erfolgreich bezogen werden konnten
	 *         oder bereits eine Urlasserverbindung zum DaV besteht, sonst
	 *         <code>true</code>.
	 * @deprecated die Funktion kann entfallen, weil das Singleton
	 *             {@link UrlasserAngemeldet} obsolet wird, da es gengt die
	 *             Anmeldungsdaten in der entsprechenden Instanz der
	 *             {@link DaVVerbindung} vorzuhalten, ansonsten mssen stndig
	 *             mindestens zwei Instanzen mit Benutzerinformationen
	 *             konsistent gehalten werden.
	 */
	@Deprecated
	public boolean holeUrlasserDaVVerbindung() {
		DaVVerbindung.LOGGER
				.warning("Urlasserverbindungen sollten nur intern verwendet werden!");
		return true;
	}

	/**
	 * Liefert die IP-Adresse vom DaV.
	 * 
	 * @return DaV IP-Adresse.
	 * @deprecated ersetzt durch {@link #getDaVHost()}, da nicht immer eine
	 *             IP-Adresse angegeben wird.
	 */
	@Deprecated
	public String getDaVIp() {
		return getDaVHost();
	}

	public String getDaVHost() {
		if (RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			return RahmenwerkService.getService().getRahmenWerk()
					.getDavVerbindung().getClientDavParameters()
					.getDavCommunicationAddress();
		}
		return null;
	}

	/**
	 * Setzt die IP-Adresse vom DaV, solange noch keine Verbindung existiert.
	 * 
	 * @param daVIp
	 *            DaV IP-Adresse.
	 * @throws IllegalArgumentException
	 *             Wird geworfen, wenn eine Verbindung zum DaV besteht.
	 * @deprecated ersetzt durch {@link #setDaVHost(String)}, da nicht immer
	 *             eine IP-Adresse angegeben wird.
	 */
	@Deprecated
	public void setDaVIp(final String daVIp) {
		DaVVerbindung.LOGGER
				.warning("Die Hostadresse der Datenverteilerverbindung wird nur intern gesetzt!");
	}

	@SuppressWarnings("unused")
	public void setDaVHost(final String host) {
		DaVVerbindung.LOGGER
				.warning("Die Hostadresse der Datenverteilerverbindung wird nur intern gesetzt!");
	}

	/**
	 * Liefert den Port vom DaV.
	 * 
	 * @return DaV-Port.
	 */

	public int getDaVPort() {
		if (RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			return RahmenwerkService.getService().getRahmenWerk()
					.getDavVerbindung().getClientDavParameters()
					.getDavCommunicationSubAddress();
		}
		return -1;
	}

	/**
	 * Setzt den DaV-Port, solange noch keine Verbindung exisitert.
	 * 
	 * @param daVPort
	 *            DaV-Port.
	 * @throws IllegalArgumentException
	 *             Wird geworfen, wenn eine Verbindung zum DaV besteht.
	 */

	public void setDaVPort(final int daVPort) {
		DaVVerbindung.LOGGER
				.warning("Der Port der Datenverteilerverbindung wird nur intern gesetzt!");
	}

	/**
	 * Liefert den DaV Benutzernamen.
	 * 
	 * @return DaV Benutzernamen
	 */

	public String getDaVBenutzer() {
		return RahmenwerkService.getService().getRahmenWerk().getBenutzerName();
	}

	/**
	 * Setzt den DaV Benutzernamen, solange noch keine Verbindung zum DaV
	 * existiert.
	 * 
	 * @param daVBenutzer
	 *            DaV Benutzernamen.
	 * @throws IllegalArgumentException
	 *             Wird geworfen, wenn eine Verbindung zum DaV besteht.
	 */

	public void setDaVBenutzer(final String daVBenutzer) {
		DaVVerbindung.LOGGER
				.warning("Der Name des Datenverteilernutzers wird nur intern gesetzt!");
	}

	/**
	 * Liefert das Passwort vom DaV.
	 * 
	 * @return DaV Passwort.
	 */
	public String getDaVPasswort() {
		return RahmenwerkService.getService().getRahmenWerk().getPasswort();
	}

	/**
	 * Setzt das Passwort zum DaV, solange noch keine Verbindung zum DaV
	 * existiert.
	 * 
	 * @param daVPasswort
	 *            DaV Passwort.
	 * @throws IllegalArgumentException
	 *             Wird geworfen, wenn eine Verbindung zum DaV besteht.
	 */

	public void setDaVPasswort(final String daVPasswort) {
		DaVVerbindung.LOGGER
				.warning("Der Passwort des Datenverteilernutzers wird nur intern zum Verbindungsaufbau verwendet!");
	}

	/**
	 * @deprecated der Urlasserbenutzer sollte nicht Teil der ffentlichen
	 *             Schnittstelle der Datenverteilerverbindung sein.
	 */
	@Deprecated
	public String getUrlasserBenutzer() {
		DaVVerbindung.LOGGER
				.warning("Urlasserverbindungen sollten nur intern verwendet werden!");
		return null;
	}

	/**
	 * @deprecated der Urlasserbenutzer sollte nicht Teil der ffentlichen
	 *             Schnittstelle der Datenverteilerverbindung sein.
	 */
	@SuppressWarnings("unused")
	@Deprecated
	public void setUrlasserBenutzer(final String urlasserBenutzer) {
		DaVVerbindung.LOGGER
				.warning("Urlasserverbindungen sollten nur intern verwendet werden!");
	}

	/**
	 * @deprecated das Passwort sollte nicht so einfach auslesbar sein,
	 *             ansonsten knnte jedes Dritt-Plugin diese Information
	 *             abrufen.
	 */
	@Deprecated
	public String getUrlasserPasswort() {
		DaVVerbindung.LOGGER
				.warning("Urlasserverbindungen sollten nur intern verwendet werden!");
		throw new UnsupportedOperationException(
				"Abruf des Passwort stellt ein Sicherheitsproblem dar!");
	}

	/**
	 * @deprecated der Urlasserbenutzer sollte nicht Teil der ffentlichen
	 *             Schnittstelle der Datenverteilerverbindung sein.
	 */
	@SuppressWarnings("unused")
	@Deprecated
	public void setUrlasserPasswort(final String urlasserPasswort) {
		DaVVerbindung.LOGGER
				.warning("Urlasserverbindungen sollten nur intern verwendet werden!");
	}

	/**
	 * @deprecated wird nicht mehr bentigt, scheint nur eine Krcke gewesen zu
	 *             sein, um den Tray-Ballon zu behandeln.
	 */
	@SuppressWarnings("unused")
	@Deprecated
	public void setzeArbeitsbereich(final boolean wert) {
		DaVVerbindung.LOGGER
				.warning("Der Arbeitsbereich ist nicht Bestandteil der Datenverteilerverbindung!");
	}

	/**
	 * @deprecated sollte bei Bedarf ber eine direkte Abfrage des
	 *             Workbench-Zustandes umgesetzt werden
	 */

	@Deprecated
	public boolean istArbeitsbereichAktiv() {
		DaVVerbindung.LOGGER
				.warning("Der Arbeitsbereich ist nicht Bestandteil der Datenverteilerverbindung!");
		return true;
	}

	/**
	 * @deprecated sollte nicht von auen gesetzt werden, sondern wird durch die
	 *             angemeldeten Clients entschieden
	 *             {@link IDaVVerbindungsAbbauWarten}
	 */
	@SuppressWarnings("unused")
	@Deprecated
	public void setTrennenOk(final boolean status) {
		DaVVerbindung.LOGGER
				.warning("Die Trennung der Datenverteilerverbindung kann nur durch entsprechende Listener verhindert werden!");
	}
}
