/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GMBH
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225 / 64-03
 * Fax: +49 (0)7225 / 64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.rw.rw.dav.datenbasis;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeMap;

import org.eclipse.core.runtime.IProgressMonitor;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Diese Klasse liefert die Basis fr GTM-artige Darstellungen der Daten des
 * DaV. Es knnen entweder alle Objekte des jeweiligen DaV dargestellt werden
 * (durch einen Aufruf von {@link GtmDatenBasis#setAll()}) oder Systemobjekte
 * von bestimmten Objekttypen ({@link GtmDatenBasis#setTypes(List)}). Es sind
 * Getter fr SystemObjekte, SystemObjektTypen, Attributgruppen und Aspekte
 * vorhanden. Die Methode {@link GtmDatenBasis#filtere(List, List, List, List)}
 * ist in der Lage, SystemObjekte nach SystemObjektTyp, Attributgruppe und
 * Aspekt zu filtern.
 *
 * @author HofmannJ
 * @deprecated wird von Plug-ins in der Regel nach Bedarf gehandelt und wahrscheinlich nicht verwendet
 */
@Deprecated
public class GtmDatenBasis {

	/** Datenmodell des DaV. */
	private DataModel dataModel;

	/** Nimmt die SystemObjekte auf. */
	private final TreeMap<String, SystemObject> tmSystemObjects = new TreeMap<String, SystemObject>();

	/**
	 * Konstruktor.
	 *
	 * @param connection
	 *            Verbindung zum DaV
	 * @param selfInit
	 *            Wenn <code>true</code>, wird die Struktur aus allen im DaV
	 *            verfgbaren Informationen aufgebaut, bei <code>false</code>
	 *            nicht.
	 * @param monitor
	 *            Hier kann der {@link IProgressMonitor} des Jobs bergeben
	 *            werden. In diesem wird dann der aktuelle Konfigurationsbereich
	 *            ausgegeben, dessen SystemObjekte gerade geladen werden. Wenn
	 *            dieser Parameter <code>null</code> ist, so wird diese Ausgabe
	 *            unterdrckt.
	 */
	public GtmDatenBasis(final ClientDavInterface connection,
			final boolean selfInit, final IProgressMonitor monitor) {
		if (connection != null) {
			this.dataModel = connection.getDataModel();
		}
	}

	/**
	 * Fllt die Struktur mit allen vom DaV gelieferten {@link SystemObject}s
	 * und {@link SystemObjectType}s.
	 *
	 * @deprecated es wird immer auf das komplette Datenmodell zurckgegriffen
	 */
	@Deprecated
	public void setAll() {
		// es ist keine spezielle Fllung erforderlich
	}

	/**
	 * Fllt die Struktur mit den bergebenen {@link SystemObjectType}s und
	 * sucht dazu vom DaV die zugehrigen {@link SystemObject}s heraus.
	 *
	 * @param types
	 *            {@link SystemObjectType}s.
	 * @deprecated Typen knnen nicht gesetzt werden, es wird immer das
	 *             komplette Datenmodell bercksichtigt.
	 */
	@Deprecated
	public void setTypes(final List<SystemObjectType> types) {
		// die Funktion tut nichts mehr
	}

	/**
	 * Fllt die Struktur mit den bergebenen {@link SystemObject}s und bestimmt
	 * die zugehrigen {@link SystemObjectType} s.
	 *
	 * @param systemObjects
	 *            {@link SystemObject}s.
	 * @deprecated Systemobjekte knnen nicht gesetzt werden, es wird immer das
	 *             komplette Datenmodell bercksichtigt.
	 */
	@Deprecated
	public void setSystemObjects(final List<SystemObject> systemObjects) {
		// die Funktion tut nichts mehr
	}

	/**
	 * @deprecated Systemobjekte knnen nicht gesetzt werden, es wird immer das
	 *             komplette Datenmodell bercksichtigt.
	 */
	@Deprecated
	public void initAlle() {
		// die Funktion tut nichts mehr
	}

	/**
	 * Liefert eine Liste von {@link SystemObject}s, passend zu den durch die
	 * Setter getroffene Auswahl zurck.
	 *
	 * @return Liste von {@link SystemObject}s
	 */
	public List<SystemObject> getSystemObjects() {
		final Set<SystemObject> objects = new HashSet<SystemObject>();

		final List<SystemObjectType> systemObjectTypes = getSystemObjectTypes();
		for (final SystemObjectType type : systemObjectTypes) {
			objects.addAll(type.getElements());
		}

		return new ArrayList<SystemObject>(objects);
	}

	/**
	 * Liefert eine Liste von {@link SystemObjectType}s, passend zu die durch
	 * die Setter getroffene Auswahl zurck.
	 *
	 * @return
	 */
	public List<SystemObjectType> getSystemObjectTypes() {
		return new ArrayList<SystemObjectType>(
				(Collection<? extends SystemObjectType>) dataModel
						.getTypeTypeObject().getElements());
	}

	/**
	 * Bestimmt eine Liste von {@link AttributeGroup}s passend zu einer Liste
	 * von {@link SystemObjectType}s.
	 *
	 * @param listSystemObjectTypes
	 *            Liste von {@link SystemObjectType}s.
	 * @return Liste von {@link AttributeGroup}s
	 */
	public List<AttributeGroup> getAtg(final List<SystemObjectType> types) {

		final Set<AttributeGroup> atgs = new HashSet<AttributeGroup>();
		for (final SystemObjectType type : types) {
			atgs.addAll(type.getAttributeGroups());
		}

		return new ArrayList<AttributeGroup>(atgs);
	}

	/**
	 * Bestimmt zu einer Liste von {@link AttributeGroup}s eine Liste von
	 * {@link Aspect}s.
	 *
	 * @param listAttributeGroups
	 *            Liste von {@link AttributeGroup}s
	 * @return Liste von {@link Aspect}s.
	 */
	public List<Aspect> getAsp(final List<AttributeGroup> atgs) {
		final Set<Aspect> aspects = new HashSet<Aspect>();
		for (final AttributeGroup atg : atgs) {
			aspects.addAll(atg.getAspects());
		}

		return new ArrayList<Aspect>(aspects);
	}

	public LinkedList<AttributeGroup> getAlleAtg() {
		final LinkedList<AttributeGroup> result = new LinkedList<AttributeGroup>();
		if (dataModel != null) {
			result.addAll((Collection<? extends AttributeGroup>) dataModel
					.getType("typ.attributgruppe").getElements());
		}
		return result;
	}

	public LinkedList<Aspect> getAlleAsp() {
		final LinkedList<Aspect> result = new LinkedList<Aspect>();
		if (dataModel != null) {
			result.addAll((Collection<? extends Aspect>) dataModel.getType(
					"typ.aspekt").getElements());
		}
		return result;
	}

	/**
	 * Filtert eine Liste von SystemObjekten an Hand von SystemObjekttypen,
	 * Attributgruppen sowie Aspekten.
	 *
	 * @param sos
	 *            Liste von {@link SystemObject}en. Zwingend notwendig.
	 * @param types
	 *            Liste von {@link SystemObjectType}s, gegen welche die
	 *            {@link SystemObject}s gefiltert werden sollen oder
	 *            <code>null</code>, wenn nicht gegen {@link SystemObjectType}s
	 *            gefiltert werden soll.
	 * @param atgs
	 *            Liste von {@link AttributeGroup}s, gegen welche die
	 *            {@link SystemObject}s gefiltert werden sollen oder
	 *            <code>null</code>, wenn nicht gegen die {@link AttributeGroup}
	 *            s gefiltert werden soll.
	 * @param asps
	 *            Liste von {@link Aspect}s, gegen welche die
	 *            {@link SystemObject}s gefiltert werden sollen oder
	 *            <code>null</code>, wenn nicht gegen die {@link Aspect}s
	 *            gefiltert werden soll.
	 * @return Gefilterte Liste von {@link SystemObject}s. Waren die
	 *         Filtereinstellungen fr {@link SystemObjectType}s,
	 *         {@link AttributeGroup}s und {@link Aspect}s jeweils null, so wird
	 *         als Parameter vorgegebene Liste von {@link SystemObject}
	 *         (verpackt als {@link GtmListenElementObjekt}) zurckgeliefert.
	 */
	public LinkedList<SystemObject> filtere(final List<SystemObject> sos,
			final List<SystemObjectType> types,
			final List<AttributeGroup> atgs, final List<Aspect> asps) {
		final LinkedList<SystemObject> re = new LinkedList<SystemObject>();

		if ((sos != null) && (sos.size() > 0)) {
			final Iterator<SystemObject> itSystemObject = sos.iterator();
			while (itSystemObject.hasNext()) {
				final SystemObject systemObject = itSystemObject.next();

				/*
				 * Type-Prfung
				 */
				boolean typOk = false;
				if (types != null) {
					if (types.contains(systemObject.getType())) {
						typOk = true;
					}
				} else {
					typOk = true;
				}

				/*
				 * AttributGruppe
				 */
				boolean atgOk = true;
				if (typOk) {
					if (atgs != null) {
						final List<AttributeGroup> atgsSo = systemObject
								.getType().getAttributeGroups();

						final Iterator<AttributeGroup> itAtg = atgs.iterator();
						while (itAtg.hasNext()) {
							final AttributeGroup attributeGroup = itAtg.next();
							if (!atgsSo.contains(attributeGroup)) {
								atgOk = false;
								break;
							}
						}
					}
				}

				/*
				 * Aspekt
				 */
				boolean aspOk = true;

				if (typOk && atgOk) {
					if (asps != null) {
						/*
						 * Keine Attributgruppen bergeben. Es werden alle Atgs
						 * des jeweiligen Typs durchsucht!
						 */
						if (atgs == null) {
							final LinkedList<Aspect> aspectsSOIst = new LinkedList<Aspect>();

							final Iterator<AttributeGroup> itAtgSo = systemObject
									.getType().getAttributeGroups().iterator();
							while (itAtgSo.hasNext()) {
								final AttributeGroup attributeGroupSo = itAtgSo
										.next();
								final Iterator<Aspect> itAspSo = attributeGroupSo
										.getAspects().iterator();
								while (itAspSo.hasNext()) {
									final Aspect aspSo = itAspSo.next();
									if (!aspectsSOIst.contains(aspSo)) {
										aspectsSOIst.add(aspSo);
									}
								}
							}

							aspOk = sindAlleElementeInListe(asps, aspectsSOIst);
						} else {
							/*
							 * Attributgruppen bergben.
							 */
							final LinkedList<Aspect> aspectsSOIst = new LinkedList<Aspect>();
							final Iterator<AttributeGroup> itAtg = atgs
									.iterator();
							while (itAtg.hasNext()) {
								final AttributeGroup attributeGroup = itAtg
										.next();
								final Iterator<Aspect> itAsp = attributeGroup
										.getAspects().iterator();
								while (itAsp.hasNext()) {
									final Aspect aspect = itAsp.next();
									if (!aspectsSOIst.contains(aspect)) {
										aspectsSOIst.add(aspect);
									}
								}
							}

							aspOk = sindAlleElementeInListe(asps, aspectsSOIst);
						}
					}
				}

				/*
				 * Zur Liste hinzufgen
				 */
				if (typOk && atgOk && aspOk) {
					re.add(systemObject);
				}

			}
		} else {
			final Iterator<Entry<String, SystemObject>> it = tmSystemObjects
					.entrySet().iterator();
			while (it.hasNext()) {
				re.add(it.next().getValue());
			}
		}

		return re;
	}

	/**
	 * Prft, ob die Elemente aus einer Muss-Liste in einer Ist-Liste vorhanden
	 * sind.
	 *
	 * @param listMuss
	 *            Muss-Liste
	 * @param listIst
	 *            Ist-Liste
	 * @return Liefert <code>true</code>, wenn alle Elemente aus der Muss-Liste
	 *         in der Ist-Liste vorhanden sind.
	 */
	private boolean sindAlleElementeInListe(final List<Aspect> listMuss,
			final List<Aspect> listIst) {
		boolean re = true;

		final Iterator<Aspect> it = listMuss.iterator();
		while (it.hasNext()) {
			final Object aspMuss = it.next();
			if (!listIst.contains(aspMuss)) {
				re = false;
				break;
			}
		}

		return re;
	}

	/**
	 * @deprecated es wird keine Kopie erzeugt, es existiert lediglich eine
	 *             Instanz der Datenbasis.
	 */
	@Deprecated
	public GtmDatenBasis flacheKopie() {
		return this;
	}
}
