/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.einstellungen;

import de.bsvrz.buv.rw.basislib.urlasser.UrlasserInfo;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Netzweiter (Datenverteiler)-Einstellungsspeicher fr einen Benutzer.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BenutzerNetzSpeicher.java 51069 2014-07-30 13:31:37Z peuker $
 */
class BenutzerNetzSpeicher extends EinstellungsSpeicher {

	private final String pid;
	private final EinstellungenSpeichermanager manager;
	private static final Debug LOGGER = Debug.getLogger();

	public BenutzerNetzSpeicher(final String pid,
			final EinstellungenSpeichermanager manager) {
		this.pid = pid;
		this.manager = manager;
	}

	@Override
	protected void doSave(final String data, final UrlasserInfo info) throws EinstellungsSpeicherException {
		final DavConnector davConnector = manager.getDavConnector();
		if (davConnector != null) {
			try {
				davConnector.saveBenutzerEinstellungsSpeicher(pid, data, info);
			} catch (final DataNotSubscribedException e) {
				throw new EinstellungsSpeicherException(e);
			} catch (final SendSubscriptionNotConfirmed e) {
				throw new EinstellungsSpeicherException(e);
			}
		} else {
			throw new EinstellungsSpeicherException("Netzwerkweite Benutzereinstellungen knnen nicht gesichert werden, da keine Datenverteilerverbindung besteht!");
		}
	}

}
