/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */
package de.bsvrz.buv.rw.rw.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;

/**
 * Handler fr die Abmeldung des aktiven Benutzers vom verbundenen
 * Datenverteilersystem.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: AbmeldenHandler.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class AbmeldenHandler extends AbstractHandler implements
		DavVerbindungsListener {

	private final Rahmenwerk rahmenwerk;

	/** Standardkonstruktor. */
	public AbmeldenHandler() {
		rahmenwerk = RahmenwerkImpl.getInstanz();
		rahmenwerk.addDavVerbindungsListener(this);
		setBaseEnabled(rahmenwerk.isOnline());
	}

	@Override
	public Object execute(final ExecutionEvent event) throws ExecutionException {

		if (isEnabled()) {
			if (MessageDialog.openQuestion(PlatformUI.getWorkbench()
					.getDisplay().getActiveShell(),
					"Verbindung zum DaV schlieen?",
					"Mchten Sie wirklich die aktive Verbindung zum DaV \""
							+ RahmenwerkImpl.getInstanz().getDavVerbindung()
									.getClientDavParameters()
									.getDavCommunicationAddress()
							+ "\" schlieen?")) {

				RahmenwerkImpl.getInstanz().davVerbindungBeenden();
			}
		}

		return null;
	}

	@Override
	public void verbindungHergestellt(final DavVerbindungsEvent event) {
		setBaseEnabled(true);
	}

	@Override
	public void verbindungGetrennt(final DavVerbindungsEvent event) {
		setBaseEnabled(false);
	}

	@Override
	public boolean verbindungHalten(final DavVerbindungsEvent event) {
		return false;
	}
}
