/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */
package de.bsvrz.buv.rw.rw.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.nebula.paperclips.core.page.PagePrint;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.HandlerUtil;

import de.bsvrz.buv.rw.basislib.printing.RwPrintable;
import de.bsvrz.buv.rw.rw.printing.DruckvorschauDialog;

/**
 * Handler zur Aktivierung der Druckvorschau fr alle WorkbechParts, die eine
 * Druckvorschau untersttzen (die auf das Interface {@link RwPrintable}
 * adaptierbar sind).
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: DruckvorschauHandler.java 45947 2013-09-12 08:23:17Z schumann $
 */
public class DruckvorschauHandler extends AbstractHandler {

	@Override
	public Object execute(final ExecutionEvent event) throws ExecutionException {

		final IWorkbenchPart part = HandlerUtil.getActivePart(event);

		RwPrintable druckvorschauPart = null;
		if (part instanceof RwPrintable) {
			druckvorschauPart = (RwPrintable) part;
		}

		if (druckvorschauPart != null) {
			final PagePrint druckAuftrag = druckvorschauPart.getDruckAuftrag();
			if (druckAuftrag != null) {
				final DruckvorschauDialog druckVorschau = new DruckvorschauDialog(
						PlatformUI.getWorkbench().getDisplay().getActiveShell());
				druckVorschau
						.oeffne(druckAuftrag, druckvorschauPart.getTitel());
			} else {
				final MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
						.getDisplay().getActiveShell(), SWT.ICON_INFORMATION
						| SWT.OK);
				mb.setText("Keine Druckseite verfgbar");
				mb.setMessage("Zur Zeit ist keine Druckseite verfgbar. Kann nicht drucken.");
				mb.open();
			}
		}

		return null;
	}
}
