/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.HandlerUtil;

import de.bsvrz.buv.rw.basislib.legende.ILegende;
import de.bsvrz.buv.rw.basislib.legende.LegendeWindow;

/**
 * Handler zum Ein-/Ausblenden der Legende.
 * 
 * Der aktuelle WorkbenchPart muss auf die Schnitstelle {@link ILegende}
 * adaptierbar sein.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: ToggleLegendeHandler.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class ToggleLegendeHandler extends AbstractHandler {

	@Override
	public Object execute(final ExecutionEvent event) throws ExecutionException {

		final IWorkbenchPart activePart = HandlerUtil.getActivePart(event);
		if (activePart != null) {
			LegendeWindow legendeWindow = LegendeWindow
					.getCachedLegendeWindowFor(activePart);
			if (legendeWindow == null) {
				legendeWindow = new LegendeWindow(activePart);
				legendeWindow.open();
			} else {
				legendeWindow.close();
			}
			return null;
		}

		return null;
	}

	@Override
	public void setEnabled(final Object evaluationContext) {
		boolean state = false;
		final IWorkbenchWindow window = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow();
		if (window != null) {
			final IWorkbenchPage page = window.getActivePage();
			if (page != null) {
				final IWorkbenchPart part = page.getActivePart();
				if ((part != null) && (part.getAdapter(ILegende.class) != null)) {
					state = true;
				}
			}
		}

		setBaseEnabled(state);
	}
}
