/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.interfaces.diagramme;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Interface fr die Datenbergabe an Diagramm-Plugin's.<br>
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:02 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 * 
 */
public interface IDiagramm {
	/**
	 * Dieser Aufzhlungstyp beschreibt die mglichen Schriftausrichtungen fr
	 * die Beschriftung bei Diagrammen.<br>
	 */
	public enum SchriftAusrichtung {
		/**
		 * Schriftausrichtung von links nach rechts.
		 */
		LINKS_NACH_RECHTS,
		/**
		 * Schriftausrichtung von rechts nach links.
		 */
		RECHTS_NACH_LINKS,
		/**
		 * Schriftausrichtung von oben nach unten.
		 */
		OBEN_NACH_UNTEN,
		/**
		 * Schriftausrichtung von unten nach oben.
		 */
		UNTEN_NACH_OBEN,
	}

	/**
	 * Dieser Aufzhlungstyp beschreibt die mglichen Positionen fr die
	 * Beschriftung bei Diagrammen.<br>
	 */
	public enum BeschriftungsPosition {
		/**
		 * Beschriftung oberhalb des Diagramms.
		 */
		OBEN,
		/**
		 * Beschriftung unterhalb des Diagramms.
		 */
		UNTEN,
		/**
		 * Beschriftung links vom Diagramm.
		 */
		LINKS,
		/**
		 * Beschriftung rechts vom Diagramm.
		 */
		RECHTS
	}

	/**
	 * Dieser Aufzhlungstyp beschreibt die mglichen EinfgeArten fr Daten bei
	 * Diagrammen.<br>
	 */
	public enum DatenEinfuegeArt {
		/**
		 * Die Daten werden vor der angegebenen Position eingefgt.
		 */
		VOR,
		/**
		 * Die Daten werden hinter der angegebenen Position eingefgt.
		 */
		HINTER
	}

	/**
	 * Dieser Aufzhlungstyp beschreibt die mglichen Diagrammtypen.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum Diagrammtyp {
		/**
		 * Darstellung erfolgt als Liniendiagramm.
		 */
		LINIEN,
		/**
		 * Darstellung erfolgt als Punktdiagramm.
		 */
		PUNKT,
		/**
		 * Darstellung erfolgt als Balkendiagramm.
		 */
		BALKEN,
		/**
		 * Darstellung erfolgt als gestapeltes Balkendiagramm.
		 */
		GESTAPELTE_BALKEN,
		/**
		 * Darstellung erfolgt als 3D-Balkendiagramm.
		 */
		BALKEN3D,
		/**
		 * Darstellung erfolgt als gestapeltets 3D-Balkendiagramm.
		 */
		GESTAPELTE_BALKEN3D,
		/**
		 * Darstellung erfolgt als Flchendiagramm.
		 */
		FLAECHE,
		/**
		 * Darstellung erfolgt als gestapeltes Flchendiagramm.
		 */
		GESTAPELTE_FLAECHEN,
		/**
		 * Darstellung erfolgt als Tortendiagramm.
		 */
		TORTE,
		/**
		 * Darstellung erfolgt als 3D-Tortendiagramm.
		 */
		TORTE3D
	}

	// /**
	// * Linien-Diagramm
	// */
	// static final int DIAGRAMMTYP_LINIEN = 0;
	// /**
	// * Punkt-Diagramm
	// */
	// static final int DIAGRAMMTYP_PUNKT = 1;
	// /**
	// * Balken-Diagramm
	// */
	// static final int DIAGRAMMTYP_BALKEN = 2;
	// /**
	// * Gestapelte-Balken-Diagramm
	// */
	// static final int DIAGRAMMTYP_GESTAPELTE_BALKEN = 3;
	// /**
	// * Balken-Diagramm 3D
	// */
	// static final int DIAGRAMMTYP_BALKEN3D = 4;
	// /**
	// * Gestapelte-Balken-Diagramm 3D
	// */
	// static final int DIAGRAMMTYP_GESTAPELTE_BALKEN3D = 5;
	// /**
	// * Flchen-Diagramm
	// */
	// static final int DIAGRAMMTYP_FLAECHE = 6;
	// /**
	// * Gestapelte-Flaechen-Diagramm
	// */
	// static final int DIAGRAMMTYP_GESTAPELTE_FLAECHEN = 7;
	// /**
	// * Torten-Diagramm
	// */
	// static final int DIAGRAMMTYP_TORTE = 8;
	// /**
	// * Torten-Diagramm 3D
	// */
	// static final int DIAGRAMMTYP_TORTE3D = 9;

	/**
	 * Hinzufgen einer X-Achse zum Diagramm mit Beschriftungsfeld.<br>
	 * 
	 * @param bezeichnung
	 *            Bezeichnung der Achse, wird im Diagramm ausgegeben.
	 * @param beschriftung
	 *            Beschriftungsfeld mit Beschriftungen fr alle
	 *            X-Achsen-Positionen (abhngig von Anzahl diskrete Werte in
	 *            DatenReihe).
	 * @return Index der Achse, -1, wenn Hinzufgen fehlgeschlagen.
	 */
	int xAchseHinzufuegen(String bezeichnung, String[] beschriftung);

	/**
	 * Hinzufgen einer X-Achse zum Diagramm mit Anfangs- und Enddatum oder
	 * -zeit.<br>
	 * 
	 * @param bezeichnung
	 *            Bezeichnung der Achse, wird im Diagramm ausgegeben.
	 * @param anfang
	 *            Zeit in Millisekunden fr Anfand.
	 * @param ende
	 *            Zeit in Millisekunden fr Ende.
	 * @param datumsAngabe
	 *            <code>true</code> fr <code>anfang</code> und
	 *            <code>ende</code> sind Datumsangabe, <code>false</code> fr
	 *            sind Zeitangabe.
	 * @return Index der Achse, -1, wenn Hinzufgen fehlgeschlagen.
	 */
	int xAchseHinzufuegen(String bezeichnung, long anfang, long ende,
			boolean datumsAngabe);

	/**
	 * Hinzufgen einer X-Achse zum Diagramm mit SystemObjekt-Feld, Methode
	 * getName() wird fr die Beschriftung verwendet.<br>
	 * 
	 * @param bezeichnung
	 *            Bezeichnung der Achse, wird im Diagramm ausgegeben.
	 * @param beschriftungsObjekte
	 *            Feld mit Objekten fr alle X-Achsen-Positionen (abhngig von
	 *            Anzahl diskrete Werte in DatenReihe), die Methode getName()
	 *            wird fr die Beschriftung verwendet.
	 * @return Index der Achse, -1, wenn Hinzufgen fehlgeschlagen.
	 */
	int xAchseHinzufuegen(String bezeichnung,
			SystemObject[] beschriftungsObjekte);

	/**
	 * Hinzufgen einer Y-Achse zum Diagramm.<br>
	 * 
	 * @param einheit
	 *            Einheit der Achse, wird im Diagramm in eckigen Klammern ("["
	 *            und "]") ausgegeben.
	 * @param minimalWert
	 *            minimaler Wert dieser Achse.
	 * @param maximaleWert
	 *            maximaler Wert dieser Achse.
	 * @param schrittWeite
	 *            Schrittweite zwischen den Achsenbeschriftungen.
	 * @return Index der Achse (wird als Referenz fr die Bezugnahme auf diese
	 *         Achse bentigt), -1, wenn Hinzufgen fehlgeschlagen.
	 */
	int yAchseHinzufuegen(String einheit, double minimalWert,
			double maximaleWert, double schrittWeite);

	/**
	 * Hinzufgen einer DatenReihe zu einer Y-Achse.<br>
	 * 
	 * @param indexYAchse
	 *            Index der Y-Achse, an der die Datenreihe hinzugefgt wird
	 *            (@see #yAchseHinzufuegen(String, double, double, double)).
	 * @param daten
	 *            Feld mit den hinzuzufgenden Daten.
	 * @param farbe
	 *            Farbe der Datenreihe im RGB-Format (Beispiel Rot = 0xff0000).
	 * @param diagrammTyp
	 *            DiagrammTyp (@see Konstanten DIAGRAMMTYP_... oder erweiternder
	 *            Wert fr bei Konstanten unbekannten typ).
	 * @return Index der DatenReihe (wird als Referenz fr die Bezugnahme auf
	 *         diese Datenreihe bentigt), -1, wenn Hinzufgen fehlgeschlagen.
	 */
	int datenReiheHinzufuegen(int indexYAchse, double[] daten, long farbe,
			Diagrammtyp diagrammTyp);

	/**
	 * Hinzufgen von Daten zu einer vorhandenen Datenreihe.<br>
	 * Der Index der Y-Achse und der Index der Datenreihe wird fr diese Aktion
	 * bentigt.<br>
	 * 
	 * @param indexYAchse
	 *            Index der Y-Achse, an der die Datenreihe vorhanden ist (@see
	 *            #yAchseHinzufuegen(String, double, double, double)).
	 * @param indexDatenReihe
	 *            Index der vorhanden Datenreihe, an der die Daten hinzugefuegt
	 *            werden (@see #datenReiheHinzufuegen()).
	 * @param einfuegePosition
	 *            Position, vor bzw. hinter der die Daten eingefgt werden (-1
	 *            enspricht letzter Position, Position beginnt bei 0).
	 * @param datenEinfuegeArt
	 *            Einfgeart mit der die Daten eingefgt werden.
	 * @param daten
	 *            Feld mit den hinzuzufgenden Daten.
	 * @return Anzahl der in der Datenreihe nach dem Hinzufgen enthaltenen
	 *         Daten, -1, wenn Hinzufgen fehlgeschlagen.
	 */
	int datenHinzufuegen(int indexYAchse, int indexDatenReihe,
			int einfuegePosition, DatenEinfuegeArt datenEinfuegeArt,
			double[] daten);

	/**
	 * Steuerung der Sichtbarkeit einer Datenreihe.<br>
	 * 
	 * @param indexDatenReihe
	 *            Index der Datenreihe (@see #datenHinzufuegen(int, int, int,
	 *            DatenEinfuegeArt, double[])).
	 * @param sichtbarkeit
	 *            <code>true</code> wenn Datenreihe angezeigt werden soll, sonst
	 *            <code>false</code>
	 */
	void datenReiheSichtbarkeit(int indexDatenReihe, boolean sichtbarkeit);

	/**
	 * Hinzufgen einer Beschriftung zum Diagramm.<br>
	 * 
	 * @param text
	 *            Beschriftungstext.
	 * @param textHoehe
	 *            Texthhe in Pixel.
	 * @param schriftArt
	 *            Schriftart entsprechend Face Names.
	 * @param schriftAusrichtung
	 *            Ausrichtung des Beschriftungstextes.
	 * @param beschriftungsPosition
	 *            Position des Beschriftungstextes.
	 */
	void beschriftungHinzufuegen(String text, int textHoehe, String schriftArt,
			SchriftAusrichtung schriftAusrichtung,
			BeschriftungsPosition beschriftungsPosition);

	/**
	 * Diagramm in der bergebenen Breite und Hhe darstellen.<br>
	 * 
	 * @param breite
	 *            Breite des Diagramms in Pixel.
	 * @param hoehe
	 *            Hhe des Diagramms in Pixel.
	 */
	void darstellen(int breite, int hoehe);
}
