/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.login;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Log-in-Dialog zur Anmeldung an das System . In der unteren Reihe wird der
 * Domain-Name (bzw. die IP-Adresse) und die Port-Adresse eingegeben, darber
 * Name und Passwort des Benutzers.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ( Dambach)
 * @version $Id: LoginDialog.java 49030 2014-02-28 15:19:54Z peuker $)
 */
public class LoginDialog extends TitleAreaDialog {

	/** die im Login-Dialog bergebenen Anmeldedaten. */
	public static final class AnmeldeDaten {

		private final String name;
		private final String passwort;
		private final String host;
		private final int port;
		private final boolean online;

		private AnmeldeDaten(final String name, final String passwort,
				final String host, final int port, final boolean online) {
			super();
			this.name = name;
			this.passwort = passwort;
			this.host = host;
			this.port = port;
			this.online = online;
		}

		/**
		 * liefert den Name des Hosts, zu dem eine Datenverteilerverbindung
		 * aufgebaut werden soll oder potentiell <code>null</code>, wenn im
		 * Offlinebetrieb gestartet wird.
		 * 
		 * @return den Name oder <code>null</code>
		 */
		public String getHost() {
			return host;
		}

		/**
		 * liefert den angegebenen Nutzername fr die Anmeldung bei einem
		 * Datenverteilerswystem oder potentiell <code>null</code>, wenn im
		 * Offlinebetrieb gestartet wird.
		 * 
		 * @return den Name oder <code>null</code>
		 */
		public String getName() {
			return name;
		}

		/**
		 * liefert das angegebene Passwort fr die Anmeldung bei einem
		 * Datenverteilerswystem oder potentiell <code>null</code>, wenn im
		 * Offlinebetrieb gestartet wird.
		 * 
		 * @return das Passwort oder <code>null</code>
		 */
		public String getPasswort() {
			return passwort;
		}

		/**
		 * liefert den Port, fr die TCP-Verbindung zu einem
		 * Datenverteilersystem oder potentiell 0, wenn im Offlinebetrieb
		 * gestartet wird.
		 * 
		 * @return den Port oder 0.
		 */
		public int getPort() {
			return port;
		}

		/**
		 * ermittelt, ob im Online-Betrieb gestartet werden soll.
		 * 
		 * @return den Zustand
		 */
		public boolean isOnline() {
			return online;
		}
	}

	/** interner Listener zur Aktualisierung der Darstellung des Dialogs. */
	private class ButtonUpdater implements ModifyListener, SelectionListener {

		@Override
		public void modifyText(final ModifyEvent e) {
			updateButtons(e.getSource());
		}

		@Override
		public void widgetDefaultSelected(final SelectionEvent e) {
			updateButtons(e.getSource());
		}

		@Override
		public void widgetSelected(final SelectionEvent e) {
			updateButtons(e.getSource());
		}
	}

	/** der Logger. */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Liste der Server-IPs bzw. Domainnamen
	 */
	private final List<String> serverIps = new ArrayList<String>();

	/** Liste der Server-Ports. */
	private final List<Integer> serverPorts = new ArrayList<Integer>();

	/** Prfix fr eine ServerIP in dem Properties-File. */
	private static final String AUFBAU_STRING_SERVER_IP = "ServerIp";

	/** Prfix fr einen Server-Port in dem Properties-file. */
	private static final String AUFBAU_STRING_SERVER_PORT = "ServerPort";

	/** Speicher fr lokale Einstellungen. */
	private final IPreferenceStore preferences = RahmenwerkActivator
			.getDefault().getPreferenceStore();

	/** Maximale Anzahl von Eintrgen in der Historie. */
	private static final int MAX_LOGIN_HISTORIE = 5;

	private static final int ONLINE_ID = IDialogConstants.CLIENT_ID + 1;

	private static final int OFFLINE_ID = IDialogConstants.CLIENT_ID + 2;

	/** die eingegebenen Anmeldedaten. */
	private AnmeldeDaten anmeldeDaten;

	private Text textBenutzername;

	private Text textPasswort;

	private String anmeldeHost;

	private int anmeldePort;

	private final ButtonUpdater buttonUpdater = new ButtonUpdater();

	private final String initialErrorText;

	private Image loginLogo;

	/** Standardkonstruktor. */
	public LoginDialog() {
		this((String) null);
	}

	/**
	 * Konstruktor mit optionalem Fehlertext oder <code>null</code>, wenn kein
	 * Fehler visualisiert werden soll.
	 * 
	 * Der Fehlertext wird beim Einblenden des Dialog dargestellt.
	 * 
	 * @param errorText
	 *            der Fehlertext oder <code>null</code>
	 */
	public LoginDialog(final String errorText) {
		super(null);

		initialErrorText = errorText;

		final ArgumentList startParameter = RahmenwerkImpl.getInstanz()
				.getStartParameter();

		if (startParameter.hasArgument("-loginLogo")) {
			final String imagePath = startParameter.fetchArgument("-loginLogo")
					.getValue();
			loginLogo = new Image(Display.getCurrent(), imagePath);
		}

		if (loginLogo == null) {
			loginLogo = RahmenwerkActivator.getDefault()
					.getImageDescriptor("icons/wizban/bitctrl_wizard.png")
					.createImage();
		}
	}

	@Override
	protected void buttonPressed(final int buttonId) {

		final IPreferenceStore preferenceStore = RahmenwerkActivator
				.getDefault().getPreferenceStore();
		preferenceStore.setValue(getClass().getName() + ".X", getShell()
				.getLocation().x);
		preferenceStore.setValue(getClass().getName() + ".Y", getShell()
				.getLocation().y);

		switch (buttonId) {
		case ONLINE_ID:
			anmeldeDaten = liesAnmeldeDaten(true);
			okPressed();
			break;
		case OFFLINE_ID:
			anmeldeDaten = liesAnmeldeDaten(false);
			okPressed();
			break;
		case IDialogConstants.CANCEL_ID:
			cancelPressed();
			break;
		default:
			super.buttonPressed(buttonId);
			break;
		}
	}

	@Override
	public boolean close() {
		final boolean result = super.close();
		if (loginLogo != null) {
			loginLogo.dispose();
		}
		return result;
	}

	@Override
	protected void createButtonsForButtonBar(final Composite parent) {
		createButton(parent, LoginDialog.ONLINE_ID, "Online", true).setEnabled(
				false);
		createButton(parent, LoginDialog.OFFLINE_ID, "Offline", false);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	@Override
	protected Control createDialogArea(final Composite parent) {

		final ArgumentList startParameter = RahmenwerkImpl.getInstanz()
				.getStartParameter();

		getShell().setText("Anmeldung");
		if (startParameter.hasArgument("-loginDialogTitel")) {
			getShell().setText(
					startParameter.fetchArgument("-loginDialogTitel")
							.getValue());
		}

		setTitle("Rahmenwerk-Anmeldung");
		if (startParameter.hasArgument("-loginTitel")) {
			setTitle(startParameter.fetchArgument("-loginTitel").getValue());
		}

		setMessage("Geben Sie ihre Anmeldedaten an oder starten Sie im Offline-Betrieb");
		if (startParameter.hasArgument("-loginMessage")) {
			setMessage(startParameter.fetchArgument("-loginMessage").getValue());
		}

		setTitleImage(loginLogo);

		ladePraeferenzen();

		final Composite panel = new Composite(parent, SWT.NONE);
		panel.setLayoutData(new GridData(GridData.FILL_BOTH
				| GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));

		panel.setLayout(new GridLayout(4, false));

		final Label labelBenutzername = new Label(panel, SWT.NONE);
		labelBenutzername.setText("Benutzername:");

		textBenutzername = new Text(panel, SWT.BORDER);
		textBenutzername.setLayoutData(GridDataFactory.fillDefaults()
				.grab(true, false).span(3, 1).create());
		textBenutzername.addModifyListener(buttonUpdater);

		final Label labelPasswort = new Label(panel, SWT.NONE);
		labelPasswort.setText("Passwort:");

		textPasswort = new Text(panel, SWT.BORDER | SWT.PASSWORD);
		textPasswort.setLayoutData(GridDataFactory.fillDefaults()
				.grab(true, false).span(3, 1).create());
		textPasswort.addModifyListener(buttonUpdater);

		if (!startParameter.hasArgument("-disableHost")) {
			final Label labelAdresse = new Label(panel, SWT.NONE);
			labelAdresse.setText("Domainname / IP-Adresse:");

			erzeugeAuswahlAdresse(panel);

			final Label labelPort = new Label(panel, SWT.NONE);
			labelPort.setText("TCP-Portnummer:");

			erzeugeAuswahlPort(panel);
		} else {
			if (startParameter.hasArgument("-datenverteiler")) {
				initAnmeldedaten(startParameter
						.fetchArgument("-datenverteiler").getValue());
			} else {
				initAnmeldedaten(null);
			}
		}

		try {
			final String helpContextId = RahmenwerkActivator.PLUGIN_ID
					+ ".logindialog";
			PlatformUI.getWorkbench().getHelpSystem()
					.setHelp(panel, helpContextId);
		} catch (final IllegalStateException exception) {
			// das Hilfesystem kann erst initialisiert werden, wenn die
			// Workbench vorhanden ist.
			LoginDialog.LOGGER.fine(exception.getLocalizedMessage());
		}

		if (initialErrorText != null) {
			setErrorMessage(initialErrorText);
		}

		return super.createDialogArea(parent);
	}

	private void initAnmeldedaten(final String daten) {

		anmeldeHost = "localhost";
		anmeldePort = 8083;

		if (daten != null) {
			final String[] split = daten.split(":");
			anmeldeHost = split[0];
			if (split.length > 1) {
				try {
					anmeldePort = Integer.parseInt(split[1]);
				} catch (final NumberFormatException e) {
					LoginDialog.LOGGER.warning(e.getLocalizedMessage());
				}
			}
		}
	}

	/** Initialisierung des Eingabfelds der Hostadresse. */
	private Combo erzeugeAuswahlAdresse(final Composite comp) {
		final Combo comboAdresse = new Combo(comp, SWT.NONE);
		comboAdresse.setData(LoginDialog.AUFBAU_STRING_SERVER_IP);
		comboAdresse.setLayoutData(GridDataFactory.fillDefaults()
				.grab(true, false).create());

		for (final String ip : serverIps) {
			comboAdresse.add(ip);
		}
		if (!serverIps.isEmpty()) {
			final String host = serverIps.get(0);
			comboAdresse.setText(host);
			anmeldeHost = host;
		}

		comboAdresse.addSelectionListener(buttonUpdater);
		comboAdresse.addModifyListener(buttonUpdater);

		return comboAdresse;
	}

	private Combo erzeugeAuswahlPort(final Composite composite) {
		final Combo comboPort = new Combo(composite, SWT.NONE);
		comboPort.setLayoutData(GridDataFactory.fillDefaults()
				.grab(true, false).create());
		comboPort.setData(LoginDialog.AUFBAU_STRING_SERVER_PORT);

		if (serverPorts.size() > 0) {
			for (final Integer portItem : serverPorts) {
				comboPort.add(portItem.toString());
			}
			comboPort.setText(serverPorts.get(0).toString());
			anmeldePort = serverPorts.get(0);
		}

		comboPort.addSelectionListener(buttonUpdater);
		comboPort.addModifyListener(buttonUpdater);

		return comboPort;
	}

	/**
	 * liefert die Anmeldedaten nach dem Schlieen des Dialogs.
	 * 
	 * @return die Daten
	 */
	public AnmeldeDaten getAnmeldeDaten() {
		return anmeldeDaten;
	}

	/**
	 * Fgt eine IP-Adresse hinzu.
	 * 
	 * @param ip
	 *            Hinzufgende IP-Adresse.
	 */
	private void hinzufuegenIp(final String ip) {

		if (!ip.isEmpty()) {
			serverIps.remove(ip.toLowerCase());
			serverIps.add(0, ip.toLowerCase());

			while (serverIps.size() > LoginDialog.MAX_LOGIN_HISTORIE) {
				serverIps.remove(serverIps.size() - 1);
			}
		}
	}

	/**
	 * Fgt einen Port hinzu.
	 * 
	 * @param port
	 *            Hinzuzufgender Port.
	 */
	private void hinzufuegenPort(final Integer port) {
		if (port != null) {
			serverPorts.remove(port);
			serverPorts.add(0, port);
		}

		while (serverPorts.size() > LoginDialog.MAX_LOGIN_HISTORIE) {
			serverPorts.remove(serverPorts.size() - 1);
		}
	}

	/**
	 * Ldt die Prferenzen fr diesen Dialog. Diese beinhalten eine Liste der
	 * gesicherten DaV-Adressen und Ports.
	 */
	private void ladePraeferenzen() {

		for (int i = 0; i < LoginDialog.MAX_LOGIN_HISTORIE; i++) {
			final String tempId = LoginDialog.AUFBAU_STRING_SERVER_IP + i;
			final String tempPort = LoginDialog.AUFBAU_STRING_SERVER_PORT + i;

			final String ip = preferences.getString(tempId);
			if (ip != null) {
				serverIps.add(ip);
			}

			final int port = preferences.getInt(tempPort);
			if (port > 0) {
				serverPorts.add(port);
			}
		}
	}

	private AnmeldeDaten liesAnmeldeDaten(final boolean online) {
		final String name = textBenutzername.getText().trim();
		final String passwort = textPasswort.getText();

		if ((!anmeldeHost.isEmpty()) && (anmeldePort > 0)) {
			hinzufuegenIp(anmeldeHost);
			hinzufuegenPort(anmeldePort);
			speicherePraeferenzen();
		}

		return new AnmeldeDaten(name, passwort, anmeldeHost, anmeldePort,
				online);
	}

	@Override
	protected void setShellStyle(final int newShellStyle) {
		super.setShellStyle(newShellStyle | SWT.ON_TOP);
	}

	/**
	 * Speichert die Prferenzen, dies beinhaltet die Liste der DaV-Adressen
	 * sowie die Liste der zugehrigen Ports.
	 */
	private void speicherePraeferenzen() {

		for (int idx = 0; idx < LoginDialog.MAX_LOGIN_HISTORIE; idx++) {
			final String id = LoginDialog.AUFBAU_STRING_SERVER_IP + idx;
			if (serverIps.size() > idx) {
				preferences.setValue(id, serverIps.get(idx));
			} else {
				preferences.setValue(id, null);
			}
		}

		for (int idx = 0; idx < LoginDialog.MAX_LOGIN_HISTORIE; idx++) {
			final String id = LoginDialog.AUFBAU_STRING_SERVER_PORT + idx;
			if (serverPorts.size() > idx) {
				preferences.setValue(id, serverPorts.get(idx));
			} else {
				preferences.setValue(id, 0);
			}
		}
	}

	private void updateButtons(final Object control) {

		boolean error = false;
		if (control instanceof Combo) {
			final Combo combo = (Combo) control;
			if (LoginDialog.AUFBAU_STRING_SERVER_IP.equals(combo.getData())) {
				anmeldeHost = combo.getText().trim();
			} else if (LoginDialog.AUFBAU_STRING_SERVER_PORT.equals(combo
					.getData())) {
				try {
					anmeldePort = Integer.parseInt(combo.getText().trim());
					if ((anmeldePort < 1) || (anmeldePort > 65535)) {
						setMessage(
								"Der Port "
										+ anmeldePort
										+ " fr die Datenverteilerverbindung ist ungltig!",
								IMessageProvider.ERROR);
						error = true;
					}
				} catch (final NumberFormatException e) {
					LoginDialog.LOGGER.finer(e.getLocalizedMessage());
					setMessage("Der Eingabe \"" + combo.getText()
							+ "\" kann nicht als Zahl interpretiert werden!",
							IMessageProvider.ERROR);
					error = true;
				}
			}
		}

		if (!error) {
			if (textBenutzername.getText().trim().isEmpty()) {
				setMessage("Es wurde kein Benutzername angegeben!",
						IMessageProvider.WARNING);
				error = true;
			} else if (textPasswort.getText().trim().isEmpty()) {
				setMessage("Es wurde kein Passwort angegeben!",
						IMessageProvider.WARNING);
				error = true;
			} else if ((anmeldeHost == null) || anmeldeHost.isEmpty()) {
				setMessage("Es wurde keine Datenverteiler-Host angegeben!",
						IMessageProvider.WARNING);
				error = true;
			} else if ((anmeldePort < 1) || (anmeldePort > 65535)) {
				setMessage("Der Port " + anmeldePort
						+ " fr die Datenverteilerverbindung ist ungltig!",
						IMessageProvider.ERROR);
				error = true;
			}
		}

		if (!error) {
			setErrorMessage(null);
			setMessage("Geben Sie ihre Anmeldedaten an oder starten Sie im Offline-Betrieb");
		}
		getButton(LoginDialog.ONLINE_ID).setEnabled(!error);
	}

	@Override
	protected Point getInitialLocation(final Point initialSize) {

		final IPreferenceStore preferenceStore = RahmenwerkActivator
				.getDefault().getPreferenceStore();
		final int x = preferenceStore.getInt(getClass().getName() + ".X");
		final int y = preferenceStore.getInt(getClass().getName() + ".Y");

		if (((x == 0) && (y == 0))) {
			return super.getInitialLocation(initialSize);
		}

		final Rectangle bounds = getConstrainedShellBounds(new Rectangle(x, y,
				initialSize.x, initialSize.y));
		return new Point(bounds.x, bounds.y);
	}
}
