/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;

/**
 * Handler fr die Verarbeitung der XML-Strukturen fr Mens des Rahmenwerks.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: MenuXmlHandler.java 46077 2013-09-17 19:45:50Z peuker $
 */
class MenuXmlHandler extends DefaultHandler {

	public static final String TAG_STATUS_LEISTE = "statusleiste";
	public static final String TAG_SYMBOL_LEISTE = "symbolleiste";
	public static final String TAG_MENU_LEISTE = "menuleiste";

	public static final String TAG_MENU = "menu";
	public static final String TAG_SUBMENU = "submenu";
	public static final String TAG_ITEM = "item";
	public static final String TAG_PARAMETERS = "parameters";
	public static final String TAG_PARAMETER = "parameter";
	public static final String TAG_SEPARATOR = "separator";

	public static final String ATT_ID = "id";
	public static final String ATT_ACTION_ID = "actionId";
	public static final String ATT_NAME = "name";
	public static final String ATT_USERLABEL = "userLabel";
	public static final String ATT_VALUE = "value";

	private final Menue aktionsLeiste;
	private Menue currentMenu;
	private MenueAction currentAction;

	public MenuXmlHandler(final Menue menue) {
		this.aktionsLeiste = menue;
	}

	@Override
	public void startElement(final String uri, final String localName,
			final String qName, final Attributes attributes)
			throws SAXException {

		if (MenuXmlHandler.TAG_ITEM.equals(qName)) {
			final String id = attributes.getValue(MenuXmlHandler.ATT_ACTION_ID);
			final RwToolbarElement deskriptorAktion = RahmenwerkImpl
					.getInstanz().getRwToolBarManager().getToolBarElement(id);
			final MenueAction action = new MenueAction(deskriptorAktion, id,
					attributes.getValue(MenuXmlHandler.ATT_NAME));
			action.setUserLabel(attributes
					.getValue(MenuXmlHandler.ATT_USERLABEL));
			currentMenu.addElement(action);
			currentAction = action;

		} else if (MenuXmlHandler.TAG_PARAMETER.equals(qName)) {
			if (currentAction != null) {
				currentAction.setParameter(
						attributes.getValue(MenuXmlHandler.ATT_ID),
						attributes.getValue(MenuXmlHandler.ATT_NAME),
						attributes.getValue(MenuXmlHandler.ATT_VALUE));
			}
		} else if (MenuXmlHandler.TAG_PARAMETERS.equals(qName)) {
			// hat keine inhaltliche Bedeutung, kapselt die Parameter der Aktion
		} else if (MenuXmlHandler.TAG_MENU.equals(qName)) {
			// hat keine inhaltliche Bedeutung
		} else if (MenuXmlHandler.TAG_MENU_LEISTE.equals(qName)) {
			currentMenu = aktionsLeiste;
			// hat keine inhaltliche Bedeutung (Hauptrahmen der Definition)
		} else if (MenuXmlHandler.TAG_SYMBOL_LEISTE.equals(qName)) {
			currentMenu = aktionsLeiste;
			// hat keine inhaltliche Bedeutung (Hauptrahmen der Definition)
		} else if (MenuXmlHandler.TAG_STATUS_LEISTE.equals(qName)) {
			currentMenu = aktionsLeiste;
			// hat keine inhaltliche Bedeutung (Hauptrahmen der Definition)
		} else if (MenuXmlHandler.TAG_SEPARATOR.equals(qName)) {
			final MenueSeparator separator = new MenueSeparator();
			currentMenu.addElement(separator);
		} else if (MenuXmlHandler.TAG_SUBMENU.equals(qName)) {
			final SubMenue subMenu = new SubMenue(
					attributes.getValue(MenuXmlHandler.ATT_NAME));
			currentMenu.addElement(subMenu);
			currentMenu = subMenu;
		}
	}

	@Override
	public void endElement(final String uri, final String localName,
			final String qName) throws SAXException {

		if (MenuXmlHandler.TAG_SUBMENU.equals(qName)) {
			if (currentMenu != null) {
				currentMenu = currentMenu.getMenue();
			}
		} else if (MenuXmlHandler.TAG_ITEM.equals(qName)) {
			currentAction = null;
		}
	}
}
