/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.menu;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;

/**
 * Sammlung der Plug-ins, die Elemente fr den Meneditor zur Verfgung stellen.
 * 
 * Diese Klasse wird in erster Linie fr den ContentProvider der Ansciht der fr
 * den meneditor verfgbaren Elemente verwendet.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: PluginCollection.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class PluginCollection {

	private final String name;
	private List<RwPluginElemente> listePlugins;

	/**
	 * {@link RwToolbarElementManagerImpl}.
	 */
	private RwToolbarElementManagerImpl registrierung;

	/**
	 * Konstruktor.
	 * 
	 * @param name
	 *            Name der Sammlung
	 */
	public PluginCollection(final String name) {
		this.name = name;
		init();
	}

	private void init() {
		listePlugins = new LinkedList<RwPluginElemente>();
		registrierung = RahmenwerkImpl.getInstanz().getRwToolBarManager();

		final Map<String, String> plugins = registrierung.getPluginIdAndNames();
		for (final Entry<String, String> plugin : plugins.entrySet()) {
			final RwPluginElemente descriptorPlugin = new RwPluginElemente(
					plugin.getKey(), plugin.getValue());
			this.listePlugins.add(descriptorPlugin);
		}

	}

	/**
	 * Liefert den Namen der Wurzel.
	 * 
	 * @return Wurzelname.
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Liefert eine Liste mit Instanzen der Klasse {@link RwPluginElemente}. Es
	 * sin in dieser alle bekannten Plug-Ins vorhanden.
	 * 
	 * @return Liste aller Plugins.
	 */
	public List<RwPluginElemente> getPlugins() {
		return this.listePlugins;
	}

	@Override
	public String toString() {
		return "Wurzelknoten";
	}
}
