/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.menu;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Plug-In-Definition mit den darin zur Verfgung gestellten
 * Rahmenwerk-Toolbar-Elementen. DIe Instanzen dieser Klasse werden in erster
 * Linie fr die Umsetzung des Content-Providers fr die Ansicht der verfgbaren
 * Elemente fr den Meneditor bentigt.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: RwPluginElemente.java 49030 2014-02-28 15:19:54Z peuker $
 */
public class RwPluginElemente {

	/** Der Debugger. */
	private static final Debug LOGGER = Debug.getLogger();

	/** ID des Plug-Ins. */
	private final String pluginId;

	/** Liste der Kindelemente. */
	private final Set<RwToolbarElement> elemente = new HashSet<RwToolbarElement>();

	/**
	 * Instanz der {@link RwToolbarElementManagerImpl}.
	 */
	private final RwToolbarElementManagerImpl registrierung = RahmenwerkImpl
			.getInstanz().getRwToolBarManager();

	private final String bundleName;

	/**
	 * Konstruktor dieser Klasse.
	 * 
	 * @param pluginID
	 *            ID des Plugins.
	 * @param bundleName
	 *            der Klarname des Bundles
	 */
	public RwPluginElemente(final String pluginID, final String bundleName) {
		this.pluginId = pluginID;
		this.bundleName = bundleName;
		init();
	}

	/**
	 * Fhrt die Initialisierung durch.
	 */
	private void init() {

		// Action
		final Iterator<RwToolbarElement> it = registrierung
				.getPluginMenueElemente(this.pluginId).iterator();
		while (it.hasNext()) {
			final RwToolbarElement action = it.next();
			elemente.add(action);
		}
	}

	/**
	 * liefert eine Liste aller Elemente, die durch das Plug-in angeboten
	 * werden.
	 * 
	 * @return die Liste der Elemente.
	 */
	public Collection<RwToolbarElement> getElemente() {
		return this.elemente;
	}

	/**
	 * Liefert die ID des Plug-Ins.
	 * 
	 * @return ID des Plug-Ins.
	 */
	public String getPluginId() {
		return this.pluginId;
	}

	@Override
	public String toString() {
		return getBundleName() + "(" + getPluginId() + ")";
	}

	/**
	 * liefert den Name des Plug-ins (Bundle-Name).
	 * 
	 * @return den Name
	 */
	public String getBundleName() {
		return bundleName;
	}

}
