/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.ui.menus.CommandContributionItem;
import org.eclipse.ui.menus.CommandContributionItemParameter;
import org.eclipse.ui.services.IServiceLocator;

/**
 * Allgemeine Definition fr ein Command zur Einbindung in eine Toolbar oder das
 * Men des Rahmenwerks. Die Actions werden aus den ExtensionPoints
 * "CommandsSet" eingelesen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwToolbarCommand.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class RwToolbarCommand extends RwToolbarElement {

	private String toggleStateId;

	/**
	 * Konstruktor, erzeugt eine Elementdefinition auf Basis der bergebenen
	 * Konfigurationsdaten aus der entsprechenden Plugin-Definition.
	 * 
	 * @param configuration
	 *            die Konfigurationsdaten
	 */
	public RwToolbarCommand(final IConfigurationElement configuration) {
		super(configuration);

		final IConfigurationElement[] children = getConfiguration()
				.getChildren();
		for (final IConfigurationElement child : children) {
			if ("commandParameter".equals(child.getName())) {
				addParameter(new RwToolbarElementParameter(
						child.getAttribute("id"), child.getAttribute("name"),
						null, child.getAttribute("optional")));
			} else if ("state".equals(child.getName())) {
				toggleStateId = child.getAttribute("id");
			}
		}
	}

	@Override
	public IContributionItem createContributionItem(
			final IServiceLocator locator, final MenueAction element) {

		final Map<String, String> parameters = new HashMap<String, String>();

		for (final MenueAction.Parameter parameter : element.getParameters()) {
			final RwToolbarElementParameter rwToolbarElementParameter = getParameters()
					.get(parameter.getParameterId());
			if (rwToolbarElementParameter == null) {
				continue;
			}
			final String value = parameter.getValue();
			if (((value == null) || value.isEmpty())
					&& rwToolbarElementParameter.isOptional()) {
				continue;

			}
			parameters.put(parameter.getParameterId(), value);
		}

		int style = CommandContributionItem.STYLE_PUSH;
		if (toggleStateId != null) {
			style = CommandContributionItem.STYLE_CHECK;
		}

		final CommandContributionItemParameter commandParameter = new CommandContributionItemParameter(
				locator, getId(), getId(), parameters, null, null, null,
				element.getVisibleName(), null, null, style, null, false);
		final CommandContributionItem item = new CommandContributionItem(
				commandParameter);

		return item;
	}
}
