/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.ui.services.IServiceLocator;

import de.bsvrz.buv.rw.basislib.menu.RwMenuControl;

/**
 * Allgemeine Definition fr ein Control zur Einbindung in eine Toolbar oder das
 * Men des Rahmenwerks, insbesondere in die Statuszeile.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwToolbarControl.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class RwToolbarControl extends RwToolbarElement {

	private static long internalCounter;

	/**
	 * Konstruktor, erzeugt eine Elementdefinition auf Basis der bergebenen
	 * Konfigurationsdaten aus der entsprechenden Plugin-Definition.
	 * 
	 * @param configuration
	 *            die Konfigurationsdaten
	 */
	public RwToolbarControl(final IConfigurationElement configuration) {
		super(configuration);

		final IConfigurationElement[] children = getConfiguration()
				.getChildren("parameter");
		for (final IConfigurationElement child : children) {
			addParameter(new RwToolbarElementParameter(
					child.getAttribute("id"), child.getAttribute("name"),
					child.getAttribute("default"),
					child.getAttribute("optional")));
		}
	}

	@Override
	public IContributionItem createContributionItem(
			final IServiceLocator locator, final MenueAction element)
			throws CoreException {

		final IContributionItem item = (IContributionItem) getConfiguration()
				.createExecutableExtension("class");
		((ContributionItem) item).setId(getConfiguration().getAttribute("id")
				+ "." + (++RwToolbarControl.internalCounter));

		if (item instanceof RwMenuControl) {
			final Map<String, String> parameters = new HashMap<String, String>();
			for (final RwToolbarElementParameter entry : getParameters()
					.values()) {
				parameters.put(entry.getId(), entry.getDefaultValue());
			}
			for (final MenueAction.Parameter parameter : element
					.getParameters()) {
				parameters.put(parameter.getParameterId(), parameter.getValue());
			}
			((RwMenuControl) item).setParameter(parameters);
		}

		return item;
	}
}
