/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.menu;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;

import de.bsvrz.buv.rw.basislib.menu.RwToolBarManager;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Manager fr die Verwaltung der Toolbars und Mens der Rahmenwerk-Applikation.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwToolbarElementManagerImpl.java 47112 2013-11-12 15:25:14Z
 *          peuker $
 */
public final class RwToolbarElementManagerImpl implements RwToolBarManager {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * {@link IMenuManager} der Menleiste.
	 */
	private IMenuManager menuManager;

	/**
	 * {@link ICoolBarManager} der Symbolleiste.
	 */
	private ICoolBarManager symbolLeisteManager;

	private final Map<String, RwToolbarElement> toolbarElemente = new TreeMap<String, RwToolbarElement>();

	private IToolBarManager symbolLeisteToolbar;

	private IStatusLineManager statusLeisteManager;

	/**
	 * Privater Konstruktor.
	 */
	public RwToolbarElementManagerImpl() {
		leseActionSets();
		leseCommands();
		leseToolbarElemente();
	}

	/**
	 * liefert das mit ID definierte Element oder <code>null</code>, falls
	 * dieses nicht existiert.
	 * 
	 * @param id
	 *            die Id
	 * @return das Element oder <code>null</code>.
	 */
	public RwToolbarElement getToolBarElement(final String id) {
		synchronized (toolbarElemente) {
			if (toolbarElemente.containsKey(id)) {
				return toolbarElemente.get(id);
			}
			return null;
		}
	}

	/**
	 * Liefert den {@link IMenuManager}, der fr die Darstellung des Mens
	 * zustndig ist.<br>
	 * <b>Achtung:</b> Synchronisation auf dem zurckgegebnen Objekt nicht
	 * vergessen!
	 * 
	 * @return Liefert den {@link IMenuManager}.
	 */
	public IMenuManager getMenuManager() {
		IMenuManager re = null;
		re = this.menuManager;
		return re;
	}

	/**
	 * liefert eine Liste aller durch das mit der bergebenen ID definierte
	 * Plugin bereitgestellten Elemente.
	 * 
	 * @param pluginID
	 *            ID des Plug-Ins.
	 * @return die Liste der Elemente
	 */
	public List<RwToolbarElement> getPluginMenueElemente(final String pluginID) {
		final List<RwToolbarElement> actions = new LinkedList<RwToolbarElement>();

		synchronized (toolbarElemente) {
			final Iterator<Entry<String, RwToolbarElement>> it = toolbarElemente
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<String, RwToolbarElement> entry = it.next();
				if (entry.getValue().getPluginId().equals(pluginID)) {
					actions.add(entry.getValue());
				}
			}
		}

		return actions;
	}

	/**
	 * Liefert eine Liste mit den IDs aller Plug-Ins.
	 * 
	 * @return Liste aller Plug-In IDs.
	 */
	public Map<String, String> getPluginIdAndNames() {
		// Speichert sortiert die PluginIDs
		final TreeMap<String, String> tm = new TreeMap<String, String>();

		// Iterator ber hmActions
		synchronized (toolbarElemente) {
			final Iterator<Entry<String, RwToolbarElement>> itActions = toolbarElemente
					.entrySet().iterator();
			while (itActions.hasNext()) {
				final Entry<String, RwToolbarElement> entry = itActions.next();
				if (!tm.containsKey(entry.getValue().getPluginId())) {
					tm.put(entry.getValue().getPluginId(), entry.getValue()
							.getBundleName());
				}
			}
		}

		return tm;
	}

	/**
	 * liefert den definierten Statuszeilen-Manager.
	 * 
	 * @return den Manager
	 */
	@Override
	public IStatusLineManager getStatusLeisteManager() {
		return statusLeisteManager;
	}

	/**
	 * Liefert den {@link ICoolBarManager}, der fr die Darstellung der
	 * Symbolleiste zustndig ist.<br>
	 * <b>Achtung:</b> Synchronisation auf dem zurckgegebenen Objekt nicht
	 * vergessen!
	 * 
	 * @return Liefert den {@link ICoolBarManager}.
	 */
	public ICoolBarManager getSymbolLeisteManager() {
		return this.symbolLeisteManager;
	}

	/**
	 * liefert den Manager fr die Symbolleiste der Rahmenwerkapplikation.
	 * 
	 * @return den Manager
	 */
	public IToolBarManager getSymbolLeisteManagerToolbar() {
		return symbolLeisteToolbar;
	}

	private void leseActionSets() {
		final IExtensionRegistry reg = Platform.getExtensionRegistry();
		final IConfigurationElement[] configElements = reg
				.getConfigurationElementsFor("org.eclipse.ui.actionSets");

		for (final IConfigurationElement configElement : configElements) {
			final IConfigurationElement[] actions = configElement
					.getChildren("action");
			for (final IConfigurationElement action : actions) {
				registriereElement(new RwToolbarAction(action));
			}
		}
	}

	private void leseCommands() {

		final IExtensionRegistry reg = Platform.getExtensionRegistry();
		final IConfigurationElement[] configElements = reg
				.getConfigurationElementsFor("org.eclipse.ui.commands");

		for (final IConfigurationElement command : configElements) {

			if (!"command".equals(command.getName())) {
				continue;
			}

			if (command.getAttribute(RwToolbarElement.ATT_NAME) == null) {
				continue;
			}

			registriereElement(new RwToolbarCommand(command));

		}
	}

	private void leseToolbarElemente() {

		final IExtensionRegistry reg = Platform.getExtensionRegistry();
		final IConfigurationElement[] controls = reg
				.getConfigurationElementsFor("de.bsvrz.buv.rw.basislib.toolbarcontrol");
		for (final IConfigurationElement control : controls) {
			registriereElement(new RwToolbarControl(control));
		}
	}

	/**
	 * registriert eine Men-Element.
	 * 
	 * Wenn bereits ein Element mit der gleichen ID registriert wurde, wird die
	 * neue Registrierung ignoriert und eine entsprechende Meldung geloggt.
	 * 
	 * @param element
	 *            das Element
	 */
	private void registriereElement(final RwToolbarElement element) {

		final String aktionID = element.getId();
		synchronized (toolbarElemente) {
			if (!toolbarElemente.containsKey(aktionID)) {
				toolbarElemente.put(aktionID, element);
				RwToolbarElementManagerImpl.LOGGER
						.finest(getClass().getName()
								+ "::registriereDeskriptorAktion \n\t=> Registrierung ok -> "
								+ aktionID);
			} else {
				RwToolbarElementManagerImpl.LOGGER
						.fine("Ein Toolbarelement mit der ID \""
								+ aktionID
								+ "\" wurde schon bereits registriert, Element aus "
								+ element.getPluginId() + " wird ignoriert!");
			}
		}
	}

	/**
	 * Setzt den {@link IMenuManager} MenuManager, der fr die Darstellung des
	 * Mens zustndig ist.
	 * 
	 * @param manager
	 *            Zu setzender {@link IMenuManager}.
	 */
	public void setMenuManager(final IMenuManager manager) {
		this.menuManager = manager;
	}

	/**
	 * setzt den Manager zur Verwaltung der Statuszeile.
	 * 
	 * @param statusLeisteManager
	 *            der Manager
	 */
	public void setStatusLeisteManager(
			final IStatusLineManager statusLeisteManager) {
		this.statusLeisteManager = statusLeisteManager;
	}

	/**
	 * Setzt den {@link ICoolBarManager}, der fr die Darstellung der
	 * Symbolleiste zustndig ist.<br>
	 * <b>Achtung:</b> Synchronisation auf dem zurckgegebenen Objekt nicht
	 * vergessen!
	 * 
	 * @param manager
	 *            Zu setzender {@link ICoolBarManager}.
	 * @param toolbar
	 *            der Manager der Toolbar
	 */
	public void setSymbolLeisteManager(final ICoolBarManager manager,
			final IToolBarManager toolbar) {
		this.symbolLeisteManager = manager;
		this.symbolLeisteToolbar = toolbar;
	}
}
