/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu.editoren;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.menu.PluginCollection;
import de.bsvrz.buv.rw.rw.menu.RwPluginElemente;
import de.bsvrz.buv.rw.rw.menu.RwToolbarElement;

/**
 * Labelprovider fr die Darstellung der fr den Men-Editor verfgbaren
 * Men-Elemente.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: CommandSelektorLabelProvider.java 46953 2013-11-05 10:05:57Z peuker $
 */
class CommandSelektorLabelProvider extends ColumnLabelProvider {

	@Override
	public String getText(final Object element) {

		if (element instanceof PluginCollection) {
			final PluginCollection descriptorWurzel = (PluginCollection) element;
			return descriptorWurzel.getName();
		} else if (element instanceof RwPluginElemente) {
			final RwPluginElemente descriptorPlugin = (RwPluginElemente) element;
			return descriptorPlugin.getBundleName();
		} else if (element instanceof RwToolbarElement) {
			String returnText = "";
			final RwToolbarElement descriptorAction = (RwToolbarElement) element;
			returnText = returnText + descriptorAction.getName() + "("
					+ descriptorAction.getId() + ")";

			return returnText;
		} else {
			return null;
		}

	}

	@Override
	public String getToolTipText(final Object element) {
		if (element instanceof RwPluginElemente) {
			final RwPluginElemente descriptorPlugin = (RwPluginElemente) element;
			return descriptorPlugin.getPluginId();
		} else if (element instanceof RwToolbarElement) {
			final RwToolbarElement action = (RwToolbarElement) element;
			return action.getBeschreibung();
		}
		return super.getToolTipText(element);
	}

	@Override
	public Image getImage(final Object element) {
		Image returnImage = null;

		if (element instanceof RwPluginElemente) {
			returnImage = RahmenwerkActivator.getDefault().getImage(
					"icons/obj16/plug.gif");
		}

		if (element instanceof RwToolbarElement) {
			returnImage = RahmenwerkActivator.getDefault().getImage(
					"icons/obj16/menuitem.gif");
		}

		if (element instanceof PluginCollection) {
			returnImage = RahmenwerkActivator.getDefault().getImage(
					"icons/obj16/menu.gif");
		}

		return returnImage;
	}
}
