/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.menu.editoren;

import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TransferData;

import de.bsvrz.buv.rw.rw.menu.Menue;
import de.bsvrz.buv.rw.rw.menu.MenueAction;
import de.bsvrz.buv.rw.rw.menu.MenueElement;
import de.bsvrz.buv.rw.rw.menu.RwToolbarElement;

/**
 * Der DND-Adapter zum Empfang von Daten innerhalb des Men-Editors.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: MenueDndAdapter.java 51019 2014-07-21 06:43:01Z peuker $
 */
public class MenueDndAdapter extends ViewerDropAdapter {

	MenueDndAdapter(final TreeViewer viewer) {
		super(viewer);
	}

	@Override
	public boolean performDrop(final Object data) {

		Object target = getCurrentTarget();
		 int location = getCurrentLocation();
		if (target == null) {
			target = getViewer().getInput();
		}

		if (data instanceof TreeSelection) {
			for (final Object object : ((TreeSelection) data).toArray()) {
				if (object instanceof MenueElement) {
					final MenueElement element = (MenueElement) object;
					Menue menue = null;

					MenueElement relative = null;

					if (target instanceof Menue) {
						menue = (Menue) target;
						if ((location == LOCATION_BEFORE) || (location == LOCATION_AFTER)) {
							final Menue parentMenue = ((Menue) target).getMenue();
							if (parentMenue == null) {
								if ((location == LOCATION_AFTER)
										&& (!((Menue) target).getElemente()
												.isEmpty())) {
									relative = ((Menue) target).getElemente()
											.get(0);
									location = LOCATION_BEFORE;
								}
							} else {
								menue = ((MenueElement) target).getMenue();
								relative = (MenueElement) target;
							}
						}
					} else if (target instanceof MenueElement) {
						menue = ((MenueElement) target).getMenue();
						relative = (MenueElement) target;
					}

					if (menue != null) {
						final Menue prevMenue = element.getMenue();

						prevMenue.removeElement(element);
						if (relative == null) {
							menue.addElement(element);
						} else {
							if (location == ViewerDropAdapter.LOCATION_BEFORE) {
								menue.addElementBefore(element, relative);
							} else {
								menue.addElementAfter(element, relative);
							}
						}
						getViewer().refresh();
					}
				} else if (object instanceof RwToolbarElement) {
					final RwToolbarElement descAktion = (RwToolbarElement) object;
					Menue menue = null;
					MenueElement relative = null;

					if (target instanceof Menue) {
						menue = (Menue) target;
						if ((location == LOCATION_BEFORE) || (location == LOCATION_AFTER)) {
							final Menue parentMenue = ((Menue) target).getMenue();
							if (parentMenue == null) {
								if ((location == LOCATION_AFTER)
										&& (!((Menue) target).getElemente()
												.isEmpty())) {
									relative = ((Menue) target).getElemente()
											.get(0);
									location = LOCATION_BEFORE;
								}
							} else {
								menue = ((MenueElement) target).getMenue();
								relative = (MenueElement) target;
							}
						}
					} else if (target instanceof MenueElement) {
						menue = ((MenueElement) target).getMenue();
						relative = (MenueElement) target;
					}

					if (menue != null) {
						final MenueAction action = new MenueAction(
								(RwToolbarElement) object, descAktion.getId(),
								descAktion.getName());
						if (relative == null) {
							menue.addElement(action);
						} else {
							if (location == ViewerDropAdapter.LOCATION_BEFORE) {
								menue.addElementBefore(action, relative);
							} else {
								menue.addElementAfter(action, relative);
							}
						}
						getViewer().refresh();
					}
				}
			}

		}

		return true;

	}

	@Override
	public boolean validateDrop(final Object target, final int operation,
			final TransferData transferType) {

		boolean result = false;
		final int location = getCurrentLocation();

		if (target instanceof MenueElement) {

			final Object dragData = LocalSelectionTransfer.getTransfer()
					.getSelection();

			if (dragData instanceof TreeSelection) {
				for (final Object object : ((TreeSelection) dragData).toArray()) {
					if (object instanceof MenueElement) {
						result = true;
						if (target instanceof MenueElement) {
							MenueElement currentTarget = (MenueElement) target;
							while (currentTarget != null) {
								if (object.equals(currentTarget)) {
									result = false;
									break;
								}
								currentTarget = currentTarget.getMenue();
							}
						}
					} else if (object instanceof RwToolbarElement) {
						if (target instanceof Menue) {
							result = true;
						} else if (target instanceof MenueElement) {
							if ((location == ViewerDropAdapter.LOCATION_AFTER)
									|| ((location == ViewerDropAdapter.LOCATION_BEFORE))) {
								result = true;
							}
						}
					}
				}
			}
		}

		return result;
	}
}
