/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu.editoren.handler;

import java.io.FileWriter;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;

import de.bsvrz.buv.rw.rw.menu.AbstractMenueLeiste;
import de.bsvrz.buv.rw.rw.menu.editoren.MenueEditorInput;

/**
 * Handler fr das allgemeine Command "Export" ("org.eclipse.ui.file.export") im
 * Men-Editor.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: MenueEditorExportHandler.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class MenueEditorExportHandler extends AbstractHandler {

	@Override
	public Object execute(final ExecutionEvent event) throws ExecutionException {
		final IEditorPart activeEditor = HandlerUtil.getActiveEditor(event);
		final AbstractMenueLeiste menueLeiste = ((MenueEditorInput) activeEditor
				.getEditorInput()).getMenueLeiste();

		final FileDialog dialog = new FileDialog(activeEditor.getSite()
				.getShell(), SWT.SAVE);

		if (menueLeiste.isEmpty()) {
			MessageDialog.openInformation(activeEditor.getSite().getShell(),
					"Export",
					"Eine leere Menleiste kann nicht exportiert werden!");
		} else {

			final String filePath = dialog.open();
			if (filePath != null) {
				FileWriter writer = null;
				try {
					writer = new FileWriter(filePath);
					writer.write(menueLeiste.getXmlFormat());
					writer.close();
				} catch (final IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (final ParserConfigurationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (final TransformerException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} finally {
					if (writer != null) {
						try {
							writer.close();
						} catch (final IOException e) {
							// wird nicht weiter betrachtet
							e.printStackTrace();
						}
					}
				}
			}
		}
		return null;
	}
}
