/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu.editoren.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;

import de.bsvrz.buv.rw.rw.menu.Menue;
import de.bsvrz.buv.rw.rw.menu.MenueAction;
import de.bsvrz.buv.rw.rw.menu.MenueElement;

/**
 * Handler fr das allgemeine Command "Einfgen" ("org.eclipse.ui.edit.paste")
 * im Men-Editor.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: MenueEditorPasteHandler.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class MenueEditorPasteHandler extends AbstractHandler {

	@Override
	public Object execute(final ExecutionEvent event) throws ExecutionException {
		final ISelection currentSelection = HandlerUtil
				.getCurrentSelection(event);
		final ISelection selection = LocalSelectionTransfer.getTransfer()
				.getSelection();

		if ((currentSelection instanceof IStructuredSelection)
				&& (selection instanceof IStructuredSelection)) {
			final Object destination = ((IStructuredSelection) currentSelection)
					.getFirstElement();

			Menue destMenue = null;
			MenueElement elementAfter = null;

			if (destination instanceof MenueAction) {
				destMenue = ((MenueAction) destination).getMenue();
				elementAfter = (MenueElement) destination;
			} else if (destination instanceof Menue) {
				destMenue = (Menue) destination;
			}

			if (destMenue != null) {
				for (final Object selected : ((IStructuredSelection) selection)
						.toArray()) {
					if (selected instanceof MenueElement) {
						final MenueElement newElement = ((MenueElement) selected)
								.createDeepCopy();
						if (elementAfter != null) {
							destMenue.addElementAfter(newElement, elementAfter);
						} else {
							if (destMenue.getElemente().isEmpty()) {
								destMenue.addElement(newElement);
							} else {
								destMenue.addElementBefore(newElement,
										destMenue.getElemente().get(0));

							}
							elementAfter = newElement;
						}
					}
				}
			}
		}

		return null;
	}

	@Override
	public void setEnabled(final Object evaluationContext) {

		final ISelection selection = LocalSelectionTransfer.getTransfer()
				.getSelection();
		Object element = null;
		if (selection instanceof IStructuredSelection) {
			element = ((IStructuredSelection) selection).getFirstElement();
		}
		setBaseEnabled(element instanceof MenueElement);
	}
}
