/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.offlinekonfiguration;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * Liefert den Dialog zur Auswahl der Konfigurationsdatei.
 * 
 * @author HofmannJ
 * 
 */
public class KonfigurationsDateiAuswahlDialog extends Dialog {

	private Text textDatei;

	/** Shell fr die Darstellung des Dialogs. */
	private Shell shell;

	/**
	 * Flag: Zeigt an, ob eine Konfigurationsdatei ausgewhlt wurde oder nicht.
	 */
	private boolean konfigurationsdateiAusgewaehlt;

	/** Konfigurationsdatei. */
	private KonfigurationsDatei konfigurationsDatei;

	/** Pfad zur Konfigurationsdatei. */
	private String dateiPfad;

	public KonfigurationsDateiAuswahlDialog(final Shell elternShell) {
		super(elternShell);
	}

	/**
	 * ffnet den Dialog zur Auswahl der Konfigurationsdatei.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn eine Konfigurationsdatei
	 *         ausgewhlt wurde, ansonsten wird <code>false</code> zurck
	 *         geliefert.
	 */
	public boolean anzeigen() {
		final Shell parent = getParent();
		shell = new Shell(parent, SWT.TITLE | SWT.BORDER
				| SWT.APPLICATION_MODAL | SWT.CLOSE);

		shell.setText("Konfigurationsdateiauswahl Dialog");

		shell.setSize(new Point(500, 130));

		final GridLayout gridLayout = new GridLayout();
		shell.setLayout(gridLayout);
		erzeugeGruppeKonfigurationsdatei();
		erzeugeKontrolle();

		/*
		 * ffnen der Shell
		 */
		shell.open();

		/*
		 * Zentrieren der Darstellung des Dialogs
		 */
		final Display display = parent.getDisplay();
		final Monitor primary = display.getPrimaryMonitor();
		final Rectangle bounds = primary.getBounds();
		final Rectangle rect = shell.getBounds();
		final int x = bounds.x + ((bounds.width - rect.width) / 2);
		final int y = bounds.y + ((bounds.height - rect.height) / 2);
		shell.setLocation(x, y);

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		return konfigurationsdateiAusgewaehlt;
	}

	/** This method initializes groupKonfigurationsdatei. */
	private void erzeugeGruppeKonfigurationsdatei() {
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		final GridLayout gridLayout1 = new GridLayout();
		gridLayout1.numColumns = 3;
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		final Group gruppeKonfigurationsdatei = new Group(shell, SWT.NONE);
		gruppeKonfigurationsdatei.setLayoutData(gridData);
		gruppeKonfigurationsdatei.setLayout(gridLayout1);
		gruppeKonfigurationsdatei.setText("Konfigurationsdatei auswhlen:");
		final Label labelDatei = new Label(gruppeKonfigurationsdatei, SWT.NONE);
		labelDatei.setText("Konfigurationsdateipfad:");
		textDatei = new Text(gruppeKonfigurationsdatei, SWT.BORDER);
		textDatei.setEditable(false);
		textDatei.setLayoutData(gridData1);
		final Button buttonDatei = new Button(gruppeKonfigurationsdatei,
				SWT.NONE);
		buttonDatei.setText("Datei auswhlen");
		buttonDatei.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionKonfigurationsdateiAuswaehlen();
			}
		});
	}

	/** liefert einen Dateiauswahldialog und setzt die Konfigdatei. */
	private void aktionKonfigurationsdateiAuswaehlen() {
		final FileDialog dateiAuswahlDialog = new FileDialog(shell, SWT.OPEN);
		dateiAuswahlDialog
				.setFilterNames(new String[] { "Alle Dateien (*.*)" });
		dateiAuswahlDialog.setFilterExtensions(new String[] { "*.*" });
		dateiAuswahlDialog.setText("Auswahl Konfigurationsdatei");
		final String pfad = dateiAuswahlDialog.open();
		if (pfad != null) {
			dateiPfad = pfad;
			textDatei.setText(dateiPfad);
		}
	}

	/** This method initializes compositeKontrolle. */
	private void erzeugeKontrolle() {
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		gridLayout.makeColumnsEqualWidth = true;
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.END;
		gridData2.grabExcessHorizontalSpace = false;
		gridData2.verticalAlignment = GridData.CENTER;
		final Composite kontrolle = new Composite(shell, SWT.NONE);
		kontrolle.setLayoutData(gridData2);
		kontrolle.setLayout(gridLayout);
		final Button buttonUebernehmen = new Button(kontrolle, SWT.NONE);
		buttonUebernehmen.setText("bernehmen");
		buttonUebernehmen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionUebernehmen();
			}
		});

		final Button buttonAbbrechen = new Button(kontrolle, SWT.NONE);
		buttonAbbrechen.setText("Abbrechen");
		buttonAbbrechen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionSchliessen();
			}
		});

		final GridData gridDataButtons = new GridData();
		gridDataButtons.horizontalAlignment = GridData.FILL;
		gridDataButtons.verticalAlignment = GridData.CENTER;
		buttonAbbrechen.setLayoutData(gridDataButtons);
		buttonUebernehmen.setLayoutData(gridDataButtons);
	}

	/**
	 * bernehmen der ausgewhlten Datei.
	 * 
	 */
	private void aktionUebernehmen() {
		if (dateiPfad != null) {
			konfigurationsdateiAusgewaehlt = true;

			/*
			 * bernahme des Pfades zur Konfigurationsdatei.
			 */
			konfigurationsDatei = KonfigurationsDatei.getInstanz();
			konfigurationsDatei.setPfadZurKonfigurationsDatei(dateiPfad);
			// KonfigurationsDatei.getInstanz()
			// .setPfadZurKonfigurationsDatei(_dateiPfad);

			aktionSchliessen();
		}
	}

	/** Schliet den Dialogs. */
	private void aktionSchliessen() {
		shell.dispose();
	}

}
