/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************/

package de.bsvrz.buv.rw.rw.perspective;

import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

/**
 * Verwaltung der im Rahmenwerk verfgbaren Perspektiven (beim Start).
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwPerspectives.java 47179 2013-11-18 08:03:03Z peuker $
 */
public class RwPerspectives {

	private final Map<String, IConfigurationElement> perspectiveMap = new ConcurrentHashMap<String, IConfigurationElement>();

	/** Standardkonstruktor. */
	public RwPerspectives() {
		final IExtensionRegistry reg = Platform.getExtensionRegistry();
		final IConfigurationElement[] perspectives = reg
				.getConfigurationElementsFor("org.eclipse.ui.perspectives");

		for (final IConfigurationElement perspective : perspectives) {
			perspectiveMap.put(perspective.getAttribute("id"), perspective);
		}
	}

	/**
	 * prft, ob die mit der angegebenen ID definierte Perspektive verfgbar
	 * ist.
	 * 
	 * @param id
	 *            die Id
	 * @return die Verfgbarkeit
	 */
	public boolean hasPerspective(final String id) {
		return perspectiveMap.containsKey(id);
	}

	/**
	 * liefert die Bezeichnungen und ID der Perspektiven.
	 * 
	 * @return die Liste
	 */
	public String[][] getPerspektiveId() {
		final String[][] result = new String[perspectiveMap.size()][2];

		int idx = 0;
		for (final Entry<String, IConfigurationElement> entry : perspectiveMap
				.entrySet()) {
			result[idx][0] = entry.getValue().getAttribute("name");
			result[idx][1] = entry.getKey();
			idx++;
		}

		return result;
	}
}
