/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.preferences;

import de.bsvrz.buv.rw.rw.perspective.DefaultPerspective;

/**
 * Definition der allgemeinen Einstellungen fr das Rahmenwerk und der
 * zugehrigen Standardwerte.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RahmenwerkPreference.java 46800 2013-10-23 14:35:48Z peuker $
 */
public enum RahmenwerkPreference {

	/** das Format des Windows-Titels im Online-Betrieb. */
	ONLINE_TITLE_FORMAT("onlineTitleFormat",
			"Rahmenwerk \"{aoe}\" Version: {version}"),

	/** das Format des Windows-Titels im Offnline-Betrieb. */
	OFFLINE_TITLE_FORMAT("offlineTitleFormat", "Rahmenwerk Version: {version}"),

	/** die URL der Intro-Seite. */
	INTRO_URL("introUrl", "http://www.bitctrl.de"),

	/** definiert, ob die INTRO-Seite bei jedem Start angezeigt werden soll. */
	INTRO_ALWAYS("introAlways", Boolean.FALSE),

	/** definiert, die ID der Standardperspektive beim Start des Rahemenwerks. */
	START_PERSPEKTIVE("defaultStartPerspective",
			DefaultPerspective.PERSPECTIVE_ID);

	private String id;
	private Object defaultValue;

	private RahmenwerkPreference(final String id, final Object defaultValue) {
		this.id = id;
		this.defaultValue = defaultValue;
	}

	/**
	 * liefert den Standardwert der Einstellung.
	 * 
	 * @return den Wert
	 */
	public Object getDefaultValue() {
		return defaultValue;
	}

	/**
	 * liefert die ID der Einstellung.
	 * 
	 * @return die ID
	 */
	public String getId() {
		return id;
	}
}
