/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.statusleiste;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TimerTask;

import org.eclipse.jface.action.StatusLineLayoutData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Widget;

import de.bsvrz.buv.rw.basislib.menu.AbstractRwMenuControl;
import de.bsvrz.buv.rw.basislib.util.RwColor;
import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;

/**
 * Statuszeilen-Element, das die aktuelle Zeit anzeigt.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: AktuelleZeitElement.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class AktuelleZeitElement extends AbstractRwMenuControl {

	/** die Parameter des Elements. */
	private static final String PARAM_HINTERGRUND = "hintergrund";
	private static final String PARAM_FARBE = "farbe";
	private static final String PARAM_FORMAT = "format";
	private static final String PARAM_DAVZEIT = "davzeit";

	private Widget anzeigeText;
	private TimerTask task;

	private String formatStr = "HH:MM:SS";
	private String foreGround;
	private String backGround;
	private boolean davZeit;

	@Override
	public void fill(final Composite parent) {

		if (anzeigeText == null) {
			anzeigeText = new CLabel(parent, SWT.SHADOW_NONE | SWT.BORDER);
			((CLabel) anzeigeText).setText(formatStr);

			Color color = new Color(anzeigeText.getDisplay(),
					RwColor.getRgbFromString(foreGround));
			((CLabel) anzeigeText).setForeground(color);
			color.dispose();

			if ((backGround != null) && (!backGround.trim().isEmpty())) {
				color = new Color(anzeigeText.getDisplay(),
						RwColor.getRgbFromString(backGround));
				((CLabel) anzeigeText).setBackground(color);
				color.dispose();
			} else {
				((CLabel) anzeigeText).setBackground((Color) null);
			}

			((CLabel) anzeigeText).setAlignment(SWT.CENTER);
			anzeigeText.addDisposeListener(new DisposeListener() {

				@Override
				public void widgetDisposed(final DisposeEvent e) {
					anzeigeText = null;
					stopTimer();
				}
			});

			final Point preferredSize = ((Control) anzeigeText).computeSize(
					SWT.DEFAULT, SWT.DEFAULT);
			final int widthHint = preferredSize.x + 10;

			final StatusLineLayoutData data = new StatusLineLayoutData();
			data.widthHint = widthHint;
			((CLabel) anzeigeText).setLayoutData(data);

			updateTimeDisplay();
			startTimer();

		}
	}

	private void startTimer() {

		task = new TimerTask() {
			@Override
			public void run() {
				updateTimeDisplay();
			}

		};

		RahmenwerkActivator.getRahmenwerkTimer().schedule(task, 1000, 1000);
	}

	private void stopTimer() {
		task.cancel();
		RahmenwerkActivator.getRahmenwerkTimer().purge();
	}

	private void updateTimeDisplay() {

		Date date = new Date();
		final String zeitStr;
		if (davZeit) {
			if (RahmenwerkImpl.getInstanz().isOnline()) {
				date = new Date(RahmenwerkImpl.getInstanz().getDavVerbindung()
						.getTime());
				zeitStr = new SimpleDateFormat(formatStr).format(date);
			} else {
				zeitStr = "???";
			}
		} else {
			zeitStr = new SimpleDateFormat(formatStr).format(date);
		}
		if ((anzeigeText != null) && (!anzeigeText.isDisposed())) {
			anzeigeText.getDisplay().asyncExec(new Runnable() {
				@Override
				public void run() {
					if ((anzeigeText != null) && (!anzeigeText.isDisposed())) {
						if (anzeigeText instanceof ToolItem) {
							((ToolItem) anzeigeText).setText(zeitStr);
						} else if (anzeigeText instanceof MenuItem) {
							((MenuItem) anzeigeText).setText(zeitStr);
						} else if (anzeigeText instanceof CLabel) {
							((CLabel) anzeigeText).setText(zeitStr);
						}
					}
				}
			});
		}
	}

	@Override
	public void fill(final Menu menu, final int index) {
		if (anzeigeText == null) {
			anzeigeText = new MenuItem(menu, SWT.CASCADE, index);
			((MenuItem) anzeigeText).setText(formatStr);
			startTimer();
		}
	}

	@Override
	public void fill(final ToolBar parent, final int index) {
		if (anzeigeText == null) {
			anzeigeText = new ToolItem(parent, SWT.NONE, index);
			((ToolItem) anzeigeText).setText(formatStr);
			((ToolItem) anzeigeText).setImage(null);
			startTimer();
		}
	}

	@Override
	public void setParameter(final Map<String, String> parameters) {
		for (final Entry<String, String> parameter : parameters.entrySet()) {
			if (AktuelleZeitElement.PARAM_FORMAT.equals(parameter.getKey())) {
				formatStr = parameter.getValue();
			} else if (AktuelleZeitElement.PARAM_FARBE.equals(parameter
					.getKey())) {
				foreGround = parameter.getValue();
			} else if (AktuelleZeitElement.PARAM_HINTERGRUND.equals(parameter
					.getKey())) {
				backGround = parameter.getValue();
			} else if (AktuelleZeitElement.PARAM_DAVZEIT.equals(parameter
					.getKey())) {
				davZeit = Boolean.parseBoolean(parameter.getValue());
			}
		}
	}

	@Override
	public boolean useInMenue() {
		return true;
	}

	@Override
	public boolean useInStatusLine() {
		return true;
	}

	@Override
	public boolean useInToolbar() {
		return true;
	}
}
