/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 **************************************************************************************/

package de.bsvrz.buv.rw.rw.ui.internal;

import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.NotEnabledException;
import org.eclipse.core.commands.NotHandledException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.services.IServiceLocator;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Dialog, der beim Abbruch der Datenverteilerverbindung angezeigt wird und eine
 * potentielle Neuverbindung ermglicht.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id$
 */
public class ReconnectDialog extends Dialog {

	private static final int RECONNECT_ID = 1000;
	private static final String RECONNECT_LABEL = "Neu anmelden";

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Standardkonstruktor.
	 *
	 * @param shell
	 *            die bergeordnete Shell
	 */
	public ReconnectDialog(final Shell shell) {
		super(shell);
	}

	@Override
	protected Control createDialogArea(final Composite parent) {

		getShell().setText("Warnung");

		final Composite composite = (Composite) super.createDialogArea(parent);

		final Label label = new Label(composite, SWT.NONE);
		label.setText("Die Datenverteilerverbindung wurde unterbrochen,\neinige Plug-ins arbeiten mglicherweise nicht mehr korrekt!");
		GridDataFactory.fillDefaults().grab(true, true).applyTo(label);

		return composite;
	}

	@Override
	protected void createButtonsForButtonBar(final Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, ReconnectDialog.RECONNECT_ID,
				ReconnectDialog.RECONNECT_LABEL, false);
	}

	@Override
	protected void buttonPressed(final int buttonId) {

		if (buttonId == ReconnectDialog.RECONNECT_ID) {
			executeReconnect();
			okPressed();
		} else {
			super.buttonPressed(buttonId);
		}
	}

	private void executeReconnect() {

		final IServiceLocator serviceLocator = PlatformUI.getWorkbench();
		final ICommandService commandService = (ICommandService) serviceLocator
				.getService(ICommandService.class);

		try {
			final Command command = commandService
					.getCommand("de.bsvrz.buv.rw.rw.aktionAnmelden");
			command.executeWithChecks(new ExecutionEvent());
		} catch (final ExecutionException e) {
			e.printStackTrace();
		} catch (final NotDefinedException e) {
			ReconnectDialog.LOGGER.warning(e.getLocalizedMessage());
		} catch (final NotEnabledException e) {
			ReconnectDialog.LOGGER.warning(e.getLocalizedMessage());
		} catch (final NotHandledException e) {
			ReconnectDialog.LOGGER.warning(e.getLocalizedMessage());
		}
	}
}
