/*
 * Copyright (C) 2004 by Friederich Kupzog Elektronik & Software
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 
 Author: Friederich Kupzog  
 fkmk@kupzog.de
 www.kupzog.de/fkmk
 */
package de.kupzog.examples;

import java.util.HashMap;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import de.kupzog.ktable.KTableCellEditor;
import de.kupzog.ktable.KTableCellRenderer;
import de.kupzog.ktable.KTableDefaultModel;
import de.kupzog.ktable.SWTX;
import de.kupzog.ktable.editors.KTableCellEditorCheckbox2;
import de.kupzog.ktable.renderers.CheckableCellRenderer;
import de.kupzog.ktable.renderers.FixedCellRenderer;

/**
 * @author Friederich Kupzog
 */
public class BooleanModelExample extends KTableDefaultModel {

	private final HashMap content = new HashMap();

	private final KTableCellRenderer m_CheckableRenderer = new CheckableCellRenderer(
			CheckableCellRenderer.INDICATION_CLICKED
					| CheckableCellRenderer.INDICATION_FOCUS);

	private final KTableCellRenderer m_FixedRenderer = new FixedCellRenderer(
			FixedCellRenderer.STYLE_FLAT | FixedCellRenderer.INDICATION_FOCUS);

	/**
	 * Initialize the underlying model
	 */
	public BooleanModelExample() {
		// before initializing, you probably have to set some member values
		// to make all model getter methods work properly.
		initialize();
	}

	@Override
	public Object doGetContentAt(final int col, final int row) {
		// System.out.println("col "+col+" row "+row);
		Boolean val = (Boolean) content.get(col + "/" + row);
		if (val != null) {
			return val;
		}

		if (((col + row) % 2) == 1) {
			val = new Boolean(true);
		} else {
			val = new Boolean(false);
		}
		content.put(col + "/" + row, val);
		return val;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public KTableCellEditor doGetCellEditor(final int col, final int row) {
		// make only the image active for the checkbox editor
		final Rectangle imgBounds = CheckableCellRenderer.IMAGE_CHECKED
				.getBounds();
		final Point sensible = new Point(imgBounds.width, imgBounds.height);
		return new KTableCellEditorCheckbox2(sensible,
				SWTX.ALIGN_HORIZONTAL_CENTER, SWTX.ALIGN_VERTICAL_CENTER);
	}

	// Rendering
	@Override
	public KTableCellRenderer doGetCellRenderer(final int col, final int row) {
		if (isFixedCell(col, row)) {
			return m_FixedRenderer;
		} else {
			return m_CheckableRenderer;
		}
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public void doSetContentAt(final int col, final int row, final Object value) {
		System.out.println("Set val at: " + col + "/" + row + ": " + value);
		content.put(col + "/" + row, value);
	}

	// Umfang

	@Override
	public int doGetRowCount() {
		return 1000000 + getFixedRowCount();
	}

	@Override
	public int doGetColumnCount() {
		return 1000000 + getFixedRowCount();
	}

	@Override
	public int getFixedHeaderRowCount() {
		return 1;
	}

	@Override
	public int getFixedHeaderColumnCount() {
		return 1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#getFixedSelectableRowCount()
	 */
	@Override
	public int getFixedSelectableRowCount() {
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#getFixedSelectableColumnCount()
	 */
	@Override
	public int getFixedSelectableColumnCount() {
		return 0;
	}

	@Override
	public boolean isColumnResizable(final int col) {
		return true;
	}

	@Override
	public int getInitialFirstRowHeight() {
		return 22;
	}

	@Override
	public boolean isRowResizable(final int row) {
		return true;
	}

	@Override
	public int getRowHeightMinimum() {
		return 18;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableDefaultModel#getInitialColumnWidth(int)
	 */
	@Override
	public int getInitialColumnWidth(final int column) {
		return 35;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableDefaultModel#getInitialRowHeight(int)
	 */
	@Override
	public int getInitialRowHeight(final int row) {
		return 18;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableDefaultModel#doGetTooltipAt(int, int)
	 */
	@Override
	public String doGetTooltipAt(final int col, final int row) {
		return "This cell (" + col + "/" + row + ") has the value '"
				+ getContentAt(col, row) + "'!";
	}
}
