/*
 * Copyright (C) 2004 by Friederich Kupzog Elektronik & Software
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 */
package de.kupzog.ktable.renderers;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

import de.kupzog.ktable.KTableModel;

/**
 * 
 * @author Lorenz Maierhofer <lorenz.maierhofer@logicmindguide.com>
 */
public class BarDiagramCellRenderer extends DefaultCellRenderer {

	/**
	 * @param style
	 *            The style bits to use. Currently supported are:<br>
	 *            - INDICATION_FOCUS<br>
	 *            - INDICATION_FOCUS_ROW<br>
	 *            - INDICATION_GRADIENT
	 */
	public BarDiagramCellRenderer(final int style) {
		super(style);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.kupzog.ktable.KTableCellRenderer#getOptimalWidth(org.eclipse.swt.graphics
	 * .GC, int, int, java.lang.Object, boolean)
	 */
	@Override
	public int getOptimalWidth(final GC gc, final int col, final int row,
			final Object content, final boolean fixed, final KTableModel model) {
		return 20;
	}

	/**
	 * @param content
	 *            The content is expected to be a Float value between 0 and 1
	 *            that represents the fraction of the cell width that should be
	 *            used for the bar.
	 * @see de.kupzog.ktable.KTableCellRenderer#drawCell(GC, Rectangle, int,
	 *      int, Object, boolean, boolean, boolean, KTableModel)
	 */
	@Override
	public void drawCell(final GC gc, Rectangle rect, final int col,
			final int row, final Object content, final boolean focus,
			final boolean fixed, final boolean clicked, final KTableModel model) {

		if (focus && ((m_Style & DefaultCellRenderer.INDICATION_FOCUS) != 0)) {
			rect = drawDefaultSolidCellLine(gc, rect,
					DefaultCellRenderer.COLOR_LINE_LIGHTGRAY,
					DefaultCellRenderer.COLOR_LINE_LIGHTGRAY);
			drawBar(gc, rect, content, DefaultCellRenderer.COLOR_BGFOCUS,
					getForeground());
			gc.drawFocus(rect.x, rect.y, rect.width, rect.height);

		} else if (focus
				&& ((m_Style & DefaultCellRenderer.INDICATION_FOCUS_ROW) != 0)) {
			rect = drawDefaultSolidCellLine(gc, rect,
					DefaultCellRenderer.COLOR_BGROWFOCUS,
					DefaultCellRenderer.COLOR_BGROWFOCUS);
			final Color defaultBg = COLOR_BACKGROUND;
			setDefaultBackground(DefaultCellRenderer.COLOR_BGROWFOCUS);
			drawBar(gc, rect, content, getBackground(), getForeground());
			setDefaultBackground(defaultBg);

		} else {
			rect = drawDefaultSolidCellLine(gc, rect,
					DefaultCellRenderer.COLOR_LINE_LIGHTGRAY,
					DefaultCellRenderer.COLOR_LINE_LIGHTGRAY);
			drawBar(gc, rect, content, getBackground(), getForeground());
		}
	}

	/**
	 * @param gc
	 * @param rect
	 * @param m_fraction
	 * @param background
	 */
	protected void drawGradientBar(final GC gc, final Rectangle rect,
			final float m_fraction, final Color background,
			final Color foreground) {
		final int barWidth = Math.round(rect.width * m_fraction);
		gc.setForeground(background);
		gc.setBackground(foreground);
		gc.fillGradientRectangle(rect.x, rect.y, barWidth, rect.height, false);
		gc.setBackground(COLOR_BACKGROUND);
		gc.fillRectangle(rect.x + barWidth, rect.y, rect.width - barWidth,
				rect.height);
	}

	/**
	 * @param gc
	 * @param rect
	 * @param background
	 * @param foreground
	 * @param m_fraction
	 */
	protected void drawNormalBar(final GC gc, final Rectangle rect,
			final Color background, final Color foreground,
			final float m_fraction) {
		final int barWidth = Math.round(rect.width * m_fraction);
		gc.setBackground(foreground);
		gc.fillRectangle(rect.x, rect.y, barWidth, rect.height);
		gc.setBackground(background);
		gc.fillRectangle(rect.x + barWidth, rect.y, rect.width - barWidth,
				rect.height);
	}

	/**
	 * @param gc
	 * @param rect
	 * @param m_fraction
	 * @param background
	 */
	protected void drawBar(final GC gc, final Rectangle rect,
			final Object content, final Color background, final Color foreground) {
		float m_fraction;
		if (content instanceof Float) {
			m_fraction = ((Float) content).floatValue();
		} else if (content instanceof Double) {
			m_fraction = ((Double) content).floatValue();
		} else if (content instanceof IPercentage) {
			m_fraction = ((IPercentage) content).getPercentage();
		} else {
			m_fraction = 0.0f;
			// Sans 20091124
		}

		if (m_fraction > 1.0f) {
			m_fraction = 1.0f;
		}
		if (m_fraction < 0.0f) {
			m_fraction = 0.0f;
		}

		if ((m_Style & DefaultCellRenderer.INDICATION_GRADIENT) != 0) {
			drawGradientBar(gc, rect, m_fraction, background, foreground);
		} else {
			drawNormalBar(gc, rect, background, foreground, m_fraction);
		}
	}
}
